package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.db.RtcBattleRoomInDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomRankItem;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 备注
 *
 * @author Lay
 * @date 2021/5/11
 */
@Mapper
@Repository
public interface RtcBattleRoomInDAO {
    /**
     * 创建记录
     *
     * @param rtcBattleRoomInDO
     * @return
     */
    int insert(RtcBattleRoomInDO rtcBattleRoomInDO);

    /**
     * 更新完成状态
     *
     * @param battleId
     * @param battleStatus
     * @param finishTime
     * @param whereStatus
     * @return
     */
    int updateFinished(@Param("battleId") Long battleId, @Param("battleStatus") Integer battleStatus, @Param("finishTime") Long finishTime, @Param("whereStatus") Integer whereStatus);

    /**
     * 更新 PK 结果
     *
     * @param battleId
     * @param battleResult
     * @param leftCampScore
     * @param rightCampScore
     * @return
     */
    int updateResult(@Param("battleId") Long battleId, @Param("battleResult") int battleResult, @Param("leftCampScore") long leftCampScore, @Param("rightCampScore") long rightCampScore, @Param("leftCampFlow") long leftCampFlow, @Param("rightCampFlow") long rightCampFlow);

    /**
     * 根据主键查询
     *
     * @param battleId
     * @return
     */
    @Select("SELECT battleId, roomId, battleStatus, topic, duration, createTime FROM tb_rtc_battle_room_in WHERE battleId = #{battleId}")
    RtcBattleRoomInDO selectById(Long battleId);

    /**
     * 查询列表
     *
     * @param roomId
     * @param pageCursor
     * @param pageSize
     * @return
     */
    @Select("SELECT battleId, roomId, battleStatus, topic, duration, createTime, finishTime, battleResult, leftCampScore, rightCampScore FROM tb_rtc_battle_room_in WHERE roomId = #{roomId} ORDER BY battleId DESC LIMIT #{pageCursor}, #{pageSize}")
    List<RtcBattleRoomInDO> selectAll(@Param("roomId") Long roomId, @Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 根据房间标识统计
     *
     * @param roomId
     * @return
     */
    @Select("SELECT COUNT(0) FROM tb_rtc_battle_room_in WHERE roomId = #{roomId}")
    int countByRoomId(@Param("roomId") Long roomId);

    /**
     * 获取pk擂台榜
     *
     * @param startTime
     * @param endTime
     */
    List<RtcRoomRankItem> selectRtcBattleRank(@Param("startTime") Long startTime, @Param("endTime") Long endTime);
}
