package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.RtcRoomAuctionDO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 拍卖房记录 Mapper
 */
@Mapper
@Repository
public interface RtcRoomAuctionDAO {

    /**
     * 插入数据
     */
    int insert(RtcRoomAuctionDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<RtcRoomAuctionDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(RtcRoomAuctionDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    RtcRoomAuctionDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    RtcRoomAuctionDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<RtcRoomAuctionDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("rtcRoomAuctionId")
    Map<Long, RtcRoomAuctionDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 查询竞拍中的记录数
     *
     * @param roomId
     * @param biddingStatus 竞拍中状态
     * @return
     */
    int selectBiddingCount(@Param("roomId") Long roomId, @Param("biddingStatus") int biddingStatus);

}