package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.db.RtcRoomAuctionDO;
import com.starbuds.server.common.pojo.db.RtcRoomAuctionHisDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 拍卖房记录流水 Mapper
 */
@Mapper
@Repository
public interface RtcRoomAuctionHisDAO {

    /**
     * 插入数据
     */
    int insert(RtcRoomAuctionHisDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<RtcRoomAuctionHisDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(RtcRoomAuctionHisDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    RtcRoomAuctionHisDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    RtcRoomAuctionHisDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<RtcRoomAuctionHisDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("hisId")
    Map<Long, RtcRoomAuctionHisDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 查询中标用户
     *
     * @param rtcRoomAuctionId
     * @return
     */
    RtcRoomAuctionHisDO selectWinner(@Param("rtcRoomAuctionId") Long rtcRoomAuctionId);

    /**
     * 查询本次竞拍人数
     *
     * @param rtcRoomAuctionId
     * @return
     */
    int selectBidUserCount(@Param("rtcRoomAuctionId") Long rtcRoomAuctionId);

    /**
     * 查询竞拍流水总和
     *
     * @param rtcRoomAuctionId
     * @return
     */
    Long selectTotalPrice(@Param("rtcRoomAuctionId") Long rtcRoomAuctionId);

}