package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.db.RtcRoomDO;
import com.starbuds.server.common.pojo.rtc.*;
import com.starbuds.server.common.pojo.rtc.vo.RtcBattleRoomCross2VO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/7/6
 */
@Mapper
@Repository
public interface RtcRoomDAO {
    /**
     * 创建房间
     *
     * @param rtcRoomDO
     * @return
     */
    int insert(RtcRoomDO rtcRoomDO);

    /**
     * 根据房间 id 统计
     *
     * @param roomId
     * @return
     */
    int countById(@Param("roomId") Long roomId);

    /**
     * 房间是否开播
     *
     * @param roomId
     * @return
     */
    int isRoomOnline(@Param("roomId") Long roomId);

    /**
     * 房间是否开播
     *
     * @param roomId
     * @return
     */
    int isSingleRtcRoom(@Param("roomId") Long roomId);

    /**
     * 根据用户 id 统计
     *
     * @param userId
     * @return
     */
    int countByUserId(@Param("userId") Long userId, @Param("businessType") Integer businessType);

    /**
     * 获取指定用户指定类型的房间数量
     *
     * @param userId
     * @param roomType
     * @return
     */
    int countByUserIdAndRoomType(@Param("userId") Long userId, @Param("roomType") Integer roomType);

    /**
     * 查询房间密码
     *
     * @param roomId
     * @return
     */
    String selectRoomLockPass(@Param("roomId") Long roomId);

    /**
     * 设置房间密码
     *
     * @param roomId
     * @param roomLockPass
     * @return
     */
    int updateRoomLockPass(@Param("roomId") Long roomId, @Param("roomLockPass") String roomLockPass);

    /**
     * 根据 id 查询房间信息
     *
     * @param roomId
     * @return
     */
    RtcRoom selectById(@Param("roomId") Long roomId);

    /**
     * 根据userid，familyid查询房间信息
     *
     * @return
     */
    RtcRoomDO selectSingleRtcRoom(@Param("userId") Long userId, @Param("familyId") Long familyId);

    /**
     * 查询基本信息
     *
     * @param roomId
     * @return
     */
    RtcRoomDO selectRoomById(@Param("roomId") Long roomId);

    /**
     * 查询摘要信息
     *
     * @param roomId
     * @return
     */
    RtcRoomDO selectRoomSummaryById(@Param("roomId") Long roomId);

    /**
     * 语音房摘要信息Map
     *
     * @param roomIds
     * @return
     */
    @MapKey("roomId")
    Map<Long, RtcRoomSummary> selectRtcRoomSummaryMap(@Param("roomIds") Collection<Long> roomIds);

    /**
     * 获取roomMap
     *
     * @param roomIds
     * @return
     */
    @MapKey("roomId")
    Map<Long, RtcRoom> selectRoomMap(@Param("roomIds") Set<Long> roomIds);

    /**
     * 更新房间信息
     *
     * @param rtcRoomEditDTO
     * @return
     */
    int update(RtcRoomEditDTO rtcRoomEditDTO);

    /**
     * 查询房主 id
     *
     * @param roomId
     * @return
     */
    Long selectUserId(@Param("roomId") Long roomId);

    /**
     * 查询是否自由上麦
     *
     * @param roomId
     * @return
     */
    Integer selectRoomMicFree(@Param("roomId") Long roomId);

    /**
     * 查询心动值开关
     *
     * @param roomId
     * @return
     */
    Integer selectRoomScoreEnabled(@Param("roomId") Long roomId);

    /**
     * 查询公屏开关
     *
     * @param roomId
     * @return
     */
    Integer selectRoomChatEnabled(@Param("roomId") Long roomId);

    /**
     * 获得制定类型的房间类别
     *
     * @param pageCursor
     * @param pageSize
     * @param roomType
     * @return
     */
    List<RtcRoomListItem> selectRoomListByType(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("roomType") long roomType);

    /**
     * 查询列表
     *
     * @param pageCursor
     * @param pageSize
     * @param categoryId
     * @return
     */
    List<RtcRoomListItem> selectListItems(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize, @Param("categoryId") Long categoryId, @Param("businessType") Integer businessType);

    /**
     * 根据 id 查询
     *
     * @param roomIds
     * @return
     */
    List<RtcRoomListItem> selectByIds(List<Long> roomIds);

    /**
     * 统计
     *
     * @param categoryId
     * @return
     */
    int count(@Param("categoryId") Long categoryId);

    /**
     * 更新自由上麦
     *
     * @param roomId
     * @param micFree
     * @return
     */
    int updateMicFree(@Param("roomId") Long roomId, @Param("micFree") int micFree);

    /**
     * 更新心动值开关
     *
     * @param roomId
     * @param scoreEnabled
     * @return
     */
    int updateScoreEnabled(@Param("roomId") Long roomId, @Param("scoreEnabled") int scoreEnabled);

    /**
     * 更新公屏开关
     *
     * @param roomId
     * @param chatEnabled
     * @return
     */
    int updateChatEnabled(@Param("roomId") Long roomId, @Param("chatEnabled") int chatEnabled);

    /**
     * 更新大冒险开关
     *
     * @param roomId
     * @param bigAdventureEnabled
     * @return
     */
    int updateBigAdventureEnabled(@Param("roomId") Long roomId, @Param("bigAdventureEnabled") int bigAdventureEnabled);

    /**
     * 更新房间封面
     *
     * @param roomId
     * @param roomCover
     * @return
     */
    int updateRoomCover(@Param("roomId") Long roomId, @Param("roomCover") String roomCover);

    /**
     * 查询房间类型
     *
     * @param roomId
     * @return
     */
    Integer selectRoomType(@Param("roomId") Long roomId);

    /**
     * 查询房间业务类型
     *
     * @param roomId
     * @return
     */
    Integer selectRoomBusinessType(@Param("roomId") Long roomId);

    /**
     * 只保留允许热门的 id
     *
     * @param roomIds
     * @return
     */
    List<RtcRoomListItem> selectHotEnableRooms(List<Long> roomIds);

    /**
     * 查询热门房间列表
     *
     * @return
     */
    List<RtcRoomListItem> selectHotList(@Param("num") int num, @Param("businessType") Integer businessType);

    /**
     * 房间获取
     *
     * @param roomId
     * @return
     */
    RtcRoomListItem getRoomById(@Param("roomId") Long roomId);

    /**
     * 推荐房间列表需要的房间信息
     *
     * @param roomIds
     * @param noPass  true:不包含上锁的房间
     * @return
     */
    List<RtcRoomSuggestItem> selectSuggestList(@Param("roomIds") Collection roomIds,
                                               @Param("noPass") boolean noPass,
                                               @Param("status") Integer status);

    /**
     * 查询房号
     *
     * @param roomId
     * @return
     */
    Long selectRoomNo(@Param("roomId") Long roomId);

    /**
     * 查询房间名称
     *
     * @param roomId
     * @return
     */
    String selectRoomName(@Param("roomId") Long roomId);

    /**
     * 获取交友房间列表
     *
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<RtcRoomListItem> selectMakeFriendRoomList(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 获取交友房间列表大小
     *
     * @return
     */
    int countMakeFriendRoomList();

    /**
     * 获取家族id
     *
     * @param roomId
     * @return
     */
    Long selectFamilyId(@Param("roomId") Long roomId);

    /**
     * 获取家族房间数量
     *
     * @param familyId
     * @return
     */
    int countByFamilyId(@Param("familyId") Long familyId, @Param("businessType") Integer businessType);

    /**
     * 获取家族房间id
     *
     * @param familyId
     * @return
     */
    List<Long> selectRoomIds(@Param("familyId") Long familyId);

    /**
     * 获取用户分成比例
     *
     * @param roomId
     * @return
     */
    Double selectUserRatio(@Param("roomId") Long roomId);

    /**
     * 更新转盘
     *
     * @param roomId
     * @param turntableConfig
     * @return
     */
    int updateTurntableConfig(@Param("roomId") Long roomId, @Param("turntableConfig") String turntableConfig);

    /**
     * 获取转盘配置
     *
     * @param roomId
     * @return
     */
    String selectTurntableConfig(@Param("roomId") Long roomId);

    /**
     * 获取大冒险状态
     *
     * @param roomId
     * @return
     */
    Integer selectBigAdventureEnabled(@Param("roomId") Long roomId);

    /**
     * 查询房间类目标识
     *
     * @param roomId
     * @return
     */
    Long selectCategoryId(Long roomId);

    /**
     * 批量查询房间热度值
     *
     * @param roomIds
     * @return
     */
    List<RtcRoomDO> selectRoomHeats(List<Long> roomIds);

    /**
     * 批量查询跨房PK开关
     *
     * @param roomIds
     * @return
     */
    List<RtcRoomDO> selectBattleMatchEnabled(List<Long> roomIds);

    /**
     * 更新跨房PK开关
     *
     * @param roomId
     * @param enabled
     * @return
     */
    int updateBattleMatchEnabled(@Param("roomId") Long roomId, @Param("enabled") int enabled);

    /**
     * 获取最火的竞拍房ID
     */
    Long getHotAuctionRoomId();

    /**
     * 获取 跨房PK 房间列表
     */
    List<RtcBattleRoomCross2VO> selectBattleMatchEnabledRoomList(@Param("roomId") Long roomId, @Param("keyword") String keyword, @Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 获取 跨房PK 房间列表
     */
    Integer countBattleRoomCross(@Param("keyword") String keyword);

    /**
     * 获取所有房间id
     *
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<Long> selectAllRoomIds(@Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 房间总数
     *
     * @return
     */
    int countAllRoomIds();

    /**
     * 获得房间列表的房间拥有者
     *
     * @param roomIds
     * @return
     */
    @MapKey("roomId")
    Map<Long, RtcRoomDO> selectRoomOwnerMap(@Param("roomIds") List<Long> roomIds);
}
