package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.db.RtcRoomLiveHisDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 个播开播记录 Mapper
 */
@Mapper
@Repository
public interface RtcRoomLiveHisDAO {

    /**
     * 插入数据
     */
    int insert(RtcRoomLiveHisDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<RtcRoomLiveHisDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(RtcRoomLiveHisDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    RtcRoomLiveHisDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    List<RtcRoomLiveHisDO> selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<RtcRoomLiveHisDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("hisId")
    Map<Long, RtcRoomLiveHisDO> selectMapByIds(@Param("ids") Collection ids);


    /**
     * 根据房间id查询正在进行的个播场次
     *
     * @param roomId
     * @return
     */
    RtcRoomLiveHisDO selectCurrentRoundByRoomId(@Param("roomId") Long roomId);

    /**
     * 根据房间id查询正在进行的个播场次
     *
     * @param roomId
     * @return
     */
    RtcRoomLiveHisDO selectLastRoundByRoomId(@Param("roomId") Long roomId);

    /**
     * 将正在进行的个播场次结束
     *
     * @param roomId
     * @return
     */
    boolean setRoundEnd(@Param("roomId") Long roomId);

    /**
     * 获取今日开播总时长，毫秒数
     *
     * @param roomId
     * @return
     */
    Long getTodayTotalTimeMillis(@Param("roomId") Long roomId);


    /**
     * 获取今日开播总场次
     *
     * @param roomId
     * @return
     */
    Integer getTodayRoundNum(@Param("roomId") Long roomId);

}