package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.db.RtcRoomRoleDO;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.pojo.rtc.RtcRoomUser;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/7/6
 */
@Mapper
@Repository
public interface RtcRoomRoleDAO {
    /**
     * 创建或更新
     *
     * @param rtcRoomRoleDO
     * @return
     */
    int insertOrUpdate(RtcRoomRoleDO rtcRoomRoleDO);

    /**
     * 查询房间
     *
     * @param userId
     * @return
     */
    List<RtcRoomListItem> selectRooms(@Param("userId") Long userId);

    /**
     * 查询角色
     *
     * @param roomId
     * @param userId
     * @return
     */
    Integer selectRole(@Param("roomId") Long roomId, @Param("userId") Long userId);

    /**
     * 查询房间管理者
     *
     * @param roomId
     * @param roomRole
     * @return
     */
    List<RtcRoomUser> selectAllUsers(@Param("roomId") Long roomId, @Param("roomRole") Integer roomRole);

    /**
     * 删除角色
     *
     * @param roomId
     * @param userId
     * @return
     */
    int delete(@Param("roomId") Long roomId, @Param("userId") Long userId);

    /**
     * 根据用户 id 查询用户列表
     *
     * @param roomId
     * @param userIds
     * @return
     */
    List<RtcRoomUser> selectUsersByIdList(@Param("roomId") Long roomId, @Param("userIds") List<Long> userIds);

    /**
     * 查询用户
     *
     * @param roomId
     * @param userId
     * @return
     */
    RtcRoomUser selectUserById(@Param("roomId") Long roomId, @Param("userId") Long userId);

    /**
     * 根据房间id和角色查询
     *
     * @param roomId
     * @param roomRole
     * @return
     */
    int countByIdAndRole(@Param("roomId") Long roomId, @Param("roomRole") int roomRole);

    /**
     * 判断是否至少某个角色之上
     *
     * @param roomId
     * @param userId
     * @param roomRole
     * @return
     */
    int countByIdAndUserAndRole(@Param("roomId") Long roomId, @Param("userId") Long userId, @Param("roomRole") int roomRole);

    /**
     * 获取主持人名
     *
     * @param roomId
     * @return
     */
    String selectHostUserName(@Param("roomId") Long roomId);

    /**
     * 获取主持人名
     *
     * @param roomIds
     * @return
     */
    @MapKey("roomId")
    Map<Long, RtcRoom> selectHostUserNameMap(@Param("roomIds") Set<Long> roomIds);

    /**
     * 判断用户是否担任角色
     *
     * @param userId
     * @param roomIds
     * @return
     */
    int countByUserId(@Param("userId") Long userId, @Param("roomIds") List<Long> roomIds);
}
