package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.RtcSkillOrderDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 派单订单表 Mapper
 */
@Mapper
@Repository
public interface RtcSkillOrderDAO {

    /**
     * 插入数据
     */
    int insert(RtcSkillOrderDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<RtcSkillOrderDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(RtcSkillOrderDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    RtcSkillOrderDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    List<RtcSkillOrderDO> selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<RtcSkillOrderDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("orderId")
    Map<Long, RtcSkillOrderDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 查询列表(分页)
     */
    List<RtcSkillOrderDO> selectPageListAsc(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<RtcSkillOrderDO> selectUserPageList(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    int selectUserPageListCount(QueryWrapper qw);

    /**
     * 查询大厅内订单数量
     *
     * @return
     */
    int selectOrderCreatedCount(@Param("userId") Long userId);

    /**
     * 查询大厅内有关某用户的订单
     *
     * @return
     */
    List<RtcSkillOrderDO> selectOrderCreatedListByUserId(@Param("userId") Long userId);

    /**
     * 查询大厅最近创建的订单
     *
     * @return
     */
    RtcSkillOrderDO selectLastOrderCreated();

    /**
     * 查询大厅内订单数量
     *
     * @return
     */
    int selectOrderCreatedCountBySkill(@Param("skillId") Long skillId);

    /**
     * 查询房间内订单数量
     *
     * @param roomId
     * @return
     */
    int selectOrderInRoomCount(@Param("roomId") Long roomId);

    /**
     * 查询待接单的订单+已完成的订单 总共100条
     *
     * @return
     */
    Long selectListInStatusLimit();

    /**
     * 查询待接单的订单+已完成的订单 总共100条
     *
     * @return
     */
    List<RtcSkillOrderDO> selectListInStatus(QueryWrapper qw);

    /**
     * 查询待接单的订单+已完成的订单的数量
     *
     * @return
     */
    Integer selectListInStatusCount(QueryWrapper qw);
}