package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.db.StatRtcRoomDO;
import com.starbuds.server.common.pojo.rtc.RtcRankItem;
import com.starbuds.server.common.pojo.rtc.StatRtcRoomVO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/8/5 14:37
 * @Description 房间统计持久层
 */
@Mapper
@Repository
public interface StatRtcRoomDAO {

    /**
     * 获取房间统计
     *
     * @param roomId
     * @param startTime
     * @param endTime
     * @param statTimeType
     * @param statType
     * @return
     */
    List<StatRtcRoomDO> selectRoomStat(@Param("roomId") Long roomId,
                                       @Param("startTime") Long startTime,
                                       @Param("endTime") Long endTime,
                                       @Param("statTimeType") int statTimeType,
                                       @Param("statType") int statType);

    /**
     * 获取房间统计
     *
     * @param roomId
     * @param startTime
     * @param endTime
     * @param statTimeType
     * @param statTypes
     * @param isFamily
     * @param familyId
     * @param liveType     1:个播 0：普通房
     * @return
     */
    List<StatRtcRoomVO> selectRoomStats(@Param("roomId") Long roomId,
                                        @Param("startTime") Long startTime,
                                        @Param("endTime") Long endTime,
                                        @Param("statTimeType") int statTimeType,
                                        @Param("statTypes") List<Integer> statTypes,
                                        @Param("isFamily") Integer isFamily,
                                        @Param("familyId") Long familyId,
                                        @Param("liveType") Integer liveType);

    /**
     * 获取房间统计
     *
     * @param roomId
     * @param date
     * @param statTimeType
     * @param statTypes
     * @param isFamily
     * @param familyId
     * @return
     */
    List<StatRtcRoomVO> selectRoomStatsForHour(@Param("roomId") Long roomId,
                                               @Param("date") Long date,
                                               @Param("statTimeType") int statTimeType,
                                               @Param("statTypes") List<Integer> statTypes,
                                               @Param("isFamily") Integer isFamily,
                                               @Param("familyId") Long familyId);

    /**
     * 获取房间统计
     *
     * @param roomId
     * @param startTime
     * @param endTime
     * @param statTimeType
     * @param statType
     * @return
     */
    List<StatRtcRoomDO> selectRoomStatByFamilyId(@Param("roomId") Long roomId,
                                                 @Param("startTime") Long startTime,
                                                 @Param("endTime") Long endTime,
                                                 @Param("statTimeType") int statTimeType,
                                                 @Param("statType") int statType,
                                                 @Param("familyId") Long familyId);

    /**
     * 获取小时榜单
     *
     * @param statTime
     * @return
     */
    List<RtcRankItem> selectRtcHourRank(@Param("statTime") Long statTime);

    /**
     * 获取小时榜单
     *
     * @param statTime
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<RtcRankItem> selectRtcHourRankPageList(@Param("statTime") Long statTime, @Param("pageCursor") int pageCursor, @Param("pageSize") int pageSize);

    /**
     * 获取小时榜榜首
     *
     * @param statTime
     * @return
     */
    RtcRankItem getRtcHourRankWinner(@Param("statTime") Long statTime);

    /**
     * 条数
     *
     * @param statTime
     * @return
     */
    int countRtcHourRankPageList(@Param("statTime") Long statTime);
}
