package com.starbuds.server.service.rtc.dao;

import com.starbuds.server.common.pojo.db.WishUserTaskDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 许愿用户任务表，用户打开许愿任务时创建当天记录 Mapper
 */
@Mapper
@Repository
public interface WishUserTaskDAO {

    /**
     * 插入数据
     */
    int insert(WishUserTaskDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<WishUserTaskDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(WishUserTaskDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    WishUserTaskDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    List<WishUserTaskDO> selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<WishUserTaskDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("wishTaskId")
    Map<Long, WishUserTaskDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 查询用户当天许愿任务
     */
    List<WishUserTaskDO> selectUserCurrentDayList(@Param("userId") Long userId, @Param("dayStart") Long dayStart);

    /**
     * 更新许愿任务状态
     */
    boolean updateWishTaskStatus(@Param("wishTaskId") Long wishTaskId, @Param("userId") Long userId, @Param("status") Integer status);

    /**
     * 更新许愿任务开启状态
     */
    boolean updateWishTaskOpenStatus(@Param("wishTaskId") Long wishTaskId, @Param("userId") Long userId, @Param("openTime") Long openTime);

    /**
     * 更新许愿任务完成
     */
    boolean updateWishTaskComplete(@Param("wishTaskId") Long wishTaskId, @Param("userId") Long userId, @Param("now") Long now);

    /**
     * 更新许愿任务进度
     */
    boolean updateWishTask(@Param("wishTaskId") Long wishTaskId, @Param("userId") Long userId, @Param("sendGiftNum") Integer sendGiftNum);

    /**
     * 批量获取用户开启的许愿任务
     *
     * @param userIds id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("userId")
    Map<Long, WishUserTaskDO> selectOpenedMapByIds(@Param("userIds") Collection userIds);

    /**
     * 将已经绑定的用户任务删除
     */
    boolean deleteTaskByWishId(@Param("wishId") Long wishId);
}