package com.starbuds.server.service.rtc.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.media.BattleStatusEnum;
import com.starbuds.server.common.constant.message.im.ImMsgType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.rtc.RtcRoomBattleCrossCmdType;
import com.starbuds.server.common.constant.rtc.RtcRoomBattleCrossStatus;
import com.starbuds.server.common.constant.rtc.RtcRoomBattleType;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.rtc.RtcBattleRoomCrossCmdMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcBattleRoomCrossEnableListenMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcBattleRoomCrossStatusMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcBattleRoomInStatusMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcBattleRingTimeoutMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcRoomBattleTimeUpMsg;
import com.starbuds.server.common.pojo.rtc.RtcBattleRoomCrossVO;
import com.starbuds.server.common.pojo.rtc.RtcBattleStatus;
import com.starbuds.server.common.pojo.rtc.vo.RtcBattleRoomCross2VO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcBattleService;
import com.starbuds.server.service.rtc.dao.*;
import com.starbuds.server.service.rtc.provider.RtcBattleProvider;
import com.starbuds.server.service.rtc.provider.RtcBattleRoomCrossProvider;
import com.starbuds.server.service.rtc.provider.RtcBattleRoomInProvider;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.common.constant.media.BattleStatusEnum.Matched;
import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;


/**
 * 语聊 PK 服务实现
 *
 * @author Lay
 * @date 2021/5/8
 */
@Service(version = "1.0.0")
@Slf4j
public class RtcBattleServiceImpl implements RtcBattleService {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    RtcBattleRoomInDAO rtcBattleRoomInDAO;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Resource
    RtcRoomCategoryDAO rtcRoomCategoryDAO;

    @Resource
    RtcBattleRoomInConfigDAO rtcBattleRoomInConfigDAO;

    @Resource
    RtcBattleRoomCrossDAO rtcBattleRoomCrossDAO;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    RtcBattleProvider rtcBattleProvider;

    @Resource
    RtcBattleRoomInProvider rtcBattleRoomInProvider;

    @Resource
    RtcBattleRoomCrossProvider rtcBattleRoomCrossProvider;

    /**
     * 开始房间内 PK
     *
     * @param roomId
     * @param topic
     * @param duration
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean startRoomInBattle(Long roomId, String topic, Long duration) {
        Long battleId = identityService.getId();
        Long curTime = System.currentTimeMillis();

        String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(roomId);
        // 设置房间 PK 状态
        RtcBattleStatus status = new RtcBattleStatus(battleId, BattleStatusEnum.Battling, RtcRoomBattleType.ROOM_IN, null);
        if (redisService.setIfAbsent(battleStatusKey, status.toString())) {
            try {
                // 创建 PK 记录
                RtcBattleRoomInDO rtcBattleRoomIn = new RtcBattleRoomInDO();
                rtcBattleRoomIn.setBattleId(battleId);
                rtcBattleRoomIn.setRoomId(roomId);
                rtcBattleRoomIn.setBattleStatus(BattleStatusEnum.Battling.getValue());
                rtcBattleRoomIn.setTopic(topic);
                rtcBattleRoomIn.setDuration(duration);
                rtcBattleRoomIn.setCreateTime(curTime);
                rtcBattleRoomInDAO.insert(rtcBattleRoomIn);

                // 发送房间消息
                RtcBattleRoomInStatusMsg msg = new RtcBattleRoomInStatusMsg();
                msg.setBattleId(battleId);
                msg.setTopic(topic);
                msg.setDuration(duration);
                msg.setBattleStatus(BattleStatusEnum.Battling.getValue());
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, msg);

                // 启动计时
                QueueRtcRoomBattleTimeUpMsg timeUpMsg = new QueueRtcRoomBattleTimeUpMsg();
                timeUpMsg.setBattleId(battleId);
                timeUpMsg.setRoomId(roomId);
                timeUpMsg.setStartTime(curTime);
                timeUpMsg.setBattleType(RtcRoomBattleType.ROOM_IN);
                rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.RTC_ROOM_BATTLE_TIME_UP, timeUpMsg.serialize(), curTime + duration);

                return true;
            } catch (Exception e) {
                redisService.delKey(battleStatusKey);
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.error("房间内 PK 启动失败:", e);
            }
        }

        return false;
    }

    /**
     * 获取房间内 PK 状态
     *
     * @param roomId
     * @return
     */
    @Override
    public int getRoomInBattleStatus(Long roomId) {
        String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(roomId);
        String status = redisService.get(battleStatusKey);
        if (status != null) {
            RtcBattleStatus battleStatus = new RtcBattleStatus(status);
            return battleStatus.getBattleStatus().getValue();
        }
        return -1;
    }

    /**
     * 获取房间pk状态
     *
     * @param roomIds
     * @return
     */
    @Override
    public Map<Long, Boolean> getRoomInBattleStatusMap(Set<Long> roomIds) {
        if (CollectionUtil.isEmpty(roomIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Boolean> statusMap = new HashMap<>();
        for (Long roomId : roomIds) {
            statusMap.put(roomId, getRoomInBattleStatus(roomId) == BattleStatusEnum.Battling.getValue());
        }
        return statusMap;
    }

    /**
     * 获取房间内 PK 标识
     *
     * @param roomId
     * @return
     */
    @Override
    public Long getRoomInBattleId(Long roomId) {
        return rtcBattleRoomInProvider.getBattleId(roomId);
    }

    /**
     * 判断房间是否启用 PK
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isRoomBattleEnabled(Long roomId) {
        if (roomId == null) {
            return false;
        }

        Long categoryId = rtcRoomDAO.selectCategoryId(roomId);
        if (categoryId != null) {
            return Optional.ofNullable(rtcRoomCategoryDAO.selectBattleEnabledById(categoryId)).orElse(BoolIntVal.False) == BoolIntVal.True;
        }
        return false;
    }

    /**
     * 结束 PK
     *
     * @param roomId
     * @param battleId
     * @return
     */
    @Override
    public boolean stopRoomInBattle(Long roomId, Long battleId) {
        return rtcBattleRoomInProvider.battleOver(roomId, battleId);
    }

    /**
     * 获取房间内PK配置
     *
     * @return
     */
    @Override
    public List<RtcBattleRoomInConfigDO> getRoomInConfigList() {
        return rtcBattleRoomInConfigDAO.selectAll();
    }

    /**
     * 获取当前房间内 PK 详情
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcBattleRoomInDO getCurrentBattleRoomIn(Long roomId) {
        Long battleId = rtcBattleRoomInProvider.getBattleId(roomId);
        if (battleId != null) {
            return rtcBattleRoomInDAO.selectById(battleId);
        }
        return null;
    }

    /**
     * 获取 PK 记录列表
     *
     * @param roomId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<RtcBattleRoomInDO> getBattleRoomInHisList(Long roomId, int pageIndex, int pageSize) {
        PageList<RtcBattleRoomInDO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(rtcBattleRoomInDAO.selectAll(roomId, pageList.getPageCursor(), pageSize));
        pageList.setTotal(rtcBattleRoomInDAO.countByRoomId(roomId));
        return pageList;
    }

    /**
     * 获取 跨房PK 房间列表
     *
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<RtcBattleRoomCross2VO> getBattleRoomCrossList(Long roomId, String keyword, Integer pageIndex, Integer pageSize) {
        PageList<RtcBattleRoomCross2VO> resultPageList = new PageList<>(pageIndex, pageSize);

        List<RtcBattleRoomCross2VO> list = rtcRoomDAO.selectBattleMatchEnabledRoomList(roomId, keyword, resultPageList.getPageCursor(), pageSize);

        list.forEach(vo -> vo.setBattleStatus(isRoomBusy(vo.getRoomId()) ? RtcRoomBattleCrossStatus.BATTLING : RtcRoomBattleCrossStatus.FREE));

        resultPageList.setList(list);
        resultPageList.setTotal(rtcRoomDAO.countBattleRoomCross(keyword));
        return resultPageList;
    }

    /**
     * 获取 跨房PK 房间状态
     *
     * @return
     */
    @Override
    public Integer getBattleRoomCrossStatus(Long roomId) {
        return isRoomBusy(roomId) ? RtcRoomBattleCrossStatus.BATTLING : RtcRoomBattleCrossStatus.FREE;
    }

    /**
     * 获取 跨房PK 房间收听状态
     *
     * @return
     */
    @Override
    public void setBattleRoomCrossListenStatus(Long roomId, Integer enableListen) {
        redisService.set(RedisRtcKeys.getRtcBattleListenStatus(roomId), String.valueOf(enableListen));

        RtcBattleRoomCrossEnableListenMsg msg = new RtcBattleRoomCrossEnableListenMsg();
        msg.setEnableListen(enableListen);
        msg.setType(ImMsgType.RTC_BATTLE_ROOM_CROSS_LISTEN_STATUS);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, msg);
    }

    /**
     * 跨房pk房间是否空闲
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isRoomBusy(Long roomId) {
        RtcBattleStatus status = getBattleStatus(roomId);
        if (status == null) {
            return false;
        }

        if (Matched.equals(status.getBattleStatus())
                || BattleStatusEnum.Battling.equals(status.getBattleStatus())
                || BattleStatusEnum.Ringing.equals(status.getBattleStatus())) {

            return true;
        }
        return false;
    }

    /**
     * 获取当前 PK 状态
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcBattleStatus getBattleStatus(Long roomId) {
        return rtcBattleProvider.getBattleStatus(roomId);
    }

    /**
     * 开启跨房 PK
     *
     * @param roomId
     * @param targetRoomId
     * @param topic
     * @param duration
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean startRoomCrossBattle(Long roomId, Long targetRoomId, String topic, long duration) {
        Long battleId = identityService.getId();
        Long curTime = System.currentTimeMillis();

        String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(roomId);
        String targetBattleStatusKey = RedisRtcKeys.getRtcBattleStatus(targetRoomId);

        // 查询房间
        RtcRoomDO room = rtcRoomDAO.selectRoomById(roomId);
        if (room == null) {
            redisService.delKey(targetBattleStatusKey);
            redisService.delKey(battleStatusKey);
            rtcBattleRoomCrossProvider.addToMatchPool(targetRoomId);
            return false;
        }

        // 查询目标房间是否存在
        RtcRoomDO targetRoom = rtcRoomDAO.selectRoomById(targetRoomId);
        if (targetRoom == null) {
            redisService.delKey(targetBattleStatusKey);
            redisService.delKey(battleStatusKey);
            rtcBattleRoomCrossProvider.addToMatchPool(targetRoomId);
            return false;
        }

        if (isDev()) {
            log.info("房间都存在:room:{}, targetRoom:{}", JSON.toJSONString(room), JSON.toJSONString(targetRoom));
        }

        // 设置房间 PK 状态
        boolean targetRoomStatusSet = redisService.setIfAbsent(targetBattleStatusKey, new RtcBattleStatus(battleId, BattleStatusEnum.Ringing, RtcRoomBattleType.ROOM_CROSS, targetRoomId).toString());
        boolean roomStatusSet = redisService.setIfAbsent(battleStatusKey, new RtcBattleStatus(battleId, BattleStatusEnum.Ringing, RtcRoomBattleType.ROOM_CROSS, roomId).toString());
        if (!targetRoomStatusSet) {
            // 判断是否匹配状态
            RtcBattleStatus battleStatus = rtcBattleProvider.getBattleStatus(targetRoomId);
            log.info("判断目标房间状态 roomId: {} == {}", roomId, battleStatus.getTargetRoomId());
            if (Matched == battleStatus.getBattleStatus() && roomId.equals(battleStatus.getTargetRoomId())) {
                battleStatus.setBattleId(battleId);
                battleStatus.setBattleStatus(BattleStatusEnum.Ringing);
                redisService.set(targetBattleStatusKey, battleStatus.toString());
                targetRoomStatusSet = true;
            } else {
                if (isDev()) {
                    log.info("[跨房]房间未匹配, battleStatusKey:{}, targetBattleStatusKey:{}", JSON.toJSONString(battleStatusKey), JSON.toJSONString(targetBattleStatusKey));
                }
                redisService.delKey(targetBattleStatusKey);
            }
        }
        if (!roomStatusSet) {
            // 判断是否匹配状态
            RtcBattleStatus battleStatus = rtcBattleProvider.getBattleStatus(roomId);
            log.info("判断目标房间状态 roomId: {} == {}", targetRoomId, battleStatus.getTargetRoomId());
            if (Matched == battleStatus.getBattleStatus() && targetRoomId.equals(battleStatus.getTargetRoomId())) {
                battleStatus.setBattleId(battleId);
                battleStatus.setBattleStatus(BattleStatusEnum.Ringing);
                redisService.set(battleStatusKey, battleStatus.toString());
                roomStatusSet = true;
            } else {
                if (isDev()) {
                    log.info("[跨房]房间未匹配, battleStatusKey:{}, targetBattleStatusKey:{}", JSON.toJSONString(battleStatusKey), JSON.toJSONString(targetBattleStatusKey));
                }
                redisService.delKey(battleStatusKey);
            }
        }
        if (targetRoomStatusSet && roomStatusSet) {
            try {
                rtcBattleRoomCrossProvider.removeFromMatchPool(Arrays.asList(roomId, targetRoomId));

                // 创建 PK 记录
                RtcBattleRoomCrossDO rtcBattleRoomCross = new RtcBattleRoomCrossDO();
                rtcBattleRoomCross.setBattleId(battleId);
                rtcBattleRoomCross.setRoomId(roomId);
                rtcBattleRoomCross.setTargetRoomId(targetRoomId);
                rtcBattleRoomCross.setBattleStatus(BattleStatusEnum.Ringing.getValue());
                rtcBattleRoomCross.setTopic(topic);
                rtcBattleRoomCross.setDuration(duration);
                rtcBattleRoomCross.setCreateTime(curTime);
                rtcBattleRoomCrossDAO.insert(rtcBattleRoomCross);

                // 呼叫超时
                QueueRtcBattleRingTimeoutMsg timeoutMsg = new QueueRtcBattleRingTimeoutMsg();
                timeoutMsg.setBattleId(battleId);
                rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.RTC_ROOM_BATTLE_RING_TIMEOUT, timeoutMsg.serialize(), System.currentTimeMillis() + TimeConstants.SECOND_30_MILLS);

                // 发送房间消息
                RtcBattleRoomCrossCmdMsg cmdMsg = new RtcBattleRoomCrossCmdMsg();
                cmdMsg.setBattleId(battleId);
                cmdMsg.setTopic(topic);
                cmdMsg.setDuration(duration);
                cmdMsg.setRoomId(roomId);
                cmdMsg.setTargetRoomId(targetRoomId);
                cmdMsg.setCmdType(RtcRoomBattleCrossCmdType.Ringing);
                cmdMsg.setRoom(room);
                RtcBattleRoomCrossCmdMsg roomCmdMsg = new RtcBattleRoomCrossCmdMsg();
                roomCmdMsg.setBattleId(battleId);
                roomCmdMsg.setTopic(topic);
                roomCmdMsg.setDuration(duration);
                roomCmdMsg.setRoomId(roomId);
                roomCmdMsg.setTargetRoomId(targetRoomId);
                roomCmdMsg.setCmdType(RtcRoomBattleCrossCmdType.Ringing);
                roomCmdMsg.setRoom(targetRoom);
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, targetRoomId, cmdMsg);
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, roomCmdMsg);

                return true;
            } catch (Exception e) {
                redisService.delKey(targetBattleStatusKey);
                redisService.delKey(battleStatusKey);
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                log.error("跨房 PK 启动失败:", e);
            }
        } else {
            if (targetRoomStatusSet) {
                redisService.delKey(targetBattleStatusKey);
            }
            if (roomStatusSet) {
                redisService.delKey(battleStatusKey);
            }
        }
        return false;
    }

    /**
     * 取消跨房 PK
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean cancelRoomCrossBattle(Long roomId) {
        RtcBattleStatus battleStatus = rtcBattleProvider.getBattleStatus(roomId);

        if (isDev()) {
            log.info("取消pk,获取pk状态battleStatus:{}", JSON.toJSONString(battleStatus));
        }

        if (battleStatus != null) {
            Long battleId = battleStatus.getBattleId();
            // 只有呼叫中才能取消
            if (BattleStatusEnum.Ringing == battleStatus.getBattleStatus()) {
                // 查询
                RtcBattleRoomCrossDO rtcBattleRoomCross = rtcBattleRoomCrossDAO.selectById(battleId);
                if (rtcBattleRoomCross == null || BattleStatusEnum.Ringing.getValue() != rtcBattleRoomCross.getBattleStatus()) {
                    return false;
                }

                // 尝试修改状态
                if (rtcBattleRoomCrossDAO.updateStatus(battleId, BattleStatusEnum.Canceled.getValue(), BattleStatusEnum.Ringing.getValue(), System.currentTimeMillis()) > 0) {
                    // 移除缓存
                    String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(roomId);
                    String targetBattleStatusKey = RedisRtcKeys.getRtcBattleStatus(rtcBattleRoomCross.getTargetRoomId());

                    redisService.delKeys(Arrays.asList(battleStatusKey, targetBattleStatusKey));

                    if (isDev()) {
                        log.info("取消pk,删掉redis battleStatusKey:{},targetBattleStatusKey:{}", battleStatusKey, targetBattleStatusKey);
                    }
                    // 返回匹配池
                    rtcBattleRoomCrossProvider.addToMatchPool(Arrays.asList(roomId, rtcBattleRoomCross.getTargetRoomId()));

                    // 发送消息
                    RtcBattleRoomCrossCmdMsg cmdMsg = new RtcBattleRoomCrossCmdMsg();
                    cmdMsg.setBattleId(battleId);
                    cmdMsg.setRoomId(rtcBattleRoomCross.getRoomId());
                    cmdMsg.setTargetRoomId(rtcBattleRoomCross.getTargetRoomId());
                    cmdMsg.setCmdType(RtcRoomBattleCrossCmdType.Canceled);
                    imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, rtcBattleRoomCross.getTargetRoomId(), cmdMsg);

                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 获取跨房 PK 记录
     *
     * @param battleId
     * @return
     */
    @Override
    public RtcBattleRoomCrossDO getBattleRoomCross(Long battleId) {
        return rtcBattleRoomCrossDAO.selectById(battleId);
    }

    /**
     * 决定跨房 PK 请求
     *
     * @param battleId
     * @param decision
     * @return
     */
    @Override
    public boolean decideRoomCrossBattle(Long battleId, int decision) {
        decision = decision == BoolIntVal.True ? BoolIntVal.True : BoolIntVal.False;

        RtcBattleRoomCrossDO rtcBattleRoomCross = rtcBattleRoomCrossDAO.selectById(battleId);
        if (rtcBattleRoomCross == null) {
            return false;
        }

        String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(rtcBattleRoomCross.getRoomId());
        String targetBattleStatusKey = RedisRtcKeys.getRtcBattleStatus(rtcBattleRoomCross.getTargetRoomId());

        if (BoolIntVal.True == decision) {
            // 接收 PK 请求
            if (rtcBattleRoomCrossDAO.updateStatus(battleId, BattleStatusEnum.Battling.getValue(), BattleStatusEnum.Ringing.getValue(), null) > 0) {
                // 设置缓存
                redisService.set(battleStatusKey, new RtcBattleStatus(battleId, BattleStatusEnum.Battling, RtcRoomBattleType.ROOM_CROSS, rtcBattleRoomCross.getTargetRoomId()).toString());
                redisService.set(targetBattleStatusKey, new RtcBattleStatus(battleId, BattleStatusEnum.Battling, RtcRoomBattleType.ROOM_CROSS, rtcBattleRoomCross.getRoomId()).toString());
                // 设置房间收听状态, 默认收听对方讲话
                redisService.set(RedisRtcKeys.getRtcBattleListenStatus(rtcBattleRoomCross.getRoomId()), String.valueOf(BoolIntVal.True));
                redisService.set(RedisRtcKeys.getRtcBattleListenStatus(rtcBattleRoomCross.getTargetRoomId()), String.valueOf(BoolIntVal.True));

                // 发送房间消息
                RtcBattleRoomCrossStatusMsg msg = new RtcBattleRoomCrossStatusMsg();
                msg.setBattleId(battleId);
                msg.setRoomId(rtcBattleRoomCross.getRoomId());
                msg.setTargetRoomId(rtcBattleRoomCross.getTargetRoomId());
                msg.setTopic(rtcBattleRoomCross.getTopic());
                msg.setDuration(rtcBattleRoomCross.getDuration());
                msg.setBattleStatus(BattleStatusEnum.Battling.getValue());
                msg.setEnableListen(BoolIntVal.True);
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, msg.getRoomId(), msg);
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, msg.getTargetRoomId(), msg);

                // 启动计时
                QueueRtcRoomBattleTimeUpMsg timeUpMsg = new QueueRtcRoomBattleTimeUpMsg();
                timeUpMsg.setBattleId(battleId);
                timeUpMsg.setRoomId(rtcBattleRoomCross.getRoomId());
                timeUpMsg.setStartTime(System.currentTimeMillis());
                timeUpMsg.setBattleType(RtcRoomBattleType.ROOM_CROSS);
                rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.RTC_ROOM_BATTLE_TIME_UP, timeUpMsg.serialize(), timeUpMsg.getStartTime() + rtcBattleRoomCross.getDuration());

                return true;
            }
        } else {
            // 拒绝 PK 请求
            if (rtcBattleRoomCrossDAO.updateStatus(battleId, BattleStatusEnum.Rejected.getValue(), BattleStatusEnum.Ringing.getValue(), System.currentTimeMillis()) > 0) {
                // 移除缓存
                redisService.delKey(battleStatusKey);
                redisService.delKey(targetBattleStatusKey);

                // 返回匹配池
                rtcBattleRoomCrossProvider.addToMatchPool(Arrays.asList(rtcBattleRoomCross.getRoomId(), rtcBattleRoomCross.getTargetRoomId()));

                // 发送消息
                RtcBattleRoomCrossCmdMsg cmdMsg = new RtcBattleRoomCrossCmdMsg();
                cmdMsg.setBattleId(battleId);
                cmdMsg.setRoomId(rtcBattleRoomCross.getRoomId());
                cmdMsg.setTargetRoomId(rtcBattleRoomCross.getTargetRoomId());
                cmdMsg.setCmdType(RtcRoomBattleCrossCmdType.Rejected);
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, rtcBattleRoomCross.getRoomId(), cmdMsg);

                return true;
            }
        }
        return false;
    }

    /**
     * 结束 PK
     *
     * @param roomId
     * @param battleId
     * @return
     */
    @Override
    public boolean stopRoomCrossBattle(Long roomId, Long battleId) {
        return rtcBattleRoomCrossProvider.battleOver(battleId);
    }

    /**
     * 获取 PK 记录列表
     *
     * @param roomId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<RtcBattleRoomCrossVO> getBattleRoomCrossHisList(Long roomId, int pageIndex, int pageSize) {
        PageList<RtcBattleRoomCrossVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(rtcBattleRoomCrossDAO.selectAll(roomId, pageList.getPageCursor(), pageSize));
        pageList.setTotal(rtcBattleRoomCrossDAO.countByRoomId(roomId));
        return pageList;
    }

    /**
     * 随机匹配对手
     *
     * @param roomId
     * @return
     */
    @Override
    public Long getRandomBattleRoomId(Long roomId) {
        return rtcBattleRoomCrossProvider.getRandomBattleRoomId(roomId);
    }

    /**
     * 设置房间匹配开关
     *
     * @param roomId
     * @param enabled
     * @return
     */
    @Override
    public boolean setBattleRoomCrossMatchEnabled(Long roomId, boolean enabled) {
        rtcBattleRoomCrossProvider.setMatchEnabled(roomId, enabled);
        return true;
    }

    /**
     * 获取当前跨房 PK
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcBattleRoomCrossVO getCurrentBattleRoomCross(Long roomId) {
        RtcBattleStatus battleStatus = rtcBattleProvider.getBattleStatus(roomId);
        if (battleStatus != null) {
            log.info("获取当前跨房PK详情 {} {} {}", roomId, battleStatus.getBattleType(), BattleStatusEnum.Battling == battleStatus.getBattleStatus());
        }
        if (battleStatus != null && RtcRoomBattleType.ROOM_CROSS == battleStatus.getBattleType() && BattleStatusEnum.Battling == battleStatus.getBattleStatus()) {
            RtcBattleRoomCrossVO vo = rtcBattleRoomCrossDAO.selectViewById(battleStatus.getBattleId());
            String enableListen = redisService.get(RedisRtcKeys.getRtcBattleListenStatus(roomId));
            vo.setEnableListen(Integer.parseInt(Optional.ofNullable(enableListen).orElse(String.valueOf(BoolIntVal.True))));

            if (isDev()) {
                log.info("获取房间消息:{}", JSON.toJSONString(vo));
            }
            return vo;
        }
        return null;
    }
}
