package com.starbuds.server.service.rtc.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rtc.RtcLoveStage;
import com.starbuds.server.common.pojo.message.im.rtc.RtcLoveChoseMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcLoveStageChangedMsg;
import com.starbuds.server.common.pojo.rtc.RtcLoveChose;
import com.starbuds.server.common.pojo.rtc.RtcLoveMatch;
import com.starbuds.server.common.pojo.rtc.RtcScoreAssister;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcLoveService;
import com.starbuds.server.service.rtc.provider.*;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * 相亲模式服务实现
 *
 * @author Lay
 * @date 2020/7/22
 */
@Service(version = "1.0.0")
public class RtcLoveServiceImpl implements RtcLoveService {

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    RtcLoveStageProvider rtcLoveStageProvider;

    @Resource
    RtcLoveChooseProvider rtcLoveChooseProvider;

    @Resource
    RtcLoveVipProvider rtcLoveVipProvider;

    @Resource
    RtcScoreProvider rtcScoreProvider;

    @Resource
    RtcSeatProvider rtcSeatProvider;

    @Resource
    RtcLoveScoreProvider rtcLoveScoreProvider;

    /**
     * 获取阶段
     *
     * @param roomId
     * @return
     */
    @Override
    public int getStage(Long roomId) {
        return rtcLoveStageProvider.getStage(roomId);
    }

    /**
     * 下一阶段
     *
     * @param roomId
     * @return
     */
    @Override
    public int nextStage(Long roomId, String appVer) {
        int curStage = rtcLoveStageProvider.getStage(roomId);
        int nextStage = curStage + 1;
        if (nextStage > RtcLoveStage.MATCH) {
            nextStage = 0;
        }

        // 设置阶段
        rtcLoveStageProvider.setStage(roomId, nextStage);

        // 判断下一阶段
        List<RtcLoveMatch> loveMatchList = null;
        List<RtcLoveChose> loveChoseList = null;
        switch (nextStage) {
            case RtcLoveStage.MATCH:
                // 公布心动
                loveMatchList = rtcLoveChooseProvider.getMatchList(roomId);
                loveChoseList = rtcLoveChooseProvider.getChoseList(roomId);
                if (CollectionUtil.isNotEmpty(loveMatchList)) {
                    rtcLoveChooseProvider.notifyAsync(roomId, loveMatchList);
                }
                break;
            case RtcLoveStage.READY:
                // 嘉宾上座
                rtcLoveChooseProvider.init(roomId);
                rtcLoveVipProvider.init(roomId, appVer);
                rtcScoreProvider.clearAll(roomId);
                rtcLoveScoreProvider.init(roomId);
                // 全部嘉宾下麦
                rtcSeatProvider.leaveAllAsync(roomId, appVer);
                break;
            default:
                break;
        }

        // 发送阶段改变消息
        RtcLoveStageChangedMsg loveStageChangedMsg = new RtcLoveStageChangedMsg();
        loveStageChangedMsg.setRoomId(roomId);
        loveStageChangedMsg.setPrevStage(curStage);
        loveStageChangedMsg.setStage(nextStage);
        loveStageChangedMsg.setLoveChoseList(loveChoseList);
        loveStageChangedMsg.setLoveMatchList(loveMatchList);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, loveStageChangedMsg);

        return nextStage;
    }

    /**
     * 重置阶段
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean resetStage(Long roomId, String appVer) {
        int curStage = rtcLoveStageProvider.getStage(roomId);

        // 设置阶段
        rtcLoveStageProvider.setStage(roomId, RtcLoveStage.READY);
        rtcLoveChooseProvider.init(roomId);
        rtcLoveVipProvider.init(roomId, appVer);
        rtcScoreProvider.clearAll(roomId);
        rtcLoveScoreProvider.init(roomId);

        // 发送消息
        RtcLoveStageChangedMsg loveStageChangedMsg = new RtcLoveStageChangedMsg();
        loveStageChangedMsg.setRoomId(roomId);
        loveStageChangedMsg.setPrevStage(curStage);
        loveStageChangedMsg.setStage(RtcLoveStage.READY);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, loveStageChangedMsg);

        return true;
    }

    /**
     * 心动选人
     *
     * @param roomId
     * @param userId
     * @param seatNo
     * @param targetUserId
     * @param targetSeatNo
     * @return
     */
    @Override
    public boolean chooseLove(Long roomId, Long userId, int seatNo, Long targetUserId, int targetSeatNo) {
        // 心动选人阶段才可选人
        if (RtcLoveStage.CHOOSE == rtcLoveStageProvider.getStage(roomId)) {
            if (rtcLoveChooseProvider.chooseLove(roomId, seatNo, targetSeatNo)) {
                // 发送消息
                RtcLoveChoseMsg loveChoseMsg = new RtcLoveChoseMsg();
                loveChoseMsg.setRoomId(roomId);
                loveChoseMsg.setUserId(userId);
                loveChoseMsg.setSeatNo(seatNo);
                imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, loveChoseMsg);

                return true;
            }
        }

        return false;
    }

    /**
     * 判断是否已选人
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @Override
    public boolean isLoveChose(Long roomId, int seatNo) {
        return rtcLoveChooseProvider.getChose(roomId, seatNo) != -1;
    }

    /**
     * 获取心动值助攻榜
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public List<RtcScoreAssister> getAssisterList(Long roomId, Long userId) {
        return rtcLoveScoreProvider.getAssisterList(roomId, userId);
    }
}
