package com.starbuds.server.service.rtc.impl;

import com.starbuds.server.common.pojo.rtc.RtcQueueUser;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcQueueService;
import com.starbuds.server.service.rtc.provider.RtcQueueProvider;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * RTC 队列服务实现
 *
 * @author Lay
 * @date 2020/7/7
 */
@Service(version = "1.0.0")
public class RtcQueueServiceImpl implements RtcQueueService {

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    private RtcQueueProvider queueProvider;

    /**
     * 加入队列
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean addQueueUser(Long roomId, Long userId) {
        if (queueProvider.addUser(roomId, userId)) {
            return true;
        }
        return false;
    }

    /**
     * 移出队列
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean removeQueueUser(Long roomId, Long userId) {
        if (queueProvider.removeUser(roomId, userId)) {
            return true;
        }
        return false;
    }

    /**
     * 清空队列
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean clearQueue(Long roomId) {
        if (queueProvider.clear(roomId)) {
            return true;
        }
        return false;
    }

    /**
     * 判断用户是否在队列中
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean isUserInQueue(Long roomId, Long userId) {
        return queueProvider.isExists(roomId, userId);
    }

    /**
     * 获取麦序用户列表
     *
     * @param roomId
     * @return
     */
    @Override
    public List<RtcQueueUser> getUsers(Long roomId) {
        return queueProvider.getUsers(roomId);
    }

    /**
     * 获取麦序用户数量
     *
     * @param roomId
     * @return
     */
    @Override
    public int getUsersCount(Long roomId) {
        return queueProvider.count(roomId);
    }
}
