package com.starbuds.server.service.rtc.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.rtc.RtcAuctionImKey;
import com.starbuds.server.common.constant.rtc.RtcAuctionSeatNo;
import com.starbuds.server.common.constant.rtc.RtcAuctionSeatType;
import com.starbuds.server.common.constant.rtc.RtcAuctionStatus;
import com.starbuds.server.common.exception.rtc.RtcAuctionRoomSetSitException;
import com.starbuds.server.common.exception.rtc.RtcEndAuctionException;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.RtcRoomAuctionDO;
import com.starbuds.server.common.pojo.db.RtcRoomAuctionHisDO;
import com.starbuds.server.common.pojo.db.WatchConfigDO;
import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.media.GiftItemsGetQuery;
import com.starbuds.server.common.pojo.message.im.rtc.RtcAuctionRoomEndMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcAuctionRoomStartMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcAuctionEndMsg;
import com.starbuds.server.common.pojo.rtc.RtcQueueUser;
import com.starbuds.server.common.pojo.rtc.qo.RtcRoomAuctionPageListQO;
import com.starbuds.server.common.pojo.rtc.vo.RtcAuctionSettingVO;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomAuctionBidRankVO;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomAuctionVO;
import com.starbuds.server.common.pojo.rtc.vo.WatchConfigVO;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcQueueService;
import com.starbuds.server.common.service.rtc.RtcRoomAuctionService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.service.user.UserWatchRelationService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.rtc.dao.RtcRoomAuctionDAO;
import com.starbuds.server.service.rtc.provider.RtcRoomAuctionHisProvider;
import com.starbuds.server.service.rtc.provider.RtcRoomAuctionProvider;
import com.starbuds.server.service.rtc.provider.RtcSeatProvider;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 拍卖房记录
 */
@Service(version = "1.0.0")
@Slf4j
public class RtcRoomAuctionServiceImpl implements RtcRoomAuctionService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "rtcRoomAuctionId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private RtcRoomAuctionProvider rtcRoomAuctionProvider;

    @Resource
    private RtcRoomAuctionDAO rtcRoomAuctionDAO;


    @Resource
    private RtcSeatProvider rtcSeatProvider;

    @Resource
    private RtcRoomAuctionHisProvider rtcRoomAuctionHisProvider;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Reference(version = "1.0.0")
    private UserWatchRelationService userWatchRelationService;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Reference(version = "1.0.0")
    private RtcQueueService rtcQueueService;

    @Reference(version = "1.0.0")
    private ImService imService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Resource
    private RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    private RtcSeatService rtcSeatService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public RtcRoomAuctionDO create(RtcRoomAuctionDO po) {
        return rtcRoomAuctionProvider.create(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<RtcRoomAuctionDO> list) {
        return rtcRoomAuctionProvider.createBatch(list);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(RtcRoomAuctionDO po) {
        return rtcRoomAuctionProvider.createOrUpdate(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(RtcRoomAuctionDO po) {
        return rtcRoomAuctionProvider.updateById(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        return rtcRoomAuctionProvider.update(uw);
    }

    @Override
    public RtcRoomAuctionDO getById(Long id) {
        return rtcRoomAuctionProvider.getById(id);
    }

    @Override
    public boolean exist(Long id) {
        return rtcRoomAuctionProvider.exist(id);
    }

    @Override
    public PageList<RtcRoomAuctionDO> getPageList(RtcRoomAuctionPageListQO qo) {
        return rtcRoomAuctionProvider.getPageList(qo);
    }

    @Override
    public Map<Long, RtcRoomAuctionDO> getMapByIds(Collection ids) {
        return rtcRoomAuctionProvider.getMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 获取拍卖房起拍设置的相关数据
     *
     * @return
     */
    @Override
    public RtcAuctionSettingVO getStartAuctionSettingData() {
        // 语聊礼物
        List<GiftItemVO> gifts = giftService.getGiftItems(GiftItemsGetQuery.builder().displayType(3).build());
        if (!CollectionUtils.isEmpty(gifts)) {
            gifts = gifts.stream()
                    .filter(Objects::nonNull)
                    .filter(gift -> gift.getGiftType() != null)
                    .filter(gift -> gift.getGiftListVisible() != null && gift.getGiftListVisible() == BoolIntVal.True)
                    .filter(gift -> {
                        WalletTypeEnum priceType = WalletTypeEnum.toEnum(gift.getGiftPriceType());
                        return WalletTypeEnum.STAR_DIAMOND == priceType;
                    })
                    .filter(gift -> gift.getGiftType() - GiftTypeEnum.LUCKY.getValue() != 0 && gift.getGiftType() - GiftTypeEnum.LOTTERY.getValue() != 0)
                    .collect(Collectors.toList());

        }

        List<WatchConfigDO> watchConfigList = userWatchRelationService.getWatchConfigList(BoolIntVal.Ignore);
        List<WatchConfigVO> watchItems = Collections.emptyList();
        if (!CollectionUtils.isEmpty(watchConfigList)) {
            watchItems = watchConfigList.stream()
                    .filter(Objects::nonNull)
                    .map(config -> {
                        WatchConfigVO watchVO = new WatchConfigVO();
                        BeanUtils.copyProperties(config, watchVO);
                        return watchVO;
                    })
                    .collect(Collectors.toList());
        }

        RtcAuctionSettingVO vo = new RtcAuctionSettingVO();
        vo.setGifts(gifts);
        vo.setWatchDurations(userWatchRelationService.getWatchDurationList());
        vo.setWatchItems(watchItems);

        return vo;
    }

    /**
     * 设置拍卖信息
     *
     * @param userId        拍卖对象用户id
     * @param roomId        拍卖房id
     * @param watchConfigId 拍卖物品（守护）id
     * @param watchDuration 守护时长
     * @param watchName     守护名称
     * @param startGiftId   起拍礼物id
     * @param giftName      礼物名称
     * @param giftPrice     礼物价格
     * @param giftIcon      礼物图标
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public RtcRoomAuctionDO setAuction(Long userId, Long roomId,
                                       Long watchConfigId, Integer watchDuration, String watchName,
                                       Long startGiftId, String giftName, Long giftPrice, String giftIcon)
            throws RtcAuctionRoomSetSitException {
        RtcRoomAuctionDO po = new RtcRoomAuctionDO();
        po.setBidUserId(userId);
        po.setRoomId(roomId);
        po.setBidItemId(watchConfigId);
        po.setBidDuration(watchDuration);
        po.setStartingGiftId(startGiftId);
        po.setStatus(RtcAuctionStatus.BIDDING);
        po.setCreateDate(DateUtil.getIntDate());
        po.setIsDeleted(BoolIntVal.False);

        po.setBidItemName(watchName);
        po.setStartingGiftName(giftName);
        po.setStartingGiftPrice(giftPrice);

        Long hostUserId = rtcSeatProvider.getSeatUserId(roomId, RtcAuctionSeatNo.HOST);
        if (hostUserId != null) {
            po.setHostUserId(hostUserId);
        }
        String key = RedisRtcKeys.getRtcAuctionRoomSetLock(roomId);
        String value = UUID.randomUUID().toString();
        String auctionUserKey = RedisRtcKeys.getRtcAuctionRoomUser(roomId);
        try {
            boolean lock = redisService.tryLock(key, value, 10, TimeUnit.SECONDS);
            if (lock) {
                // 检查是否有竞拍中的记录
                if (isBidding(roomId)) {
                    throw new RtcAuctionRoomSetSitException("竞拍进行中，不能重复创建");
                }
                int biddingCount = rtcRoomAuctionDAO.selectBiddingCount(roomId, RtcAuctionStatus.BIDDING);
                if (biddingCount > 0) {
                    throw new RtcAuctionRoomSetSitException("竞拍进行中，不能重复创建！");
                }

                po = rtcRoomAuctionProvider.create(po);
                if (po == null) {
                    throw new RtcAuctionRoomSetSitException("创建竞拍记录失败");
                }

                // 设置拍卖房中的竞拍对象
                redisService.set(auctionUserKey, userId + "");
                // 设置拍卖信息
                RtcRoomAuctionVO vo = new RtcRoomAuctionVO();
                BeanUtils.copyProperties(po, vo);
                vo.setStartingGiftIcon(giftIcon);
                redisService.set(RedisRtcKeys.getRtcAuctionRoomInfo(roomId), JSON.toJSONString(vo));
                // 清空排名
                clearBidRank(roomId);
            }
        } catch (Exception e) {
            log.error("解锁设置拍卖信息出现错误", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            // 清除房间中竞拍对象
            redisService.delKey(auctionUserKey);
            if (e instanceof RtcAuctionRoomSetSitException) {
                throw e;
            }
            throw new RtcAuctionRoomSetSitException("创建竞拍记录失败！");
        } finally {
            redisService.releaseLock(key, value);
        }

        // 发送拍卖开始消息
        RtcAuctionRoomStartMsg startMsg = new RtcAuctionRoomStartMsg();
        BeanUtils.copyProperties(po, startMsg);
        startMsg.setStartingGiftIcon(giftIcon);
        // 被拍卖的用户
        UserCardVO bidUser = userService.getNameAndAvatarById(userId);
        if (bidUser != null) {
            startMsg.setBidUserName(bidUser.getUserName());
            startMsg.setBidUserAvatar(bidUser.getUserAvatar());
        }
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, startMsg);
        return po;
    }

    /**
     * 结束拍卖
     *
     * @param roomId
     * @return true 成功
     * @throws RtcAuctionRoomSetSitException
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void endAuction(Long roomId, String appVer) throws RtcEndAuctionException {
        String key = RedisRtcKeys.getRtcAuctionRoomSetLock(roomId);
        String value = UUID.randomUUID().toString();
        String auctionUserKey = RedisRtcKeys.getRtcAuctionRoomUser(roomId);
        try {
            boolean lock = redisService.tryLock(key, value, 10, TimeUnit.SECONDS);
            if (lock) {
                QueryWrapper qw = new QueryWrapper();
                qw.eq("roomId", roomId);
                int status = RtcAuctionStatus.BIDDING;
                qw.eq("status", status);
                RtcRoomAuctionDO biddingPo = rtcRoomAuctionDAO.selectOne(qw);
                if (biddingPo == null) {
                    log.error(String.format("结束拍卖失败：当前房间没有在竞拍中 roomId = %s status = %s", roomId, status));
                    throw new RtcEndAuctionException("结束拍卖失败：当前房间没有在竞拍中");
                }
                long auctionUserId = getAuctionUserId(roomId);
                if (biddingPo.getBidUserId() - auctionUserId != 0) {
                    log.error(String.format("结束拍卖失败：竞拍对象发生变化 roomId = %s status = %s auctionUserId = [数据库 %s redis %s]",
                            roomId, status, biddingPo.getBidUserId(), auctionUserId));
                    throw new RtcEndAuctionException("结束拍卖失败：竞拍对象发生变化");
                }

                // 计算拍卖结果
                Long rtcRoomAuctionId = biddingPo.getRtcRoomAuctionId();
                RtcRoomAuctionHisDO winner = getWinner(roomId);
                if (winner == null) {
                    status = RtcAuctionStatus.PASS;
                } else {
                    // 拍卖信息
                    String auctionStr = redisService.get(RedisRtcKeys.getRtcAuctionRoomInfo(roomId));
                    if (JSON.isValidObject(auctionStr)) {
                        RtcRoomAuctionVO auction = JSON.parseObject(auctionStr, RtcRoomAuctionVO.class);
                        if (auction != null) {
                            Long startingGiftPrice = auction.getStartingGiftPrice();
                            Long giftTotalPrice = winner.getGiftTotalPrice();
                            if (startingGiftPrice != null && giftTotalPrice != null
                                    && giftTotalPrice >= startingGiftPrice) {
                                status = RtcAuctionStatus.WIN;
                            } else {
                                log.info("拍卖结束：流拍 winner = {} auctionStr = {}", winner, auctionStr);
                                status = RtcAuctionStatus.PASS;
                                winner = null;
                            }
                        }
                    }
                }
                int bidUserCount = rtcRoomAuctionHisProvider.selectBidUserCount(rtcRoomAuctionId);
                Long totalPrice = rtcRoomAuctionHisProvider.selectTotalPrice(rtcRoomAuctionId);

                // 更新拍卖结果
                RtcRoomAuctionDO upPo = new RtcRoomAuctionDO();
                if (winner != null) {
                    upPo.setWinUserId(winner.getSendUserId());
                    upPo.setHighestBidPrice(winner.getGiftTotalPrice());
                }
                upPo.setStatus(status);
                upPo.setBidUserCount(bidUserCount);
                upPo.setBidTotalPrice(totalPrice);
                upPo.setBidEndTime(System.currentTimeMillis());

                QueryWrapper upQw = new QueryWrapper();
                upQw.eq(PRIMARY_KEY, rtcRoomAuctionId);
                qw.eq("status", RtcAuctionStatus.BIDDING);

                UpdateWrapper uw = new UpdateWrapper();
                uw.setQw(upQw);
                uw.setPo(upPo);
                boolean update = update(uw);
                if (!update) {
                    log.error(String.format("结束拍卖失败：更新拍卖房记录失败 id = %s status = %s", rtcRoomAuctionId, status));
                    throw new RtcEndAuctionException("结束拍卖失败");
                }

                // 清除房间中竞拍对象
                Boolean clearAuctionUser = redisService.delKey(auctionUserKey);
                if (!clearAuctionUser) {
                    log.error(String.format("结束拍卖失败：清除房间中竞拍对象 redis 数据失败 roomId = %s status = %s", roomId, status));
                    throw new RtcEndAuctionException("结束拍卖失败");
                }
                // 清除拍卖信息
                Boolean clearAuctionRoomInfo = redisService.delKey(RedisRtcKeys.getRtcAuctionRoomInfo(roomId));
                if (!clearAuctionRoomInfo) {
                    log.error(String.format("结束拍卖失败：清除拍卖信息 redis 数据失败 roomId = %s status = %s", roomId, status));
                    throw new RtcEndAuctionException("结束拍卖失败");
                }
                clearBidRank(roomId);

                // 强制下麦
                if (rtcSeatService.leaveSeat(roomId, RtcAuctionSeatType.BID, true, appVer) > -1) {
                    log.info("竞拍麦下麦成功 roomId = ", roomId);
                } else {
                    log.error("竞拍麦下麦失败 roomId = " + roomId);
                }

                // 发送拍卖结束消息
                RtcAuctionRoomEndMsg endMsg = new RtcAuctionRoomEndMsg();
                BeanUtils.copyProperties(biddingPo, endMsg);
                endMsg.setStatus(status);
                // 被拍卖的用户
                UserCardVO bidUser = userService.getNameAndAvatarById(biddingPo.getBidUserId());
                if (bidUser != null) {
                    endMsg.setBidUserName(bidUser.getUserName());
                    endMsg.setBidUserAvatar(bidUser.getUserAvatar());
                }
                if (winner != null) {
                    // 中标 设置竞拍人信息
                    Long winUserId = winner.getSendUserId();
                    UserCardVO winUser = userService.getNameAndAvatarById(winUserId);
                    endMsg.setWinUserId(winUserId);
                    endMsg.setWinUserAvatar(winUser.getUserAvatar());
                    endMsg.setWinUserName(winUser.getUserName());
                }
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, endMsg);

                // 丢出拍卖结束消息到消息队列
                QueueRtcAuctionEndMsg rocketAuctionEndMsg = new QueueRtcAuctionEndMsg();
                BeanUtils.copyProperties(getById(rtcRoomAuctionId), rocketAuctionEndMsg);
                boolean sendQueueRtcAuctionEndMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_AUCTION_END, rocketAuctionEndMsg.serialize());
                log.info("拍卖结束：丢出拍卖结束消息到消息队列 {}", sendQueueRtcAuctionEndMsg);
                if (isDev()) {
                    log.info("拍卖结束 {}", JSON.toJSONString(rocketAuctionEndMsg, true));
                }

                log.info("结束拍卖成功 roomId = {} auctionUserId= [{} 结束后 {}]",
                        roomId, biddingPo.getBidUserId(), getAuctionUserId(roomId));
            }
        } catch (Exception e) {
            log.error("解锁结束拍卖出现错误", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            if (e instanceof RtcEndAuctionException) {
                throw e;
            }
            throw new RtcEndAuctionException("结束拍卖失败！");
        } finally {
            redisService.releaseLock(key, value);
        }
    }

    /**
     * 是否竞拍中
     *
     * @param roomId
     * @return true 竞拍中
     */
    @Override
    public boolean isBidding(Long roomId) {
        return rtcRoomAuctionProvider.isBidding(roomId);
    }

    /**
     * 获得正在被竞拍的用户id
     *
     * @param roomId
     * @return 大于0才是正常id
     */
    @Override
    public long getAuctionUserId(Long roomId) {
        return rtcRoomAuctionProvider.getAuctionUserId(roomId);
    }

    /**
     * 添加申请上竞拍麦的用户
     *
     * @param roomId
     * @param userId
     */
    @Override
    public void saveApplyAuctionSeatUser(Long roomId, Long userId) {
        String key = RedisRtcKeys.getRtcAuctionRoomSeatUsers(roomId);
        redisService.sAdd(key, userId + "");
    }

    /**
     * 所有申请上竞拍麦位的用户
     *
     * @param roomId
     * @return
     */
    @Override
    public Set<Long> getApplyAuctionSeatUsers(Long roomId) {
        Set<String> userIds = redisService.sMembers(RedisRtcKeys.getRtcAuctionRoomSeatUsers(roomId));
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptySet();
        }
        return userIds.stream()
                .map(NumberUtils::toLong)
                .filter(userId -> userId > 0)
                .collect(Collectors.toSet());
    }

    /**
     * 移除申请上竞拍麦位的用户
     *
     * @param roomId
     * @param userId
     */
    @Override
    public void removeApplyAuctionSeatUsers(Long roomId, Long userId) {
        redisService.sRemove(RedisRtcKeys.getRtcAuctionRoomSeatUsers(roomId), String.valueOf(userId));
    }

    /**
     * 获取麦序用户列表-拍卖房
     *
     * @param roomId
     * @param type   麦位类型：1:竞拍麦位；2:普通麦位
     * @return
     */
    @Override
    public List<RtcQueueUser> getQueueUsersInAuction(Long roomId, Integer type) {
        // 普通、竞拍所有麦序
        List<RtcQueueUser> users = rtcQueueService.getUsers(roomId);
        if (CollectionUtils.isEmpty(users)) {
            return Collections.emptyList();
        }
        Set<Long> applyAuctionSeatUsers = getApplyAuctionSeatUsers(roomId);

        if (isDev()) {
            log.info("获取麦序用户列表-拍卖房\nusers = {}\napplyAuctionSeatUsers = {}",
                    JSON.toJSONString(users, true), JSON.toJSONString(applyAuctionSeatUsers, true));
        }

        return users.stream()
                .filter(Objects::nonNull)
                .filter(user -> user.getUserId() != null)
                .filter(user -> {
                    boolean isAuctionUser = applyAuctionSeatUsers.contains(user.getUserId());
                    if (RtcAuctionSeatType.BID == type) {
                        return isAuctionUser;
                    } else {
                        return !isAuctionUser;
                    }
                })
                .collect(Collectors.toList());
    }

    /**
     * 获得竞拍拍卖列表
     *
     * @param roomId
     * @return
     */
    @Override
    public List<RtcRoomAuctionBidRankVO> getBidRankTops(Long roomId) {
        return rtcRoomAuctionProvider.getBidRankTops(roomId);
    }

    /**
     * 获取热度最高的拍卖房ID
     *
     * @return
     */
    @Override
    public Long getHotAuctionRoomId() {
        return rtcRoomAuctionProvider.getHotAuctionRoomId();
    }

    /**
     * 清空拍卖排名信息
     *
     * @param roomId
     */
    private void clearBidRank(Long roomId) {
        redisService.delKey(RedisRtcKeys.getRtcAuctionRoomRank(roomId));
        redisService.delKey(RedisRtcKeys.getRtcAuctionRoomRankTime(roomId));
        imService.setRoomProperty(roomId, RtcAuctionImKey.BID_RANK_TOPS_KEY, "[]");
    }

    /**
     * 获得中标用户
     *
     * @param roomId
     * @return
     */
    private RtcRoomAuctionHisDO getWinner(Long roomId) {
        if (!isBidding(roomId)) {
            log.error("房间不在拍卖中，无法计算拍卖结果 roomId = " + roomId);
            return null;
        }

        List<RtcRoomAuctionBidRankVO> bidRankTops = getBidRankTops(roomId);
        if (CollectionUtils.isEmpty(bidRankTops)) {
            return null;
        }
        RtcRoomAuctionBidRankVO top = bidRankTops.get(0);
        if (top == null) {
            return null;
        }
        Long userId = top.getUserId();
        Long totalPrice = top.getTotalPrice();
        if (userId == null || totalPrice == null) {
            return null;
        }

        RtcRoomAuctionHisDO winner = new RtcRoomAuctionHisDO();
        winner.setSendUserId(userId);
        winner.setGiftTotalPrice(totalPrice);
        return winner;
    }
}