package com.starbuds.server.service.rtc.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.RtcRoomLiveHisDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.rtc.dao.RtcRoomLiveHisDAO;
import lombok.extern.slf4j.Slf4j;
import com.starbuds.server.common.service.rtc.RtcRoomLiveHisService;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 个播开播记录
 */
@Service(version = "1.0.0")
@Slf4j
public class RtcRoomLiveHisServiceImpl implements RtcRoomLiveHisService {

    @Resource
    RtcRoomLiveHisDAO rtcRoomLiveHisDAO;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Override
    public RtcRoomLiveHisDO create(RtcRoomLiveHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setCreateDate(DateUtil.getIntDate());
        boolean flag = rtcRoomLiveHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("个播开播记录创建成功 {}", json);
            return po;
        }
        log.warn("个播开播记录创建失败 {}", json);
        return null;
    }

    @Override
    public boolean createBatch(List<RtcRoomLiveHisDO> list) {
        return false;
    }

    @Override
    public boolean createOrUpdate(RtcRoomLiveHisDO po) {
        return false;
    }

    @Override
    public boolean updateById(RtcRoomLiveHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq("hisId", po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    @Override
    public boolean update(UpdateWrapper uw) {
        return rtcRoomLiveHisDAO.update(uw) > 0;
    }

    @Override
    public RtcRoomLiveHisDO getById(Long id) {
        return null;
    }

    @Override
    public boolean exist(Long id) {
        return false;
    }

    @Override
    public boolean existByRoomId(Long roomId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("roomId", roomId);
        return rtcRoomLiveHisDAO.selectCount(qw) > 0;
    }

    @Override
    public Map<Long, RtcRoomLiveHisDO> getMapByIds(Collection ids) {
        return null;
    }

    /**
     * 根据房间id查询正在进行的个播场次
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcRoomLiveHisDO getCurrentRound(Long roomId) {
        return rtcRoomLiveHisDAO.selectCurrentRoundByRoomId(roomId);
    }

    /**
     * 根据房间id查询最近的个播场次
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcRoomLiveHisDO getLastRound(Long roomId) {
        return rtcRoomLiveHisDAO.selectLastRoundByRoomId(roomId);
    }

    /**
     * 获取最近100条记录
     *
     * @param roomId
     * @return
     */
    @Override
    public PageList<RtcRoomLiveHisDO> getRecentLiveHis(Integer pageIndex, Integer pageSize, Long roomId) {
        PageList<RtcRoomLiveHisDO> pageList = new PageList<>(pageIndex, pageSize);

        QueryWrapper qw = new QueryWrapper();
        qw.setPageSize(pageList.getPageSize());
        qw.setPageCursor(pageList.getPageCursor());
        qw.eq("roomId", roomId);

        pageList.setList(rtcRoomLiveHisDAO.selectPageList(qw));
        pageList.setTotal(rtcRoomLiveHisDAO.selectCount(qw));
        return pageList;
    }

    /**
     * 将正在进行的个播场次结束
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean setRoundEnd(Long roomId) {
        return rtcRoomLiveHisDAO.setRoundEnd(roomId);
    }

    /**
     * 获取今日开播总时长，毫秒数
     *
     * @param roomId
     * @return
     */
    @Override
    public Long getTodayTotalTimeMillis(Long roomId) {
        return rtcRoomLiveHisDAO.getTodayTotalTimeMillis(roomId);
    }

    /**
     * 获取今日开播总场次
     *
     * @param roomId
     * @return
     */
    @Override
    public Integer getTodayRoundNum(Long roomId) {
        return rtcRoomLiveHisDAO.getTodayRoundNum(roomId);
    }
}