package com.starbuds.server.service.rtc.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.RtcRoomLiveOfflineHisDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.rtc.dao.RtcRoomLiveOfflineHisDAO;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;

import com.starbuds.server.common.service.rtc.RtcRoomLiveOfflineHisService;

import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import java.util.List;

/**
 * 个播-主播离线记录定时任务扫描
 */
@Service(version = "1.0.0")
@Slf4j
public class RtcRoomLiveOfflineHisServiceImpl implements RtcRoomLiveOfflineHisService {

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private RtcRoomLiveOfflineHisDAO rtcRoomLiveOfflineHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Override
    public RtcRoomLiveOfflineHisDO create(RtcRoomLiveOfflineHisDO po) {
        po.setHisId(identityService.getId());
        po.setIsDeal(BoolIntVal.False);
        boolean flag = rtcRoomLiveOfflineHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("个播-主播离线记录定时任务扫描创建成功 {}", json);
            return po;
        }
        log.warn("个播-主播离线记录定时任务扫描创建失败 {}", json);
        return null;
    }

    /**
     * 查询未处理的离线记录
     *
     * @return
     */
    @Override
    public List<RtcRoomLiveOfflineHisDO> getUnDealRecord() {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("isDeal", BoolIntVal.False);
        return rtcRoomLiveOfflineHisDAO.selectList(qw);
    }

    /**
     * 设置未处理的记录为处理
     *
     * @return
     */
    @Override
    public boolean setUnDealRecord(Long hisId) {
        RtcRoomLiveOfflineHisDO hisDO = new RtcRoomLiveOfflineHisDO();
        hisDO.setIsDeal(BoolIntVal.True);
        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(hisDO);
        QueryWrapper qw = new QueryWrapper();
        qw.eq("hisId", hisId);
        uw.setQw(qw);
        return rtcRoomLiveOfflineHisDAO.update(uw) > 0;
    }

    // 手动编码开始

}