package com.starbuds.server.service.rtc.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Preconditions;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.rtc.*;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.data.NoticeConfigVO;
import com.starbuds.server.common.pojo.data.RtcRoomBackground;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.live.LiveUserBlackMsg;
import com.starbuds.server.common.pojo.message.im.live.LiveUserForbiddenMsg;
import com.starbuds.server.common.pojo.message.im.rtc.*;
import com.starbuds.server.common.pojo.mq.QueueRtcRoomInfoESMsg;
import com.starbuds.server.common.pojo.rtc.*;
import com.starbuds.server.common.pojo.rtc.bo.RtcRoomRatio;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomVisitHisRichVO;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomVisitHisVO;
import com.starbuds.server.common.pojo.rtc.vo.StatBarrageRsp;
import com.starbuds.server.common.pojo.user.FansUser;
import com.starbuds.server.common.pojo.user.UserSimpleVO;
import com.starbuds.server.common.service.activity.ActivityAnniversaryService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.RtcRoomSearchService;
import com.starbuds.server.common.service.finance.SendGiftSnapshotService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.media.VoiceChatService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.*;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.tag.rtc.RtcRoomTag;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.rtc.dao.*;
import com.starbuds.server.service.rtc.provider.*;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.apache.dubbo.config.annotation.Method;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.apache.skywalking.apm.toolkit.trace.TraceContext;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.cache.RedisCache;
import org.springframework.scheduling.annotation.Async;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/7/6
 */
@Slf4j
@Service(version = "1.0.0", methods = {@Method(name = "addRoom", timeout = 5000)})
public class RtcRoomServiceImpl implements RtcRoomService {

    @Value("${turntable.animationDuration}")
    private Long turntableAnimationDuration;

    /**
     * 融云统计地址
     */
    @Value("${apiHost.apiNotifyRongcloud}")
    private String apiNotifyRongcloud;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    RtcRoomSearchService rtcRoomSearchService;

    @Reference(version = "1.0.0")
    RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    private SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    RtcBattleService rtcBattleService;

    @Reference(version = "1.0.0")
    VoiceChatService voiceChatService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    private RtcRoomLiveHisService rtcRoomLiveHisService;

    @Reference(version = "1.0.0")
    private RtcRoomLiveOfflineHisService rtcRoomLiveOfflineHisService;

    @Reference(version = "1.0.0")
    private SendGiftSnapshotService sendGiftSnapshotService;

    @Reference(version = "1.0.0")
    private WalletService walletService;

    @Reference(version = "1.0.0")
    private ActivityAnniversaryService activityAnniversaryService;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    private RoomNumberPoolDAO roomNumberPoolDAO;

    @Resource
    private RtcRoomDAO rtcRoomDAO;

    @Resource
    private RtcRoomRoleDAO rtcRoomRoleDAO;

    @Resource
    private RtcRoomFavDAO rtcRoomFavDAO;

    @Resource
    private AliRtcProvider aliRtcProvider;

    @Resource
    private RtcHeatV2Provider rtcHeatV2Provider;

    @Resource
    private RtcPopularityProvider rtcPopularityProvider;

    @Resource
    private RtcQueueProvider rtcQueueProvider;

    @Resource
    private RtcScoreProvider rtcScoreProvider;

    @Resource
    private RtcSeatProvider rtcSeatProvider;

    @Resource
    private RtcRoomBlackDAO rtcRoomBlackDAO;

    @Resource
    private RtcRoomForbiddenDAO rtcRoomForbiddenDAO;

    @Resource
    private FacialExpressionDAO facialExpressionDAO;

    @Resource
    StatRtcRoomDAO statRtcRoomDAO;

    @Resource
    private RtcRoomJoinHisDAO rtcRoomJoinHisDAO;

    @Resource
    private RtcRoomVisitHisDAO rtcRoomVisitHisDAO;

    @Resource
    private RtcRoomStayLogDAO rtcRoomStayLogDAO;

    @Resource
    RtcLoveScoreProvider rtcLoveScoreProvider;

    @Resource
    RtcRankService rtcRankService;

    @Resource
    RtcBattleRoomCrossProvider rtcBattleRoomCrossProvider;

    @Resource
    private RtcRoomProvider rtcRoomProvider;

    @Resource
    private NoticeProvider noticeProvider;

    @Resource
    private RtcRoomCategoryProvider rtcRoomCategoryProvider;

    @Resource
    private FamilyProvider familyProvider;

    @Resource
    private RtcRoomCategoryDAO rtcRoomCategoryDAO;

    private static final List<Integer> ROOM_STAT_TYPES = Arrays.asList(StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue(),
            StatRtcRoomTypeEnum.RECEIVE_STAR_DIAMOND_FAMILY_USER.getValue(),
            StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND.getValue(),
            StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND.getValue(),
            StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND.getValue(),
            StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND_FAMILY_USER.getValue(),
            StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND_FAMILY_USER.getValue(),
            StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND_FAMILY_USER.getValue()
    );
    /**
     * 最大尝试次数
     */
    private static final int RANDOM_ID_MAX_TIMES = 100;

    /**
     * 创建房间
     *
     * @param rtcRoomAddDTO
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long addRoom(RtcRoomAddDTO rtcRoomAddDTO) {
        // 减少用户可建房数量
        // 创建房间
        Long roomId = identityService.getId();
        RtcRoomDO rtcRoomDO = new RtcRoomDO();
        rtcRoomDO.setRoomId(roomId);
        rtcRoomDO.setUserId(rtcRoomAddDTO.getUserId());
        rtcRoomDO.setBusinessType(rtcRoomAddDTO.getBusinessType() == null ? 0 : rtcRoomAddDTO.getBusinessType());
        rtcRoomDO.setRoomType(rtcRoomAddDTO.getRoomType());
        rtcRoomDO.setRoomName(rtcRoomAddDTO.getRoomName());
        rtcRoomDO.setCategoryId(rtcRoomAddDTO.getCategoryId());
        rtcRoomDO.setRoomTitle(rtcRoomAddDTO.getRoomTitle());
        rtcRoomDO.setRoomCover(rtcRoomAddDTO.getRoomCover());
        rtcRoomDO.setRoomHorizontalCover(rtcRoomAddDTO.getRoomHorizontalCover());
        rtcRoomDO.setStatus(RtcBusinessTypeEnum.RTC_SINGLE.equals(RtcBusinessTypeEnum.toEnum(rtcRoomAddDTO.getBusinessType())) ? CommonStatus.Disabled : CommonStatus.Normal);
        rtcRoomDO.setRoomCreateTime(System.currentTimeMillis());
        rtcRoomDO.setRoomMicFree(BoolIntVal.False);
        rtcRoomDO.setRoomScoreEnabled(RoomTypeEnum.LOVE.getValue() == rtcRoomAddDTO.getRoomType() ? BoolIntVal.True : BoolIntVal.False);
        rtcRoomDO.setRoomChatEnabled(BoolIntVal.True);
        rtcRoomDO.setBigAdventureEnabled(BoolIntVal.False);
        rtcRoomDO.setRoomHeat(0L);
        rtcRoomDO.setFamilyId(rtcRoomAddDTO.getFamilyId());

        // 获取一个房间号
        int times = 0;
        while (times < RANDOM_ID_MAX_TIMES) {
            Long roomNo = roomNumberPoolDAO.selectRandomId();
            if (roomNo != null) {
                // 占用
                int updated = roomNumberPoolDAO.updateId(roomNo, roomId);
                if (updated > 0) {
                    rtcRoomDO.setRoomNo(roomNo);
                    break;
                }
            } else {
                // 号池耗尽
                throw new RuntimeException("号池耗尽");
            }

            times++;
        }
        rtcRoomDAO.insert(rtcRoomDO);

        // 创建房主角色
        RtcRoomRoleDO rtcRoomRoleDO = new RtcRoomRoleDO();
        rtcRoomRoleDO.setRoomId(roomId);
        rtcRoomRoleDO.setUserId(rtcRoomAddDTO.getUserId());
        rtcRoomRoleDO.setRoomRole(RtcRoomRole.ADMIN);
        rtcRoomRoleDAO.insertOrUpdate(rtcRoomRoleDO);

        // 初始化人数
        rtcPopularityProvider.init(roomId);

        // 同步房间
//        rtcRoomSearchService.saveOrUpdate(rtcRoomDO);

        // 语聊房间数统计
        statPlatformService.addPlatformStat(rtcRoomAddDTO.getChannelId(), StatPlatformTypeEnum.RTC_ROOM_NUM_ADD, BigDecimal.ONE);
        if (rtcRoomDO.getFamilyId() == null) {
            // 语聊房间数统计-新增个人房间数
            statPlatformService.addPlatformStat(rtcRoomAddDTO.getChannelId(), StatPlatformTypeEnum.RTC_ROOM_NUM_ADD_PRIVATE, BigDecimal.ONE);
        } else {
            // 语聊房间数统计-新增家族房间数
            statPlatformService.addPlatformStat(rtcRoomAddDTO.getChannelId(), StatPlatformTypeEnum.RTC_ROOM_NUM_ADD_FAMILY, BigDecimal.ONE);
        }

        QueueRtcRoomInfoESMsg roomESInfoMsg = new QueueRtcRoomInfoESMsg();
        roomESInfoMsg.setRoomId(roomId);
        roomESInfoMsg.setRoomName(rtcRoomDO.getRoomName());
        roomESInfoMsg.setUserId(rtcRoomDO.getUserId());
        roomESInfoMsg.setRoomType(rtcRoomDO.getRoomType());
        roomESInfoMsg.setRoomTitle(rtcRoomDO.getRoomTitle());
        roomESInfoMsg.setRoomNo(rtcRoomDO.getRoomNo());
        roomESInfoMsg.setStatus(rtcRoomDO.getStatus());
        roomESInfoMsg.setFamilyId(rtcRoomDO.getFamilyId());
        roomESInfoMsg.setRoomCreateTime(rtcRoomDO.getRoomCreateTime());
        roomESInfoMsg.setRoomLockPass(rtcRoomDO.getRoomLockPass());
        roomESInfoMsg.setRoomWelcome(rtcRoomDO.getRoomWelcome());
        roomESInfoMsg.setRoomRule(rtcRoomDO.getRoomRule());
        roomESInfoMsg.setRoomScoreEnabled(rtcRoomDO.getRoomScoreEnabled());
        roomESInfoMsg.setRoomChatEnabled(rtcRoomDO.getRoomChatEnabled());
        roomESInfoMsg.setRoomMicFree(rtcRoomDO.getRoomMicFree());
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_ROOM_INFO_ES_SYNCH, roomESInfoMsg.serialize());

        return roomId;
    }

    /**
     * 判断房间是否存在
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isRoomExists(Long roomId) {
        if (isDev()) {
            log.info("xxxx-roomId={}-{}", roomId, TraceContext.traceId());
        }

        if (roomId == null) {
            return false;
        }
        return rtcRoomDAO.countById(roomId) > 0;
    }

    /**
     * 判断房间是否开播
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isRoomOnline(Long roomId) {

        if (roomId == null) {
            return false;
        }
        return rtcRoomDAO.isRoomOnline(roomId) > 0;
    }

    /**
     * 获取用户房间数量
     *
     * @param userId
     * @return
     */
    @Override
    public int getRoomCountByUserId(Long userId) {
        return rtcRoomDAO.countByUserId(userId, RtcBusinessTypeEnum.RTC_NORMAL.getValue());
    }

    /**
     * 获取用户个播房间数量
     *
     * @param userId
     * @return
     */
    @Override
    public int getSingleRtcRoomCountByUserId(Long userId) {
        return rtcRoomDAO.countByUserId(userId, RtcBusinessTypeEnum.RTC_SINGLE.getValue());
    }

    /**
     * 关闭个播直播
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public void stopSingleRtcLive(Long roomId, Long userId, String appVer) {

        stopSingleRtcLive(roomId, userId, appVer, "");
    }


    /**
     * 关闭个播直播
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public void stopSingleRtcLive(Long roomId, Long userId, String appVer, String reason) {

        // 设置当前场次结束
        RtcRoomLiveHisDO oldDo = rtcRoomLiveHisService.getCurrentRound(roomId);

        if (oldDo == null) {
            return;
        }

        // 将rtc房间状态改为关播
        // 同步es搜索信息
        RtcRoomEditDTO editDTO = new RtcRoomEditDTO();
        editDTO.setStatus(RtcRoomStatusEnum.CLOSE.getValue());
        editDTO.setRoomId(roomId);
        editRoom(editDTO);

        Long now = System.currentTimeMillis();
        RtcRoomLiveHisDO newDo = new RtcRoomLiveHisDO();
        newDo.setHisId(oldDo.getHisId());
        newDo.setLiveDuration(now - oldDo.getLiveBeginTime());
        newDo.setLiveEndTime(now);


        // 本场直播流水
        Date dateStart = new Date(oldDo.getCreateTime());
        Date dateEnd = new Date(now);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

        List<Integer> ROOM_STAT_TYPES = Arrays.asList(
                StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue(),
                StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND.getValue(),
                StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND.getValue()
        );
        List<StatRtcRoomVO> roomFlowlist = statRtcRoomDAO.selectRoomStats(roomId, Convert.toLong(sdf.format(dateStart)), Convert.toLong(sdf.format(dateEnd)), StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE_SECOND, ROOM_STAT_TYPES, null, null, 1);
        newDo.setFlow(roomFlowlist.stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());

        // 本场直播直刷流水
        newDo.setDirectFlow(roomFlowlist.stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND.getValue() || e.getStatType() == StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());

        // 主播粉钻收益
        List<SendGiftSnapshotDO> list = sendGiftSnapshotService.getSingleRtcRoomRoundList(roomId, oldDo.getCreateTime(), null);
        Long totalValue = 0L;
        for (int i = 0; i < list.size(); i++) {
            SendGiftSnapshotDO val = list.get(i);
            if (val.getActivityLotteryHisId() != null && StringUtils.isNotBlank(val.getActivityLotteryHisId())) {
                List<Long> orderIds = Arrays.stream(val.getActivityLotteryHisId().split(",")).map(Long::parseLong).collect(Collectors.toList());
                // 将钱包流水中orderId的查出来，筛选出主持人自己的流水

                totalValue += getUserWalletHisByOrderIds(orderIds, userId, WalletTypeEnum.STAR_DUST.getValue());
            } else {
                // 直接拿orderId从钱包流水查询，筛选出主持人自己的流水
                totalValue += getUserWalletHisByOrderIds(Arrays.asList(val.getOrderId()), userId, WalletTypeEnum.STAR_DUST.getValue());
            }
        }
        newDo.setPinkDiamondFlow(totalValue);

        // 本场送礼人数
        newDo.setSendGiftPeople(Optional.ofNullable(sendGiftSnapshotService.getSingleRtcRoomRoundSendGiftPeople(roomId, oldDo.getCreateTime())).orElse(0L));

        // 本场收藏房间人数
        newDo.setFavoritePeople((long) Optional.ofNullable(rtcRoomFavDAO.countDistinctById(roomId, oldDo.getCreateTime())).orElse(0));

        StatBarrageRsp rsp = getStatBarrageRspList(roomId, oldDo.getHisId());
        // 本场发弹幕条数
        newDo.setSpeakNum(Optional.ofNullable(rsp).orElse(new StatBarrageRsp()).getBarrage());
        // 本场发弹幕人数
        newDo.setSpeakPeople(Optional.ofNullable(rsp).orElse(new StatBarrageRsp()).getPeople());

        Long stayThreeMinutesPeople = redisService.zIncrementScore(RedisRtcKeys.getRtcRoomStayThreeMinutes(roomId), Convert.toStr(roomId), 1).longValue();
        // 本场停留3分钟以上的人数
        newDo.setStayThreeMinutesPeople(stayThreeMinutesPeople);
        // 本场观看人数
//        newDo.setWatchPeople((long)rtcUserService.getRtcRoomJoinUsersBefore(roomId, oldDo.getCreateTime()));
        newDo.setWatchPeople(rtcPopularityProvider.get(roomId));

        rtcRoomLiveHisService.updateById(newDo);

        redisService.delKey(RedisRtcKeys.getRtcRoomStayThreeMinutes(roomId));

        // 查看所有人是否在麦位，如果有人还在麦位，就踢掉
        rtcSeatProvider.leaveAllAsync(roomId, appVer);

        SingleRtcLiveStatusMsg statusMsg = new SingleRtcLiveStatusMsg();
        statusMsg.setStatus(BoolIntVal.False);
        statusMsg.setRoomId(roomId);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, statusMsg);

        //通用的关播 消息，迭代版本后 SingleRtcLiveStatusMsg 可以去除
        RtcLiveStatusMsg normalStatusMsg = new RtcLiveStatusMsg();
        normalStatusMsg.setStatus(BoolIntVal.False);
        normalStatusMsg.setRoomId(roomId);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, normalStatusMsg);

        log.info("个播下播: oldDo:{},newDo:{}, reason:{}", JSON.toJSONString(oldDo), JSON.toJSONString(newDo), reason);
    }

    private Long getUserWalletHisByOrderIds(List<Long> orderIds, Long userId, Integer walletType) {

        List<UserWalletHisDO> list = walletService.getUserWalletHisByOrderIds(orderIds, userId, walletType);

        if (list == null) {
            return 0L;
        }

        return list.stream().mapToLong(UserWalletHisDO::getBalanceValue).sum();
    }

    /**
     * 个播房间弹幕统计列表
     *
     * @return
     */
    private StatBarrageRsp getStatBarrageRspList(Long roomId, Long sessionId) {
        String url = String.format("%s/%s", apiNotifyRongcloud, "v1/rongCloud/getPersonRoomBarrage");
        Map<String, Object> params = new HashMap<>();
        params.put("roomId", "" + roomId);
        params.put("sessionId", "" + sessionId);
        String rsp;
        if (isDev()) {
            log.info(LogUtil.devLog(String.format("个播房间弹幕统计列表 url=%s params=%s", url, JSON.toJSONString(params))));
        }
        try {
            rsp = HttpUtil.post(url, params);
            if (isDev()) {
                log.info("个播房间弹幕统计列表rsp={}", LogUtil.devLog(rsp));
            }
        } catch (Exception e) {
            log.error(String.format("个播房间弹幕统计列表失败 url=%s params=%s", url, JSON.toJSONString(params)), e);
            return null;
        }

        JSONObject rspJson = JSON.parseObject(rsp);
        // 1:success  0:error
        Integer code = rspJson.getInteger("code");
        if (code == null || code != 1) {
            // 返回结果错误
            log.error(String.format("个播房间弹幕统计列表返回结果错误code url=%s params=%s rsp=%s",
                    url, JSON.toJSONString(params), rsp));
            return null;
        }
        StatBarrageRsp data = rspJson.getObject("data", StatBarrageRsp.class);
        if (data == null) {
            log.error(String.format("个播房间弹幕统计列表返回结果错误data url=%s params=%s rsp=%s",
                    url, JSON.toJSONString(params), rsp));
            return null;
        }

        return data;
    }

    /**
     * 获取用户房间数量
     *
     * @param familyId
     * @return
     */
    @Override
    public int getRoomCountByFamilyId(Long familyId) {
        return rtcRoomDAO.countByFamilyId(familyId, RtcBusinessTypeEnum.RTC_NORMAL.getValue());
    }

    /**
     * 获取家族房间
     *
     * @param familyId
     * @return
     */
    @Override
    public List<Long> getRoomIdsByFamilyId(Long familyId) {
        return rtcRoomDAO.selectRoomIds(familyId);
    }

    /**
     * 获取指定用户指定类型的房间数量
     *
     * @param userId
     * @param roomType
     * @return
     */
    @Override
    public int getRoomCountByUserIdAndRoomType(Long userId, Integer roomType) {
        return rtcRoomDAO.countByUserIdAndRoomType(userId, roomType);
    }

    /**
     * 获取用户房间列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<RtcRoomListItem> getUserRoomList(Long userId) {
        List<RtcRoomListItem> list = rtcRoomRoleDAO.selectRooms(userId);
        if (CollectionUtils.isNotEmpty(list)) {
            // 写入热度值
            return appendRoomHeat(list);
        }
        return Collections.emptyList();
    }

    /**
     * 获取房间角色
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public int getRoomRole(Long roomId, Long userId) {
        return userService.isOfficial(userId) ? RtcRoomRole.OFFICIAL : Optional.ofNullable(rtcRoomRoleDAO.selectRole(roomId, userId)).orElse(RtcRoomRole.AUDIENCE);
    }

    /**
     * 获取房间锁密码
     *
     * @param roomId
     * @return
     */
    @Override
    public String getRoomLockPass(Long roomId) {
        return rtcRoomDAO.selectRoomLockPass(roomId);
    }

    /**
     * 获取房间信息
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcRoom getRoom(Long roomId) {
        RtcRoom room = rtcRoomDAO.selectById(roomId);
        if (room == null) {
            return null;
        }
        room.setRoomHeat(rtcHeatV2Provider.getHeat(roomId));
        return room;
    }

    /**
     * 获取房间类型信息
     *
     * @param roomId
     * @return
     */
    @Override
    public Integer getRoomBusinessType(Long roomId) {
        return rtcRoomDAO.selectRoomBusinessType(roomId);
    }

    /**
     * 房间是否为个播
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isSingleRtcRoom(Long roomId) {
        return rtcRoomDAO.isSingleRtcRoom(roomId) > 0;
    }

    /**
     * 获取用户当前个播房间信息
     *
     * @param userId
     * @return
     */
    @Override
    public RtcRoomDO getSingleRtcRoom(Long userId, Long familyId) {


        RtcRoomDO room = rtcRoomDAO.selectSingleRtcRoom(userId, familyId);
        if (room == null) {
            return null;
        }
        return room;
    }


    @Override
    public Map<Long, Long> getRoomHeatMap(List<Long> roomIds) {
        return rtcHeatV2Provider.getRoomHeatMap(roomIds);
    }

    /**
     * 编辑房间热度
     *
     * @param roomId
     * @param heatValue
     * @param duration  持续时间（单位：毫秒）
     * @return
     */
    @Override
    public boolean editRoomHeats(Long roomId, long heatValue, long duration) {
        return rtcHeatV2Provider.addHeatItem(roomId, heatValue, System.currentTimeMillis() + duration);
    }

    /**
     * 设置房间锁密码
     *
     * @param roomId
     * @param roomLockPass
     * @return
     */
    @Override
    public boolean setRoomLockPass(Long roomId, String roomLockPass) {
        int updated = rtcRoomDAO.updateRoomLockPass(roomId, roomLockPass);
        if (updated > 0) {
            RtcRoomLockedMsg roomLockedMsg = new RtcRoomLockedMsg();
            roomLockedMsg.setRoomId(roomId);
            roomLockedMsg.setLocked(roomLockPass == null ? BoolIntVal.False : BoolIntVal.True);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, roomLockedMsg);
        }
        // 锁定时将房间从匹配池移除
        if (StringUtils.isNotBlank(roomLockPass)) {
            rtcBattleRoomCrossProvider.removeFromMatchPool(roomId);
        } else {
            rtcBattleRoomCrossProvider.addToMatchPool(roomId);
        }
        return true;
    }

    /**
     * 编辑房间
     *
     * @param rtcRoomEditDTO
     * @return
     */
    @Override
    public boolean editRoom(RtcRoomEditDTO rtcRoomEditDTO) {
        rtcRoomDAO.update(rtcRoomEditDTO);

        QueueRtcRoomInfoESMsg roomESInfoMsg = new QueueRtcRoomInfoESMsg();
        BeanUtils.copyProperties(rtcRoomEditDTO, roomESInfoMsg);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_ROOM_INFO_ES_SYNCH, roomESInfoMsg.serialize());

        return true;
    }


    /**
     * 获取房间管理者
     *
     * @param roomId
     * @param roomRole
     * @return
     */
    @Override
    public List<RtcRoomUser> getRoomMangers(Long roomId, Integer roomRole) {
        List<RtcRoomUser> list = rtcRoomRoleDAO.selectAllUsers(roomId, roomRole);
        if (CollectionUtils.isNotEmpty(list)) {
            return list;
        }
        return Collections.emptyList();
    }

    /**
     * 获取房主 id
     *
     * @param roomId
     * @return
     */
    @Override
    public Long getRoomUserId(Long roomId) {
        return rtcRoomDAO.selectUserId(roomId);
    }

    /**
     * 添加房间管理
     *
     * @param rtcRoomRoleDO
     * @return
     */
    @Override
    public boolean addRoomRole(RtcRoomRoleDO rtcRoomRoleDO) {
        int updated = rtcRoomRoleDAO.insertOrUpdate(rtcRoomRoleDO);
        if (updated > 0) {
            // 发送消息
            RtcRoleChangedMsg roleChangedMsg = new RtcRoleChangedMsg();
            roleChangedMsg.setRoomId(rtcRoomRoleDO.getRoomId());
            roleChangedMsg.setUserId(rtcRoomRoleDO.getUserId());
            roleChangedMsg.setRoomRole(rtcRoomRoleDO.getRoomRole());
            roleChangedMsg.setAction(BoolIntVal.True);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, rtcRoomRoleDO.getUserId(), null, rtcRoomRoleDO.getRoomId(), roleChangedMsg);
        }
        return true;
    }

    /**
     * 移除房间管理
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean removeRoomRole(Long roomId, Long userId) {
        // 查询原角色
        int userRole = getRoomRole(roomId, userId);

        int updated = rtcRoomRoleDAO.delete(roomId, userId);
        if (updated > 0) {
            // 发送消息
            RtcRoleChangedMsg roleChangedMsg = new RtcRoleChangedMsg();
            roleChangedMsg.setRoomId(roomId);
            roleChangedMsg.setUserId(userId);
            roleChangedMsg.setRoomRole(userRole);
            roleChangedMsg.setAction(BoolIntVal.False);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, roleChangedMsg);
        }
        return true;
    }

    /**
     * 获取阿里 RTC 频道验证信息
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public AliRtcAuthInfo getAliAuthInfo(Long roomId, Long userId) {
        return aliRtcProvider.getAuthInfo(roomId, userId);
    }

    /**
     * 获取声网 RTC 频道验证信息
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public RtcAuthInfo getAgoraAuthInfo(Long roomId, Long userId, String appVer) {
        return voiceChatService.getAuthInfo(roomId, userId, appVer);
    }

    /**
     * 房间是否自由上麦
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isRoomMicFree(Long roomId) {
        return Optional.ofNullable(rtcRoomDAO.selectRoomMicFree(roomId)).map(t -> t == CommonStatus.Normal).orElse(false);
    }

    /**
     * 心动值是否开启
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isRoomScoreEnabled(Long roomId) {
        return Optional.ofNullable(rtcRoomDAO.selectRoomScoreEnabled(roomId)).map(t -> t == CommonStatus.Normal).orElse(false);
    }

    /**
     * 是否开启公屏聊天
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isRoomChatEnabled(Long roomId) {
        return Optional.ofNullable(rtcRoomDAO.selectRoomChatEnabled(roomId)).map(t -> t == CommonStatus.Normal).orElse(true);
    }

    /**
     * 是否小黑屋用户
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean isBlackListUser(Long roomId, Long userId) {
        RtcRoomBlackDO blackDO = rtcRoomBlackDAO.selectById(roomId, userId);
        if (blackDO == null || blackDO.getStatus() != CommonStatus.Normal) {
            return false;
        }
        if (blackDO.getDuration() != null && blackDO.getDuration() == -1) {
            return true;
        }
        return System.currentTimeMillis() < blackDO.getUpdateTime() + blackDO.getDuration();
    }

    /**
     * 是否禁言用户
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean isForbiddenListUser(Long roomId, Long userId) {
        RtcRoomForbiddenDO hisDO = rtcRoomForbiddenDAO.selectById(roomId, userId);
        if (hisDO == null || hisDO.getStatus() != CommonStatus.Normal) {
            return false;
        }
        if (hisDO.getDuration() != null && hisDO.getDuration() == -1) {
            return true;
        }
        return System.currentTimeMillis() < hisDO.getUpdateTime() + hisDO.getDuration();
    }

    /**
     * 收藏房间
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean addRoomFav(Long roomId, Long userId) {
        Long curTime = System.currentTimeMillis();
        RtcRoomFavDO rtcRoomFav = new RtcRoomFavDO();
        rtcRoomFav.setRoomId(roomId);
        rtcRoomFav.setUserId(userId);
        rtcRoomFav.setStatus(CommonStatus.Normal);
        rtcRoomFav.setCreateTime(curTime);
        rtcRoomFav.setUpdateTime(curTime);
        rtcRoomFavDAO.insertOrUpdate(rtcRoomFav);
        return true;
    }

    /**
     * 取消收藏
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean removeRoomFav(Long roomId, Long userId) {
        rtcRoomFavDAO.updateStatus(roomId, userId, CommonStatus.Deleted, System.currentTimeMillis());
        return true;
    }

    /**
     * 根据房间类型获取列表
     *
     * @param pageIndex
     * @param pageSize
     * @param roomType
     * @return
     */
    @Override
    public List<RtcRoomListItem> getRoomListByType(int pageIndex, int pageSize, long roomType) {
        if (pageIndex < 1) {
            pageIndex = 1;
        }
        int pageCursor = (pageIndex - 1) * pageSize;
        return rtcRoomDAO.selectRoomListByType(pageCursor, pageSize, roomType);
    }

    /**
     * 获取房间列表
     *
     * @param pageIndex
     * @param pageSize
     * @param categoryId
     * @return
     */
    @Override
    public PageList<RtcRoomListItem> getRoomList(int pageIndex, int pageSize, Long categoryId) {
        return getRoomList(pageIndex, pageSize, categoryId, null);
    }

    /**
     * 获取房间列表
     *
     * @param pageIndex
     * @param pageSize
     * @param categoryId
     * @return
     */
    @Override
    public PageList<RtcRoomListItem> getRoomList(int pageIndex, int pageSize, Long categoryId, Integer businessType) {
        PageList<RtcRoomListItem> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(appendRoomHeat(rtcRoomDAO.selectListItems(pageList.getPageCursor(), pageSize, categoryId, businessType)));
        pageList.setTotal(rtcRoomDAO.count(categoryId));
        return pageList;
    }

    /**
     * 获取房间列表
     *
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<RtcRoomListItem> getMakeFriendRoomList(int pageIndex, int pageSize) {
        PageList<RtcRoomListItem> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(appendRoomHeat(rtcRoomDAO.selectMakeFriendRoomList(pageList.getPageCursor(), pageSize)));
        pageList.setTotal(rtcRoomDAO.countMakeFriendRoomList());
        return pageList;
    }

    /**
     * 获取收藏房间列表
     *
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @return
     */
    @Override
    public PageList<RtcRoomListItem> getFavRoomList(int pageIndex, int pageSize, Long userId) {
        PageList<RtcRoomListItem> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(appendRoomHeat(rtcRoomFavDAO.selectListItems(pageList.getPageCursor(), pageSize, userId)));
        pageList.setTotal(rtcRoomFavDAO.countByUserId(userId));
        return pageList;
    }

    /**
     * 判断是否收藏的房间
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean isRoomFavorite(Long roomId, Long userId) {
        return rtcRoomFavDAO.countById(roomId, userId) > 0;
    }

    /**
     * 设置自由上麦
     *
     * @param roomId
     * @param micFree
     * @return
     */
    @Override
    public boolean setMicFree(Long roomId, int micFree) {
        int updated = rtcRoomDAO.updateMicFree(roomId, micFree);
        if (updated > 0) {
            // 更新成功
            if (BoolIntVal.True == micFree) {
                rtcQueueProvider.clear(roomId);
            }

            // 发送消息
            RtcMicFreeChangedMsg micFreeChangedMsg = new RtcMicFreeChangedMsg();
            micFreeChangedMsg.setRoomId(roomId);
            micFreeChangedMsg.setMicFree(micFree);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, micFreeChangedMsg);

            return true;
        }
        return false;
    }

    /**
     * 设置心动值开关
     *
     * @param roomId
     * @param scoreEnabled
     * @return
     */
    @Override
    public boolean setScoreEnabled(Long roomId, int scoreEnabled) {
        int updated = rtcRoomDAO.updateScoreEnabled(roomId, scoreEnabled);
        if (updated > 0) {

            // 发送消息
            RtcScoreEnabledChangedMsg scoreEnabledChangedMsg = new RtcScoreEnabledChangedMsg();
            scoreEnabledChangedMsg.setRoomId(roomId);
            scoreEnabledChangedMsg.setScoreEnabled(scoreEnabled);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, scoreEnabledChangedMsg);

            return true;
        }
        return false;
    }

    /**
     * 设置公屏开关
     *
     * @param roomId
     * @param chatEnabled
     * @return
     */
    @Override
    public boolean setChatEnabled(Long roomId, int chatEnabled) {
        int updated = rtcRoomDAO.updateChatEnabled(roomId, chatEnabled);
        if (updated > 0) {
            // 更新成功
            RtcChatEnabledChangedMsg chatEnabledChangedMsg = new RtcChatEnabledChangedMsg();
            chatEnabledChangedMsg.setRoomId(roomId);
            chatEnabledChangedMsg.setChatEnabled(chatEnabled);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, chatEnabledChangedMsg);
            return true;
        }
        return false;
    }

    /**
     * 设置大冒险开关
     *
     * @param roomId
     * @param bigAdventureEnabled
     * @return
     */
    @Override
    public boolean setBigAdventureEnabled(Long roomId, int bigAdventureEnabled) {
        int updated = rtcRoomDAO.updateBigAdventureEnabled(roomId, bigAdventureEnabled);
        if (updated > 0) {
            // 更新成功
            RtcBigAdventureEnabledChangedMsg bigAdventureEnabledChangedMsg = new RtcBigAdventureEnabledChangedMsg();
            bigAdventureEnabledChangedMsg.setRoomId(roomId);
            bigAdventureEnabledChangedMsg.setBigAdventureEnabled(bigAdventureEnabled);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, bigAdventureEnabledChangedMsg);
            return true;
        }
        return false;
    }

    /**
     * 设置房间背景
     *
     * @param roomId
     * @param backgroundId
     * @return
     */
    @Override
    public boolean setRoomBackground(Long roomId, Integer backgroundId) {
        RtcRoomEditDTO rtcRoomEditDTO = new RtcRoomEditDTO();
        rtcRoomEditDTO.setRoomId(roomId);
        rtcRoomEditDTO.setBackgroundId(backgroundId);
        int updated = rtcRoomDAO.update(rtcRoomEditDTO);
        if (updated > 0) {
            RtcRoomBackground roomBackground = dataService.getRtcRoomBackground(backgroundId);
            // 发送消息
            RtcBackgroundChangedMsg backgroundChangedMsg = new RtcBackgroundChangedMsg();
            backgroundChangedMsg.setRoomId(roomId);
            backgroundChangedMsg.setBackgroundId(backgroundId);
            backgroundChangedMsg.setBackgroundName(roomBackground.getBackgroundName());
            backgroundChangedMsg.setBackgroundUrl(roomBackground.getBackgroundUrl());
            backgroundChangedMsg.setDynamicFile(roomBackground.getDynamicFile());
            backgroundChangedMsg.setBackgroundIsDynamic(roomBackground.getBackgroundIsDynamic());
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, backgroundChangedMsg);
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setForbidden(Long curUserId, Long roomId, Long userId, Long duration, String remarks, Integer status) {
        RtcRoomForbiddenDO hisDO = new RtcRoomForbiddenDO();
        hisDO.setUpdateTime(System.currentTimeMillis());
        hisDO.setDuration(duration);
        hisDO.setMasterUserId(curUserId);
        hisDO.setRoomId(roomId);
        hisDO.setTargetUserId(userId);
        hisDO.setRemarks(remarks);
        hisDO.setStatus(status);
        int count = rtcRoomForbiddenDAO.insertOrUpdate(hisDO);
        if (count > 0) {
            if (status == CommonStatus.Normal) {
                // 之前未被禁言，直播间通报禁言消息
                LiveUserForbiddenMsg liveUserForbiddenMsg = new LiveUserForbiddenMsg();
                liveUserForbiddenMsg.setDuration(duration);
                imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, liveUserForbiddenMsg);
            }
            return true;
        }
        return false;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setBlack(Long roomId, Long userId, Integer status, Long duration, String blackRemarks, Long channelId, String appVer) {
        return setBlack(roomId, userId, status, duration, blackRemarks, channelId, null, appVer);
    }

    /**
     * 设置黑名单
     *
     * @param roomId
     * @param userId
     * @param status
     * @param duration
     * @param blackRemarks
     * @param channelId
     * @param masterId
     * @return
     */
    @Override
    public boolean setBlack(Long roomId, Long userId, Integer status, Long duration, String blackRemarks, Long channelId, Long masterId, String appVer) {
        RtcRoomBlackDO blackDO = new RtcRoomBlackDO();
        blackDO.setRoomId(roomId);
        blackDO.setUserId(userId);
        blackDO.setStatus(status);
        blackDO.setDuration(duration);
        blackDO.setBlackRemarks(blackRemarks);
        blackDO.setUpdateTime(System.currentTimeMillis());

        int count = rtcRoomBlackDAO.insertOrUpdate(blackDO);
        if (count > 0) {
            if (status == CommonStatus.Normal) {

                // 判断是否在麦上
                if (rtcSeatProvider.getUserSeatNo(roomId, userId) > -1) {
                    // 强制下麦
                    rtcSeatService.leaveSeat(roomId, userId, true, appVer);
                }

                // 发送拉黑消息
                LiveUserBlackMsg msg = new LiveUserBlackMsg();
                if (masterId != null) {
                    String masterUserName = userService.getUserName(masterId);
                    if (StringUtils.isNotEmpty(masterUserName)) {
                        msg.setMasterUserId(masterId);
                        msg.setMasterUserName(masterUserName);
                    }
                }
                imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, msg);
            }
            return true;
        }
        return false;
    }

    @Override
    public PageList<RtcRoomForbidden> getForbiddenList(int pageIndex, int pageSize, Long roomId) {
        PageList<RtcRoomForbidden> page = new PageList<>(pageIndex, pageSize);
        page.setList(rtcRoomForbiddenDAO.selectForbiddenList(page.getPageCursor(), pageSize, roomId));
        page.setTotal(rtcRoomForbiddenDAO.countForbiddenList(roomId));
        return page;
    }

    @Override
    public PageList<RtcRoomBlack> getBlackList(int pageIndex, int pageSize, Long roomId, Long userNo) {
        PageList<RtcRoomBlack> page = new PageList<>(pageIndex, pageSize);
        page.setList(rtcRoomBlackDAO.selectBlackList(page.getPageCursor(), pageSize, roomId, userNo));
        page.setTotal(rtcRoomBlackDAO.countBlackList(roomId, userNo));
        return page;
    }

    @Override
    public boolean isRoomMasterLimitAtLeast(Long roomId, Long userId) {
        return rtcRoomRoleDAO.countByIdAndUserAndRole(roomId, userId, RtcRoomRole.MASTER) > 0;
    }

    @Override
    public int getMasterNum(Long roomId) {
        return rtcRoomRoleDAO.countByIdAndRole(roomId, RtcRoomRole.MASTER);
    }

    @Override
    public int getHostNum(Long roomId) {
        return rtcRoomRoleDAO.countByIdAndRole(roomId, RtcRoomRole.HOST);
    }

    @Override
    public int getForbiddenNum(Long roomId) {
        return rtcRoomForbiddenDAO.countForbiddenList(roomId);
    }

    @Override
    public int getBlackNum(Long roomId) {
        return rtcRoomBlackDAO.countBlackList(roomId, null);
    }

    /**
     * 获取房间类型
     *
     * @param roomId
     * @return
     */
    @Override
    public Integer getRoomType(Long roomId) {
        return rtcRoomDAO.selectRoomType(roomId);
    }

    /**
     * 获取热门房间列表
     *
     * @return
     */
    @Override
    public List<RtcRoomListItem> getHotRoomListRealtime(int num) {
        List<RtcRoomListItem> listItems = new ArrayList<>();
        int start = 0;
        int needNum = num;
        while (listItems.size() < num) {
            List<RtcRoomHeat> list = rtcHeatV2Provider.getTopN(start, needNum);
            if (CollectionUtil.isNotEmpty(list)) {
                Map<Long, Long> roomHeatMap = list.stream().collect(Collectors.toMap(RtcRoomHeat::getRoomId, RtcRoomHeat::getHeatValue));
                List<Long> roomIds = list.stream().map(RtcRoomHeat::getRoomId).collect(Collectors.toList());

                // 查询过滤 id
                List<RtcRoomListItem> hotList = rtcRoomDAO.selectHotEnableRooms(roomIds);

                // 热度处理
                boolean isAllBreak = false;
                for (RtcRoomListItem listItem : hotList) {
                    Long roomHeat = roomHeatMap.get(listItem.getRoomId());
                    if (roomHeat == 0) {
                        isAllBreak = true;
                        break;
                    }
                    listItem.setRoomHeat(roomHeatMap.get(listItem.getRoomId()));
                    listItems.add(listItem);
                }

                // 如果没有热门数量了
                if (isAllBreak || list.size() < needNum) {
                    break;
                }

                start += needNum;
                needNum -= hotList.size();
            } else {
                break;
            }
        }

        if (CollectionUtil.isNotEmpty(listItems)) {
            // 最终排序
            listItems.sort(Comparator.comparingLong(RtcRoomListItem::getRoomHeat).reversed());
        }
        return listItems;
    }

    /**
     * 获取热门房间列表
     *
     * @param num
     * @return
     */
    @Override
    public List<RtcRoomListItem> getHotRoomList(int num, Integer businessType) {
        List<RtcRoomListItem> list = rtcRoomDAO.selectHotList(num, businessType);
        return appendRoomHeat(list);
    }

    /**
     * 获取热门房间列表
     *
     * @param num
     * @param businessType
     * @param winner
     * @return
     */
    @Override
    public List<RtcRoomListItem> getHotRoomList(int num, Integer businessType, RtcRankItem winner) {
        List<RtcRoomListItem> res = rtcRoomDAO.selectHotList(num, businessType);
        // 先移除, 然后添加到首位
        if (winner != null) {
            res.removeIf(i -> i.getRoomId().equals(winner.getRoomId()));
            RtcRoomListItem winnerItem = rtcRoomDAO.getRoomById(winner.getRoomId());
            res.add(0, winnerItem);
            if (res.size() > num) {
                res = res.subList(0, num);
            }
        }
        return appendRoomHeat(res);
    }

    /**
     * 推荐房间列表需要的房间信息
     *
     * @param roomIds
     * @param noPass  true:不包含上锁的房间
     * @return key:RoomId
     */
    @Override
    public Map<Long, RtcRoomSuggestItem> getSuggestMap(Collection roomIds, boolean noPass) {
        if (CollectionUtils.isEmpty(roomIds)) {
            return Collections.emptyMap();
        }
        return rtcRoomDAO.selectSuggestList(roomIds, noPass, CommonStatus.Normal)
                .stream()
                .filter(Objects::nonNull)
                .filter(item -> item.getRoomId() != null)
                .collect(Collectors.toMap(RtcRoomSuggestItem::getRoomId, Function.identity()));
    }

    /**
     * 获取用户所在语聊房
     *
     * @param userId
     * @return
     */
    @Override
    public RtcRoomListItem getUserStayRoom(Long userId) {
        Long roomId = rtcUserService.getUserStayRoomId(userId);
        if (roomId > 0) {
            List<RtcRoomListItem> listItems = rtcRoomDAO.selectByIds(Collections.singletonList(roomId));
            if (CollectionUtil.isNotEmpty(listItems)) {
                RtcRoomListItem listItem = listItems.get(0);
                listItem.setRoomBorder(activityAnniversaryService.getRtcRoomBorder(roomId));
                listItem.setRoomRole(getRoomRole(roomId, userId));
                return listItem;
            }
        }
        return null;
    }

    /**
     * 获取个播主持人在线状态
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isSingleRtcRoomUserOnline(Long userId) {
        // 不在线
        if (!userService.isOnline(userId)) {
            log.info("个播获取主持人在线状态: isOnline:{},userId:{}", false, userId);
            // hack个播在线状态常常不准，融云回调状态异常
            return true;
        }

        Long familyId = familyService.getFamilyId(userId);
        if (familyId == null) {
            log.info("个播获取主持人在线状态: familyId:{},userId:{}", familyId, userId);
            return false;
        }

        RtcRoomDO singleRtcRoom = rtcRoomDAO.selectSingleRtcRoom(userId, familyId);
        if (singleRtcRoom == null) {
            log.info("个播获取主持人在线状态: singleRtcRoom:不存在,userId:{},familyId:{}", userId, familyId);
            return false;
        }

        // 判断是否在自己个播房间
        Long roomId = rtcUserService.getUserStayRoomId(userId);
        if (roomId == null) {
            log.info("个播获取主持人在线状态: RTC_USER_ROOM:redis:不存在,userId:{},roomId:{}", userId, roomId);
            return false;
        }

        if (roomId.equals(singleRtcRoom.getRoomId())) {
            return true;
        }

        log.info("个播获取主持人在线状态: 不在自己房间:singleRtcRoom:{},userId:{},roomId:{},", singleRtcRoom.getRoomId(), userId, roomId);
        // hack个播在线状态常常不准，融云回调状态异常
        return true;
    }

    /**
     * 获取表情列表
     *
     * @param roomId
     * @return
     */
    @Override
    public List<FacialExpression> getFacialExpressionList(Long roomId) {
        return facialExpressionDAO.selectAll();
    }

    /**
     * 判断表情是否存在
     *
     * @param roomId
     * @param expId
     * @return
     */
    @Override
    public boolean isFacialExpressionExists(Long roomId, Integer expId) {
        return facialExpressionDAO.countById(expId) > 0;
    }

    private void loadRtcRoomData(Map.Entry<Long, List<StatRtcRoomVO>> entry, BaseRtcRoomStream item) {
        item.setStatValue(entry.getValue().stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setFamilyUserIncome(entry.getValue().stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.RECEIVE_STAR_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setNormalUserIncome(item.getStatValue() - item.getFamilyUserIncome());

        item.setNormalGiftIncome(entry.getValue().stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setLotteryIncome(entry.getValue().stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setBagIncome(entry.getValue().stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setFamilyUserNormalGiftIncome(entry.getValue().stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setFamilyUserLotteryIncome(entry.getValue().stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setFamilyUserBagIncome(entry.getValue().stream().filter(e -> e.getStatType() == StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());

        item.setNormalUserNormalGiftIncome(item.getNormalGiftIncome() - item.getFamilyUserNormalGiftIncome());
        item.setNormalUserLotteryIncome(item.getLotteryIncome() - item.getFamilyUserLotteryIncome());
        item.setNormalUserBagIncome(item.getBagIncome() - item.getFamilyUserBagIncome());
    }

    private void loadRtcRoomDataWithStatTime(Map.Entry<Long, List<StatRtcRoomVO>> entry, BaseRtcRoomStream item) {
        item.setStatValue(entry.getValue().stream().filter(e -> e.getStatTime().equals(entry.getKey()) && e.getStatType() == StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setFamilyUserIncome(entry.getValue().stream().filter(e -> e.getStatTime().equals(entry.getKey()) && e.getStatType() == StatRtcRoomTypeEnum.RECEIVE_STAR_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setNormalUserIncome(item.getStatValue() - item.getFamilyUserIncome());

        item.setNormalGiftIncome(entry.getValue().stream().filter(e -> e.getStatTime().equals(entry.getKey()) && e.getStatType() == StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setLotteryIncome(entry.getValue().stream().filter(e -> e.getStatTime().equals(entry.getKey()) && e.getStatType() == StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setBagIncome(entry.getValue().stream().filter(e -> e.getStatTime().equals(entry.getKey()) && e.getStatType() == StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setFamilyUserNormalGiftIncome(entry.getValue().stream().filter(e -> e.getStatTime().equals(entry.getKey()) && e.getStatType() == StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setFamilyUserLotteryIncome(entry.getValue().stream().filter(e -> e.getStatTime().equals(entry.getKey()) && e.getStatType() == StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
        item.setFamilyUserBagIncome(entry.getValue().stream().filter(e -> e.getStatTime().equals(entry.getKey()) && e.getStatType() == StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());

        item.setNormalUserNormalGiftIncome(item.getNormalGiftIncome() - item.getFamilyUserNormalGiftIncome());
        item.setNormalUserLotteryIncome(item.getLotteryIncome() - item.getFamilyUserLotteryIncome());
        item.setNormalUserBagIncome(item.getBagIncome() - item.getFamilyUserBagIncome());
    }

    /**
     * 获取房间日流水
     *
     * @param roomId
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public RtcRoomDayStream getRtcRoomDayStream(Long roomId, Long startTime, Long endTime) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        RtcRoomDayStream dayStream = new RtcRoomDayStream();

        List<RtcRoomDayStream.Item> items = new ArrayList<>();
        List<Integer> statTypes = ROOM_STAT_TYPES;
        List<StatRtcRoomVO> list = statRtcRoomDAO.selectRoomStats(roomId, Convert.toLong(sdf.format(startTime)), Convert.toLong(sdf.format(endTime)), StatTimeType.YEAR_MONTH_DAY, statTypes, null, null, null);

        Map<Long, List<StatRtcRoomVO>> listMap = list.stream().collect(Collectors.groupingBy(StatRtcRoomVO::getStatTime));

        for (Map.Entry<Long, List<StatRtcRoomVO>> entry : listMap.entrySet()) {
            RtcRoomDayStream.Item item = new RtcRoomDayStream.Item();
            item.setDay(Convert.toInt(entry.getKey()));

            loadRtcRoomDataWithStatTime(entry, item);

            item.setFamilyId(entry.getValue().get(0).getFamilyId());
            item.setFamilyName(entry.getValue().get(0).getFamilyName());
            items.add(item);
        }
        dayStream.setTotal(items.stream().mapToLong(BaseRtcRoomStream::getStatValue).sum());
        dayStream.setFamilyUserTotalIncome(items.stream().mapToLong(BaseRtcRoomStream::getFamilyUserIncome).sum());
        dayStream.setList(items);

        return dayStream;
    }

    /**
     * 获取房间小时流水
     *
     * @param roomId
     * @param date
     * @return
     */
    @Override
    public RtcRoomHourStream getRtcRoomHourStream(Long roomId, Long date) {
        RtcRoomHourStream hourStream = new RtcRoomHourStream();

        List<RtcRoomHourStream.Item> items = new ArrayList<>();
        List<Integer> statTypes = ROOM_STAT_TYPES;
        List<StatRtcRoomVO> list = statRtcRoomDAO.selectRoomStatsForHour(roomId, date, StatTimeType.YEAR_MONTH_DAY_HOUR, statTypes, null, null);

        Map<Long, List<StatRtcRoomVO>> statTimeMap = list.stream().collect(Collectors.groupingBy(StatRtcRoomVO::getStatTime));
        for (Map.Entry<Long, List<StatRtcRoomVO>> entry : statTimeMap.entrySet()) {
            RtcRoomHourStream.Item item = new RtcRoomHourStream.Item();
            item.setDay(Convert.toInt(entry.getKey()));

            loadRtcRoomDataWithStatTime(entry, item);

            item.setFamilyId(entry.getValue().get(0).getFamilyId());
            item.setFamilyName(entry.getValue().get(0).getFamilyName());
            items.add(item);
        }

        hourStream.setTotal(items.stream().mapToLong(BaseRtcRoomStream::getStatValue).sum());
        hourStream.setFamilyUserTotalIncome(items.stream().mapToLong(BaseRtcRoomStream::getFamilyUserIncome).sum());
        hourStream.setList(items);

        return hourStream;
    }

    /**
     * 获取房间周流水
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcRoomWeekStream getRtcRoomWeekStream(Long roomId) {
        RtcRoomWeekStream weekStream = new RtcRoomWeekStream();

        Date date = new Date();
        Map<Integer, List<Long>> weekOfYearMap = getWeekOfYearMap(date);

        List<Long> dateList = new ArrayList<>();
        for (List<Long> item : weekOfYearMap.values()) {
            dateList.addAll(item);
        }

        Long maxDay = Collections.max(dateList);
        Long minDay = Collections.min(dateList);

        List<Integer> statTypes = ROOM_STAT_TYPES;

        List<StatRtcRoomVO> list = statRtcRoomDAO.selectRoomStats(roomId, minDay, maxDay, StatTimeType.YEAR_MONTH_DAY, statTypes, null, null, null);

        if (CollectionUtil.isEmpty(list)) {
            weekStream.setList(new ArrayList<>());
            return weekStream;
        }

        List<RtcRoomWeekStream.Item> items = new ArrayList<>();
        for (Map.Entry<Integer, List<Long>> entry : weekOfYearMap.entrySet()) {
            List<Long> dateRange = entry.getValue();

            Long startDate = dateRange.get(0);
            Long endDate = dateRange.get(dateRange.size() - 1);

            Long statValue = Convert.toLong(list.stream().filter(e -> startDate <= e.getStatTime() && e.getStatTime() <= endDate && e.getStatType() == StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue()).mapToDouble(e -> Convert.toDouble(e.getStatValue())).sum());
            Long familyUserIncome = Convert.toLong(list.stream().filter(e -> startDate <= e.getStatTime() && e.getStatTime() <= endDate && e.getStatType() == StatRtcRoomTypeEnum.RECEIVE_STAR_DIAMOND_FAMILY_USER.getValue()).mapToDouble(e -> Convert.toDouble(e.getStatValue())).sum());
            if (statValue > 0) {
                RtcRoomWeekStream.Item item = new RtcRoomWeekStream.Item();
                item.setWeekOfYear(entry.getKey());
                item.setStartDate(startDate);
                item.setEndDate(endDate);
                item.setStatValue(statValue);
                item.setFamilyUserIncome(familyUserIncome);

                item.setNormalUserIncome(item.getStatValue() - item.getFamilyUserIncome());

                item.setNormalGiftIncome(list.stream().filter(e -> startDate <= e.getStatTime() && e.getStatTime() <= endDate && e.getStatType() == StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
                item.setLotteryIncome(list.stream().filter(e -> startDate <= e.getStatTime() && e.getStatTime() <= endDate && e.getStatType() == StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
                item.setBagIncome(list.stream().filter(e -> startDate <= e.getStatTime() && e.getStatTime() <= endDate && e.getStatType() == StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
                item.setFamilyUserNormalGiftIncome(list.stream().filter(e -> startDate <= e.getStatTime() && e.getStatTime() <= endDate && e.getStatType() == StatRtcRoomTypeEnum.NORMAL_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
                item.setFamilyUserLotteryIncome(list.stream().filter(e -> startDate <= e.getStatTime() && e.getStatTime() <= endDate && e.getStatType() == StatRtcRoomTypeEnum.LOTTERY_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());
                item.setFamilyUserBagIncome(list.stream().filter(e -> startDate <= e.getStatTime() && e.getStatTime() <= endDate && e.getStatType() == StatRtcRoomTypeEnum.BAG_GIFT_DIAMOND_FAMILY_USER.getValue()).mapToLong(e -> Convert.toLong(e.getStatValue())).sum());

                item.setNormalUserNormalGiftIncome(item.getNormalGiftIncome() - item.getFamilyUserNormalGiftIncome());
                item.setNormalUserLotteryIncome(item.getLotteryIncome() - item.getFamilyUserLotteryIncome());
                item.setNormalUserBagIncome(item.getBagIncome() - item.getFamilyUserBagIncome());

                items.add(item);
            }
        }
        items.sort(Comparator.comparing(RtcRoomWeekStream.Item::getWeekOfYear).reversed());
        weekStream.setList(items);
        return weekStream;
    }

    /**
     * 获取房间月流水
     *
     * @param roomId
     * @return
     */
    @Override
    public RtcRoomMonthStream getRtcRoomMonthStream(Long roomId) {
        RtcRoomMonthStream monthStream = new RtcRoomMonthStream();
        try {
            List<Integer> statTypes = ROOM_STAT_TYPES;
            List<StatRtcRoomVO> list = statRtcRoomDAO.selectRoomStats(roomId, null, null, StatTimeType.YEAR_MONTH, statTypes, null, null, null);
            List<RtcRoomMonthStream.Item> items = new ArrayList<>();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
            Map<Long, List<StatRtcRoomVO>> listMap = list.stream().collect(Collectors.groupingBy(StatRtcRoomVO::getStatTime));
            for (Map.Entry<Long, List<StatRtcRoomVO>> entry : listMap.entrySet()) {
                RtcRoomMonthStream.Item item = new RtcRoomMonthStream.Item();
                StatRtcRoomVO roomVO = entry.getValue().get(0);
                item.setStatMonth(roomVO.getStatTime());
                loadRtcRoomDataWithStatTime(entry, item);
                item.setStartDate(Convert.toLong(dateFormat.format(getFirstDayDateOfMonth(sdf.parse(Convert.toStr(roomVO.getStatTime()))))));
                item.setEndDate(Convert.toLong(dateFormat.format(getLastDayDateOfMonth(sdf.parse(Convert.toStr(roomVO.getStatTime()))))));
                items.add(item);
            }
            monthStream.setList(items);
        } catch (Exception e) {
            log.info("出现异常", e);
        }
        return monthStream;
    }

    /**
     * 获取房间流水
     *
     * @param startTime
     * @param endTime
     * @param isFamily
     * @param familyId
     * @param liveType
     * @return
     */
    @Override
    public List<RtcRoomStreamItem> getRtcRoomStream(Long startTime, Long endTime, Integer isFamily, Long familyId, Integer liveType) {
        List<Integer> statTypes = ROOM_STAT_TYPES;
        List<StatRtcRoomVO> list = statRtcRoomDAO.selectRoomStats(null, startTime, endTime, StatTimeType.YEAR_MONTH_DAY, statTypes, isFamily, familyId, liveType);
        if (CollectionUtil.isEmpty(list)) {
            return Collections.EMPTY_LIST;
        }
        List<RtcRoomStreamItem> itemList = new ArrayList<>();
        // 根据房间分组
        Map<Long, List<StatRtcRoomVO>> statMap = list.stream().collect(Collectors.groupingBy(StatRtcRoomVO::getRoomId));

        //拿到房间用户信息
        List<Long> roomIDList = list.stream().map(o -> o.getRoomId()).collect(Collectors.toList());
        Map<Long, RtcRoomDO> roomUserIdMap = rtcRoomDAO.selectRoomOwnerMap(roomIDList);
        List<Long> userIDList = roomUserIdMap.values().stream().map(o -> o.getUserId()).collect(Collectors.toList());
        Map<Long, UserSimpleVO> roomOwnerSimpleMap = userService.getSimpleUserMap(userIDList);
        for (Map.Entry<Long, List<StatRtcRoomVO>> entry : statMap.entrySet()) {
            RtcRoomStreamItem item = new RtcRoomStreamItem();
            item.setRoomId(entry.getKey());
            item.setRoomName(entry.getValue().get(0).getRoomName());
            loadRtcRoomData(entry, item);
            item.setFamilyId(entry.getValue().get(0).getFamilyId());
            item.setFamilyName(entry.getValue().get(0).getFamilyName());

            RtcRoomDO roomDO = roomUserIdMap.get(item.getRoomId());
            UserSimpleVO userSimpleVO = roomOwnerSimpleMap.get(roomDO.getUserId());
            if (userSimpleVO != null) {
                item.setUserName(userSimpleVO.getUserName());
                item.setUserNo(userSimpleVO.getUserNo() + "");
            }

            itemList.add(item);
        }
        return itemList;
    }

    /**
     * 获取房间流水
     *
     * @param startTime
     * @param endTime
     * @param familyId
     * @return
     */
    @Override
    public List<RtcRoomStreamItem> getRtcRoomStream(Long startTime, Long endTime, Long familyId) {
        List<StatRtcRoomDO> list = statRtcRoomDAO.selectRoomStatByFamilyId(null, startTime, endTime, StatTimeType.YEAR_MONTH_DAY, StatRtcRoomTypeEnum.SEND_STAR_DIAMOND.getValue(), familyId);
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>();
        }

        List<RtcRoomStreamItem> itemList = new ArrayList<>();
        Map<Long, List<StatRtcRoomDO>> statMap = list.stream().collect(Collectors.groupingBy(StatRtcRoomDO::getRoomId));

        //拿到房间用户信息
        List<Long> roomIDList = list.stream().map(o -> o.getRoomId()).collect(Collectors.toList());
        Map<Long, RtcRoomDO> roomUserIdMap = rtcRoomDAO.selectRoomOwnerMap(roomIDList);
        List<Long> userIDList = roomUserIdMap.values().stream().map(o -> o.getUserId()).collect(Collectors.toList());
        Map<Long, UserSimpleVO> roomOwnerSimpleMap = userService.getSimpleUserMap(userIDList);
        for (Map.Entry<Long, List<StatRtcRoomDO>> entry : statMap.entrySet()) {
            RtcRoomStreamItem item = new RtcRoomStreamItem();
            item.setRoomId(entry.getKey());
            item.setRoomName(rtcRoomDAO.selectRoomName(entry.getKey()));
            List<StatRtcRoomDO> value = entry.getValue();
            item.setStatValue(value.stream().mapToLong(e -> Convert.toLong(e.getStatValue())).sum());

            RtcRoomDO roomDO = roomUserIdMap.get(item.getRoomId());
            UserSimpleVO userSimpleVO = roomOwnerSimpleMap.get(roomDO.getUserId());
            if (userSimpleVO != null) {
                item.setUserName(userSimpleVO.getUserName());
                item.setUserNo(userSimpleVO.getUserNo() + "");
            }

            itemList.add(item);
        }
        return itemList;
    }

    @Async("taskExecutor")
    @Override
    public void updateRoomHeat(List<RtcRoomHeat> heats) {
        if (CollectionUtil.isEmpty(heats)) {
            return;
        }
        for (RtcRoomHeat heat : heats) {
            rtcHeatV2Provider.setHeat(heat.getRoomId(), heat.getHeatValue());
        }
    }

    @Override
    public void conveneFans(Long roomId, String roomName, Integer roomType, Boolean isLock, Integer seatNo, Long userId) {
        int pageIndex = 1;
        while (true) {
            // 粉丝列表
            PageList<FansUser> pageList = socialRelationService.getFansList(userId, userId, pageIndex++, 50);
            List<FansUser> list = pageList.getList();
            if (CollectionUtils.isEmpty(list)) {
                break;
            }

            for (FansUser fans : list) {
                if (fans == null) {
                    continue;
                }
                Long fansUserId = fans.getUserId();
                if (fansUserId == null) {
                    continue;
                }
                RtcConveneFansMsg conveneMsg = new RtcConveneFansMsg();
                conveneMsg.setRoomId(roomId);
                conveneMsg.setRoomType(roomType);
                conveneMsg.setIsLock(isLock);
                conveneMsg.setSeatNo(seatNo);
                conveneMsg.setUserId(fansUserId);
                conveneMsg.setMsg(StrUtil.format("我正在{}房间，快来和我一起玩耍吧！！！", roomName));
                imService.sendCustomImMsg(userId, fansUserId, conveneMsg, null);
            }
        }
    }

    /**
     * 获取家族id
     *
     * @param roomId
     * @return
     */
    @Override
    public Long getFamilyId(Long roomId) {
        return rtcRoomDAO.selectFamilyId(roomId);
    }

    /**
     * 清空心动值
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean clearScore(Long roomId, Long userId) {
        rtcScoreProvider.clearScore(roomId, userId);
        boolean isLove = RoomTypeEnum.LOVE.getValue() == rtcRoomDAO.selectRoomType(roomId);
        // 相亲房模式下 要清空对应用户的心动值
        if (isLove) {
            rtcLoveScoreProvider.clearScore(roomId, userId);
        }
        return true;
    }

    /**
     * 清空所有心动值
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean clearAllScore(Long roomId) {
        rtcScoreProvider.clearAll(roomId);
        boolean isLove = RoomTypeEnum.LOVE.getValue() == rtcRoomDAO.selectRoomType(roomId);
        // 相亲房模式下 要清空对应用户的心动值
        if (isLove) {
            String rtcUserScoreKey = RedisRtcKeys.getRtcUserScore(roomId);
            Set<Long> userIdSet = redisService.hashKeys(rtcUserScoreKey).stream().map(Convert::toLong).collect(Collectors.toSet());
            for (Long userId : userIdSet) {
                rtcLoveScoreProvider.clearScore(roomId, Convert.toLong(userId));
            }
        }
        return true;
    }

    /**
     * 设置语聊房信息
     *
     * @param list
     */
    @Override
    public <T> T setRtcRoomInfo(List<? extends RtcRoomTag> list) {
        if (CollectionUtil.isEmpty(list)) {
            return (T) list;
        }
        Set<Long> userIds = list.stream().map(tag -> tag.getUserId()).collect(Collectors.toSet());
        // 用户房间Map userId:roomId
        Map<Long, Long> userRoomMap = rtcUserService.getUserRoomIdMap(userIds);
        list.stream()
                .filter(Objects::nonNull)
                .filter(rtc -> rtc.getIsOnline() == 1)
                .forEach(rtcRoomTag -> {
                    // 在线时查找房间id
                    rtcRoomTag.setRoomId(userRoomMap.get(rtcRoomTag.getUserId()));
                });
        return (T) list;
    }

    /**
     * 获取房间用户分成比例
     *
     * @param roomId
     */
    @Override
    public Double getUserRatio(Long roomId) {
        return rtcRoomDAO.selectUserRatio(roomId);
    }

    /**
     * 判断是否担任角色
     *
     * @param userId
     * @param roomIds
     */
    @Override
    public boolean isUserHasRole(Long userId, List<Long> roomIds) {
        if (CollectionUtil.isEmpty(roomIds)) {
            return false;
        }
        return rtcRoomRoleDAO.countByUserId(userId, roomIds) > 0;
    }

    /**
     * 设置转盘配置
     *
     * @param roomId
     * @param turntableConfig
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setTurntableConfig(Long roomId, String turntableConfig) {
        return rtcRoomDAO.updateTurntableConfig(roomId, turntableConfig) > 0;
    }

    /**
     * 获取转盘配置
     *
     * @param roomId
     * @return
     */
    @Override
    public List<TurntableItem> getTurntableConfig(Long roomId) {
        String turntableConfig = rtcRoomDAO.selectTurntableConfig(roomId);
        if (StrUtil.isNotEmpty(turntableConfig)) {
            return JSON.parseArray(turntableConfig, TurntableItem.class);
        }
        TurntableItem item = new TurntableItem();
        item.setItemKey("");
        item.setItemValue("");
        return Collections.nCopies(8, item);
    }

    /**
     * 转盘抽奖
     *
     * @param roomId
     * @return
     */
    @Override
    public int turntableLottery(Long roomId) {
        int result = RandomUtil.randomInt(1, 9);
        String turntableConfig = rtcRoomDAO.selectTurntableConfig(roomId);
        List<TurntableItem> turntableItems = JSON.parseArray(turntableConfig, TurntableItem.class);
        TurntableItem item = turntableItems.get(result - 1);

        RtcTurntableLotteryMsg lotteryMsg = new RtcTurntableLotteryMsg();
        lotteryMsg.setRoomId(roomId);
        lotteryMsg.setLotteryResult(result);
        lotteryMsg.setLotteryReward(item.getItemValue());
        lotteryMsg.setAnimationDuration(turntableAnimationDuration);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, lotteryMsg);
        return result;
    }

    /**
     * 判断房间是否配置了
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isTurntableConfig(Long roomId) {
        String turntableConfig = rtcRoomDAO.selectTurntableConfig(roomId);
        if (StrUtil.isEmpty(turntableConfig)) {
            return false;
        }
        return JSON.parseArray(turntableConfig, TurntableItem.class).size() == 8;
    }

    /**
     * 判断是否开启大冒险
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean isBigAdventureEnabled(Long roomId) {
        return Optional.ofNullable(rtcRoomDAO.selectBigAdventureEnabled(roomId)).map(t -> t == BoolIntVal.True).orElse(false);
    }

    /**
     * 获取转盘动画时长
     *
     * @return
     */
    @Override
    public Long getTurntableAnimationDuration() {
        return turntableAnimationDuration;
    }

    /**
     * 获取room Map
     *
     * @param roomIds
     * @return
     */
    @Override
    public Map<Long, RtcRoom> getRoomMap(Set<Long> roomIds) {
        if (CollectionUtil.isEmpty(roomIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, RtcRoom> roomMap = rtcRoomDAO.selectRoomMap(roomIds);
        if (CollectionUtil.isEmpty(roomMap)) {
            return Collections.EMPTY_MAP;
        }
        return roomMap;
    }

    @Override
    public Map<Long, RtcRoomSummary> getRtcRoomSummaryMap(Collection<Long> roomIds) {
        if (CollectionUtils.isEmpty(roomIds)) {
            return Collections.emptyMap();
        }
        return rtcRoomDAO.selectRtcRoomSummaryMap(roomIds);
    }

    /**
     * 获取小时榜开关  1:开启  0:关闭
     *
     * @return
     */
    @Override
    public Integer getRtcHourRankStatus() {
        return Optional.ofNullable(redisService.get(RedisRtcKeys.RTC_HOUR_RANK_STATUS)).map(Convert::toInt).orElse(CommonStatus.Disabled);
    }

    /**
     * 获取匹配池房间列表
     *
     * @return
     */
    @Override
    public List<RtcRoomListItem> getMatchPoolRooms() {
        List<Long> roomIds = rtcBattleRoomCrossProvider.getMatchPoolRoomIds();
        if (CollectionUtil.isNotEmpty(roomIds)) {
            return rtcRoomDAO.selectByIds(roomIds);
        }
        return Collections.emptyList();
    }

    @Override
    public void sendUserJoinInRoomMsg(Long curTime, Long joinUserId, Long roomId) {
        rtcRoomProvider.sendUserJoinInRoomMsg(curTime, joinUserId, roomId);
    }

    @Override
    public void sendAnchorBackRoomMsg(Long joinUserId, Long roomId) {
        rtcRoomProvider.sendAnchorBackRoomMsg(joinUserId, roomId);
    }

    private List<RtcRoomListItem> appendRoomHeat(List<RtcRoomListItem> list) {
        if (CollectionUtil.isEmpty(list)) {
            return Collections.emptyList();
        }
        Set<Long> roomIds = list.stream().map(RtcRoomListItem::getRoomId).collect(Collectors.toSet());
        Map<Long, Long> roomHeatMap = rtcHeatV2Provider.getRoomHeatMap(new ArrayList<>(roomIds));
        Map<Long, Boolean> statusMap = rtcBattleService.getRoomInBattleStatusMap(roomIds);
        // todo 前端请求gethomelist接口报错
        Map<Long, RtcRoom> hostUserMap = rtcRoomRoleDAO.selectHostUserNameMap(roomIds);
        Map<Long, String> borderMap = activityAnniversaryService.getRtcRoomBorder(roomIds);
        for (RtcRoomListItem item : list) {
            item.setRoomBorder(borderMap.get(item.getRoomId()));
            item.setRoomHeat(CollectionUtil.isEmpty(roomHeatMap) ? 0L : roomHeatMap.get(item.getRoomId()));
            item.setInBattle(CollectionUtil.isEmpty(statusMap) ? BoolIntVal.False : statusMap.get(item.getRoomId()) ? BoolIntVal.True : BoolIntVal.False);
            item.setHostUserName(CollectionUtil.isEmpty(hostUserMap) ? "" : Optional.ofNullable(hostUserMap.get(item.getRoomId())).map(RtcRoom::getUserName).orElse(""));
        }
        return list;
    }

    /**
     * 获取指定日期前后的周对应的日期
     *
     * @param n
     * @param dateStr
     * @return
     */
    public static List<Long> getWeek(int n, String dateStr) {
        List<Long> dateList = new ArrayList<>();
        String[] date = new String[7];
        try {
            Calendar calendar = Calendar.getInstance();
            calendar.setFirstDayOfWeek(Calendar.MONDAY);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd"); //设置时间格式
            Calendar cal = Calendar.getInstance();
            Date time = sdf.parse(dateStr);
            cal.setTime(time);
            // 判断要计算的日期是否是周日，如果是则减一天计算周六的，否则会出问题，计算到下一周去了
            int dayWeek = cal.get(Calendar.DAY_OF_WEEK);//获得当前日期是一个星期的第几天
            if (1 == dayWeek) {
                cal.add(Calendar.DAY_OF_MONTH, -1);
            }
            // 设置一个星期的第一天，按中国的习惯一个星期的第一天是星期一
            cal.setFirstDayOfWeek(Calendar.MONDAY);
            // 获得当前日期是一个星期的第几天
            int day = cal.get(Calendar.DAY_OF_WEEK);
            // 根据日历的规则，给当前日期减去星期几与一个星期第一天的差值
            cal.add(Calendar.DATE, (cal.getFirstDayOfWeek() - day + 7 * n));
            date[0] = sdf.format(cal.getTime());
            dateList.add(Convert.toLong(date[0]));
            for (int i = 1; i < 7; i++) {
                cal.add(Calendar.DATE, 1);
                date[i] = sdf.format(cal.getTime());
                dateList.add(Convert.toLong(date[i]));
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        return dateList;
    }

    /**
     * 所在月第一天
     *
     * @param date
     * @return
     */
    public static Date getFirstDayDateOfMonth(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int last = cal.getActualMinimum(Calendar.DAY_OF_MONTH);
        cal.set(Calendar.DAY_OF_MONTH, last);
        return cal.getTime();
    }

    /**
     * 所在月最后一天
     *
     * @param date
     * @return
     */
    public static Date getLastDayDateOfMonth(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int last = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
        cal.set(Calendar.DAY_OF_MONTH, last);
        return cal.getTime();
    }

    /**
     * 所在年第一天
     *
     * @param date
     * @return
     */
    public static Date getFirstDayDateOfYear(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int last = cal.getActualMinimum(Calendar.DAY_OF_YEAR);
        cal.set(Calendar.DAY_OF_YEAR, last);
        return cal.getTime();
    }

    /**
     * 所在年最后一天
     *
     * @param date
     * @return
     */
    public static Date getLastDayDateOfYear(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int last = cal.getActualMaximum(Calendar.DAY_OF_YEAR);
        cal.set(Calendar.DAY_OF_YEAR, last);
        return cal.getTime();
    }

    /**
     * 获取当前时间所在年的周数
     *
     * @param date
     * @return
     */
    public static int getWeekOfYear(Date date) {
        Calendar c = new GregorianCalendar();
        c.setFirstDayOfWeek(Calendar.MONDAY);
        c.setMinimalDaysInFirstWeek(7);
        c.setTime(date);
        return c.get(Calendar.WEEK_OF_YEAR);
    }

    /**
     * 获取当前时间所在年的最大周数
     *
     * @param year
     * @return
     */
    public static int getMaxWeekNumOfYear(int year) {
        Calendar c = new GregorianCalendar();
        c.set(year, Calendar.DECEMBER, 31, 23, 59, 59);
        return getWeekOfYear(c.getTime());
    }

    /**
     * 获取指定当年对应的周以及日期
     *
     * @param date
     * @return
     */
    public static Map<Integer, List<Long>> getWeekOfYearMap(Date date) {
        Date firstDayDateOfYear = getFirstDayDateOfYear(date);
        Date lastDayDateOfYear = getLastDayDateOfYear(date);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        String format = sdf.format(firstDayDateOfYear);
        int year = Convert.toInt(format.substring(0, 4));
        Map<Integer, List<Long>> map = new HashMap<>();
        int maxWeekNumOfYear = getMaxWeekNumOfYear(year);
        Long lastDay = Convert.toLong(sdf.format(lastDayDateOfYear));

        for (int i = 0; i < maxWeekNumOfYear; i++) {
            List<Long> week = getWeek(i, format);
            map.put(i, week);
            // 最后一个的时候
            if (i == maxWeekNumOfYear - 1 && week.get(week.size() - 1) < lastDay) {
                map.put(i + 1, getWeekByDay(week.get(week.size() - 1)));
            }
        }

        return map;
    }

    private static List<Long> getWeekByDay(Long day) {
        Calendar cal = Calendar.getInstance();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Date parse = null;
        try {
            parse = sdf.parse(Convert.toStr(day));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        cal.setTime(parse);
        String[] date = new String[7];
        List<Long> dateList = new ArrayList<>();
        for (int i = 1; i < 7; i++) {
            cal.add(Calendar.DATE, 1);
            date[i] = sdf.format(cal.getTime());
            dateList.add(Convert.toLong(date[i]));
        }
        return dateList;
    }


    @Override
    public NoticeConfigVO getNoticeConfig(Long noticeId, Integer quantity, Long value) {
        return noticeProvider.getNoticeConfig(noticeId, quantity, value);
    }

    /**
     * 处理个播用户退出房间逻辑
     *
     * @param roomId
     */
    @Override
    public void dealSingleRoomOffline(Long roomId, Long exitUserId) {
        RtcRoom room = getRoom(roomId);
        RtcBusinessTypeEnum rtcBusinessTypeEnum = RtcBusinessTypeEnum.toEnum(room.getBusinessType());
        if (RtcBusinessTypeEnum.RTC_SINGLE.equals(rtcBusinessTypeEnum)) {

            // 存放在房间3分钟以上的用户
            // 先查用户什么时候加入的房间
            RtcRoomLiveHisDO his = rtcRoomLiveHisService.getCurrentRound(roomId);
            if (his != null) {

                if (isDev()) {
                    log.info("处理个播离线逻辑:joinTime开始：roomId{}, exitUserId:{}, getLiveBeginTime:{}, his:{}", roomId, exitUserId, his.getLiveBeginTime(), JSON.toJSONString(his));
                }
                Long joinTime = Optional.ofNullable(rtcRoomJoinHisDAO.selectUserRecentJoinRoomTime(roomId, exitUserId, his.getLiveBeginTime())).orElse(0L);
                if (isDev()) {
                    log.info("处理个播离线逻辑:joinTime:{}", joinTime);
                }

                // 判断是否超过3分钟
                if (isDev()) {
                    log.info("处理个播离线逻辑:是否超过时间:{}", System.currentTimeMillis() - joinTime >= 3 * 60 * 1000L);
                }
                Long time = System.currentTimeMillis() - joinTime;
                if (time >= 3 * 60 * 1000L) {
                    // 超过就放redis里
                    redisService.zIncrementScore(RedisRtcKeys.getRtcRoomStayThreeMinutes(roomId), Convert.toStr(roomId), 1);

                    // 数据存一份，供查询
                    RtcRoomStayLogDO stayLogDO = new RtcRoomStayLogDO();
                    stayLogDO.setHisId(identityService.getId());
                    stayLogDO.setRoomId(roomId);
                    stayLogDO.setUserId(exitUserId);
                    stayLogDO.setCreateTime(System.currentTimeMillis());
                    stayLogDO.setStayMinutes(Math.abs(time.intValue() / 1000));
                    stayLogDO.setCreateDate(DateUtil.getIntDate());
                    rtcRoomStayLogDAO.insert(stayLogDO);

                    if (isDev()) {
                        log.info("处理个播离线逻辑:已经放redis里里");
                    }
                }
            }

            // 主播离线处理
            if (room.getUserId().equals(exitUserId)) {
                // 插入离线记录,5分钟后如果还没有在线，就将直播间关掉
                // 插入记录，定时任务会去扫这张表
                RtcRoomLiveOfflineHisDO hisDO = new RtcRoomLiveOfflineHisDO();
                hisDO.setUserId(exitUserId);
                hisDO.setRoomId(roomId);
                hisDO.setLiveHisId(Optional.ofNullable(his).orElse(new RtcRoomLiveHisDO()).getHisId());
                hisDO.setLeaveTime(System.currentTimeMillis());
                rtcRoomLiveOfflineHisService.create(hisDO);

                // 发送给观众，主播离开
                // 如果融云消息迟到，这里不至于让主播一直是离线状态
                // 判断用户此刻是否离线并且不在房间
                if (!userService.isOnline(exitUserId) || !roomId.equals(rtcUserService.getUserStayRoomId(exitUserId))) {
                    SingleRtcLiveUserStatusMsg userMsg = new SingleRtcLiveUserStatusMsg();
                    userMsg.setStatus(SingleRtcRoomAnchorStatusEnum.OFFLINE.getValue());
                    userMsg.setRoomId(roomId);
                    userMsg.setUserId(exitUserId);
                    imService.sendRtcRoomMsg(ImTargetType.GROUP, exitUserId, null, roomId, userMsg);
                }

            }
        }
    }

    @Override
    public RtcRoomRatio getRtcRoomRatio(Long roomId) {
        Preconditions.checkNotNull(roomId, "roomId 不能为 null");
        // 个播房的categoryId每次开播时用户可以选择，使用缓存时需要注意
        RtcRoomDO room = rtcRoomDAO.selectRoomSummaryById(roomId);
        if (room == null) {
            return null;
        }

        RtcRoomRatio roomRatio = new RtcRoomRatio();
        roomRatio.setRoomId(roomId);
        roomRatio.setBusinessType(RtcBusinessTypeEnum.toEnum(room.getBusinessType()));
        roomRatio.setCategoryUserRatio(null);
        roomRatio.setFamilyRatio(null);

        Long categoryId = room.getCategoryId();
        if (categoryId != null) {
            roomRatio.setCategoryUserRatio(rtcRoomCategoryProvider.getUserRatio(categoryId));
        }
        Long familyId = room.getFamilyId();
        if (familyId != null) {
            roomRatio.setFamilyRatio(familyProvider.getFamilyRatio(familyId));
        }
        return roomRatio;
    }


    /**
     * 获取房间足迹列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<RtcRoomVisitHisVO> getRtcRoomHisList(Long userId) {
        List<RtcRoomJoinHisDO> joinHisDO = rtcRoomJoinHisDAO.getUserJoinHis(userId, 20);
        if (CollectionUtil.isNotEmpty(joinHisDO)) {
            List<RtcRoomVisitHisVO> res = new ArrayList<>(joinHisDO.size());
            Set<Long> roomSet = new HashSet<>();
            Set<Long> distinctSet = new HashSet<>();
            joinHisDO.forEach(i -> roomSet.add(i.getRoomId()));
            Map<Long, RtcRoom> rtcRoomDOMap = getRoomMap(roomSet);
            for (RtcRoomJoinHisDO rtcRoomJoinHisDO : joinHisDO) {
                if (!distinctSet.contains(rtcRoomJoinHisDO.getRoomId())) {
                    RtcRoomVisitHisVO tmp = new RtcRoomVisitHisVO();
                    RtcRoom room = rtcRoomDOMap.get(rtcRoomJoinHisDO.getRoomId());
                    if (room != null) {
                        tmp.setRoomId(room.getRoomId());
                        tmp.setRoomTitle(room.getRoomName());
                        tmp.setRoomCover(room.getRoomCover());
                        tmp.setCreateTime(rtcRoomJoinHisDO.getHisCreateTime());
                        tmp.setRoomType(room.getRoomType());
                        res.add(tmp);
                        distinctSet.add(rtcRoomJoinHisDO.getRoomId());
                    }
                }
            }
            if (res.size() > 4) {
                res = res.subList(0, 4);
            }
            return res;
        } else {
            return new ArrayList<>();
        }
    }

    /**
     * 获取房间足迹分页列表
     *
     * @param userId
     * @return
     */
    @Override
    public PageList<RtcRoomVisitHisRichVO> getRtcRoomHisPageList(Long userId, int pageIndex, int pageSize) {

        PageList<RtcRoomVisitHisRichVO> pageList = new PageList<>(pageIndex, pageSize);
        List<RtcRoomVisitHisDO> joinHisDO = rtcRoomVisitHisDAO.selectUserJoinHisPageList(userId, pageIndex, pageSize);

        if (CollectionUtil.isEmpty(joinHisDO)) {
            return pageList;
        }

        List<RtcRoomVisitHisRichVO> list = new ArrayList<>();
        Set<Long> roomIds = joinHisDO.stream().map(RtcRoomVisitHisDO::getRoomId).collect(Collectors.toSet());
        Map<Long, RtcRoom> rtcRoomMap = getRoomMap(roomIds);
        joinHisDO.forEach(val -> {
            RtcRoomVisitHisRichVO vo = new RtcRoomVisitHisRichVO();

            vo.setRoomId(val.getRoomId());

            if (MapUtil.isEmpty(rtcRoomMap)) {
                return;
            }

            RtcRoom room = rtcRoomMap.get(val.getRoomId());
            if (room == null) {
                return;
            }

            vo.setRoomCover(room.getRoomCover());
            vo.setRoomNo(room.getRoomNo());
            vo.setRoomHeat(room.getRoomHeat());
            vo.setRoomTitle(room.getRoomTitle());
            vo.setRoomName(room.getRoomName());
            vo.setCategoryName(room.getCategoryName());
            vo.setCategoryId(room.getCategoryId());
            vo.setRoomType(room.getRoomType());
            vo.setDateInt(val.getHisDate());
            vo.setIsOpen(rtcRoomDAO.isRoomOnline(val.getRoomId()));
            list.add(vo);
        });

        pageList.setList(list);
        pageList.setTotal(rtcRoomVisitHisDAO.selectUserJoinHisCount(userId));
        return pageList;
    }
}
