package com.starbuds.server.service.rtc.impl;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.RtcRoomSuggestDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomSuggestItem;
import com.starbuds.server.common.pojo.rtc.qo.RtcRoomSuggestPageListQO;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomSuggestVO;
import com.starbuds.server.common.pojo.user.UserSimpleVO;
import com.starbuds.server.common.service.activity.ActivityAnniversaryService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.rtc.RtcRoomSuggestService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.rtc.dao.RtcRoomSuggestDAO;
import com.starbuds.server.service.rtc.provider.RtcRoomSuggestProvider;
import com.starbuds.server.service.rtc.provider.RtcSeatProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;
import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 推荐语聊房间
 */
@Service(version = "1.0.0")
@Slf4j
public class RtcRoomSuggestServiceImpl implements RtcRoomSuggestService {

    @Reference(version = "1.0.0")
    private RtcRoomService rtcRoomService;

    @Resource
    private RtcRoomSuggestProvider rtcRoomSuggestProvider;

    @Resource
    private RtcRoomSuggestDAO rtcRoomSuggestDAO;

    @Resource
    private RtcSeatProvider rtcSeatProvider;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private ActivityAnniversaryService activityAnniversaryService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public RtcRoomSuggestDO create(RtcRoomSuggestDO po) {
        return rtcRoomSuggestProvider.create(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<RtcRoomSuggestDO> list) {
        return rtcRoomSuggestProvider.createBatch(list);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(RtcRoomSuggestDO po) {
        return rtcRoomSuggestProvider.createOrUpdate(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(RtcRoomSuggestDO po) {
        return rtcRoomSuggestProvider.updateById(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        return rtcRoomSuggestProvider.update(uw);
    }

    @Override
    public RtcRoomSuggestDO getById(Long id) {
        return rtcRoomSuggestProvider.getById(id);
    }

    @Override
    public boolean exist(Long id) {
        return rtcRoomSuggestProvider.exist(id);
    }

    @Override
    public PageList<RtcRoomSuggestDO> getPageList(RtcRoomSuggestPageListQO qo) {
        return rtcRoomSuggestProvider.getPageList(qo);
    }

    @Override
    public Map<Long, RtcRoomSuggestDO> getMapByIds(Collection ids) {
        return rtcRoomSuggestProvider.getMapByIds(ids);
    }

    // 手动编码开始

    @Override
    public List<RtcRoomSuggestVO> getRoomSuggestList(Integer showType) {
        RtcRoomSuggestPageListQO qo = new RtcRoomSuggestPageListQO();
        qo.setStatus(CommonStatus.Normal);
        qo.setIsDeleted(BoolIntVal.False);
        if (showType == null) {
            showType = 0;
        }
        qo.setShowType(showType);
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        List<RtcRoomSuggestDO> list = rtcRoomSuggestDAO.selectList(qw);

        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }

        List<Long> roomIds = new ArrayList<>();
        // 房间Id:主持人用户Id
        Map<Long, Long> roomUserMap = new HashMap<>();
        list.stream()
                .forEach(po -> {
                    Long roomId = po.getRoomId();
                    Map<Integer, Long> seatUserMap = rtcSeatProvider.getSeatUserMap(roomId);
                    // 不返回麦位上没有人的数据
                    if (CollectionUtils.isEmpty(seatUserMap)) {
                        return;
                    }
                    roomIds.add(roomId);
                    // 主持人id
                    Long userId = seatUserMap.get(0);
                    if (userId == null) {
                        return;
                    }
                    roomUserMap.put(roomId, userId);
                });
        if (CollectionUtils.isEmpty(roomIds)) {
            return Collections.emptyList();
        }

        List<Long> userIds = roomUserMap.values().stream().collect(Collectors.toList());
        Map<Long, UserSimpleVO> simpleUserMap = userService.getSimpleUserMap(userIds);
        Map<Long, RtcRoomSuggestItem> suggestMap = rtcRoomService.getSuggestMap(roomIds, true);
        Set<Long> roomSet = roomIds.stream().collect(Collectors.toSet());
        Map<Long, String> borderMap = activityAnniversaryService.getRtcRoomBorder(roomSet);
        // 带房间信息的列表
        List<RtcRoomSuggestVO> voList = list.stream()
                .map(room -> {
                    RtcRoomSuggestVO vo = new RtcRoomSuggestVO();
                    BeanUtils.copyProperties(room, vo);

                    RtcRoomSuggestItem item = suggestMap.get(room.getRoomId());
                    // 不返回信息缺失的数据
                    if (item == null) {
                        return null;
                    }
                    BeanUtils.copyProperties(item, vo);
                    String title = vo.getTitle();
                    if (StringUtils.isBlank(title)) {
                        vo.setTitle(item.getRoomName());
                        title = vo.getTitle();
                    }
                    // 标题最长6位
                    if (title.length() > 6) {
                        vo.setTitle(StringUtils.substring(title, 0, 5));
                    }
                    return vo;
                })
                .filter(Objects::nonNull)
                .map(vo -> {
                    // 如果主持位上有人，使用主持人头像
                    Long roomId = vo.getRoomId();
                    Long userId = roomUserMap.get(roomId);
                    if (userId == null) {
                        return vo;
                    }
                    UserSimpleVO user = simpleUserMap.get(userId);
                    if (user == null) {
                        return vo;
                    }
                    String userAvatar = user.getUserAvatar();
                    if (StringUtils.isNotBlank(userAvatar)) {
                        vo.setRoomCover(userAvatar);
                    }
                    vo.setSex(user.getUserSex());
                    vo.setRoomBorder(borderMap.get(roomId));
                    return vo;
                })
                .collect(Collectors.toList());

        return voList;
    }
}