package com.starbuds.server.service.rtc.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.rtc.RoomTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcLoveStage;
import com.starbuds.server.common.constant.rtc.RtcSeatCount;
import com.starbuds.server.common.pojo.db.FacialExpressionDO;
import com.starbuds.server.common.pojo.message.im.live.LiveUserBlackMsg;
import com.starbuds.server.common.pojo.message.im.rtc.*;
import com.starbuds.server.common.pojo.mq.QueueZegoVocieTimeoutMsg;
import com.starbuds.server.common.pojo.rtc.*;
import com.starbuds.server.common.pojo.zego.RtcServiceCallback;
import com.starbuds.server.common.pojo.zego.ZegoServerCallbackType;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.LiveService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcQueueService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.rtc.agora.constant.AgoraRealTimeEventType;
import com.starbuds.server.service.rtc.dao.FacialExpressionDAO;
import com.starbuds.server.service.rtc.dao.RtcRoomDAO;
import com.starbuds.server.service.rtc.dao.RtcRoomRoleDAO;
import com.starbuds.server.service.rtc.provider.*;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * RTC 麦位服务
 *
 * @author Lay
 * @date 2020/7/7
 */
@Slf4j
@Service(version = "1.0.0")
public class RtcSeatServiceImpl implements RtcSeatService {

//    private static ScheduledExecutorService mScheduledExcutorService = Executors.newScheduledThreadPool(4);

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    LiveService liveService;

    @Resource
    private RtcRoomRoleDAO rtcRoomRoleDAO;

    @Resource
    private RtcRoomDAO rtcRoomDAO;

    @Resource
    private FacialExpressionDAO facialExpressionDAO;

    @Resource
    private RtcSeatProvider rtcSeatProvider;

    @Resource
    private RtcQueueProvider rtcQueueProvider;

    @Resource
    private RtcScoreProvider rtcScoreProvider;

    @Resource
    private RtcLoveStageProvider rtcLoveStageProvider;

    @Resource
    private RtcLoveChooseProvider rtcLoveChooseProvider;

    @Resource
    private RtcBattleRoomInProvider rtcBattleRoomInProvider;

    @Resource
    private RtcLoveScoreProvider rtcLoveScoreProvider;

    @Reference(version = "1.0.0")
    private RtcQueueService rtcQueueService;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Reference(version = "1.0.0")
    private RtcRoomService rtcRoomService;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 上麦
     *
     * @param roomId
     * @param userId
     * @param seatNo
     * @return
     */
    @Override
    public int sitSeat(Long roomId, Long userId, Integer seatNo, String appVer) {
        return sitSeat(roomId, userId, seatNo, false, appVer);
    }

    /**
     * 上麦
     *
     * @param roomId
     * @param userId
     * @param seatNo
     * @param skipLock
     * @return
     */
    @Override
    public int sitSeat(Long roomId, Long userId, Integer seatNo, boolean skipLock, String appVer) {
        return rtcSeatProvider.sitSeat(roomId, userId, seatNo, skipLock, appVer);
    }

    /**
     * 下麦
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public int leaveSeat(Long roomId, Long userId, String appVer) {
        return leaveSeat(roomId, userId, false, appVer);
    }

    /**
     * 下麦
     *
     * @param roomId
     * @param userId
     * @param isForce
     * @return
     */
    @Override
    public int leaveSeat(Long roomId, Long userId, boolean isForce, String appVer) {
        return rtcSeatProvider.leaveSeat(roomId, userId, isForce, appVer);
    }

    /**
     * 下麦
     *
     * @param roomId
     * @param seatNo
     * @param isForce
     * @return
     */
    @Override
    public int leaveSeat(Long roomId, int seatNo, boolean isForce, String appVer) {
        return rtcSeatProvider.leaveSeat(roomId, seatNo, isForce, appVer);
    }

    /**
     * 获取麦位信息
     *
     * @param roomId
     * @return
     */
    @Override
    public List<RtcSeat> getSeats(Long roomId) {
        Set<Integer> seatLocks = rtcSeatProvider.getSeatLockSet(roomId);
        Set<Integer> silencedSet = rtcSeatProvider.getSeatMicSilencedSet(roomId);
        Set<Integer> disabledSet = rtcSeatProvider.getSeatDisabledSet(roomId);
        Set<Integer> bossSeatSet = rtcSeatProvider.getBoosSeats(roomId);
        Map<Long, Long> userScoreMap = rtcScoreProvider.getUserScoreMap(roomId);
        Map<Integer, Integer> countDownMap = rtcSeatProvider.getCountDownRemainMap(roomId);

        // 用户构造
        Map<Integer, Long> seatUserIdMap = rtcSeatProvider.getSeatUserMap(roomId);
        Map<Long, RtcRoomUser> userMap = null;
        if (seatUserIdMap.size() > 0) {
            List<RtcRoomUser> users = rtcRoomRoleDAO.selectUsersByIdList(roomId, new ArrayList<>(seatUserIdMap.values()));
            if (CollectionUtil.isNotEmpty(users)) {
                users.stream().forEach(item -> item.setChannelName("YL-" + roomId + item.getUserId()));
                userMap = users.stream().collect(Collectors.toMap(RtcRoomUser::getUserId, Function.identity()));
            }
        }

        // 房间类型
        Integer roomType = rtcRoomDAO.selectRoomType(roomId);

        int loveStage = -1;
        int maxSeats = 9;
        Map<Integer, Integer> loveChoseMap = null;
        if (RoomTypeEnum.LOVE.getValue() == roomType) {
            maxSeats = 10;
            loveStage = rtcLoveStageProvider.getStage(roomId);
            if (RoomTypeEnum.LOVE.getValue() == roomType) {
                if (RtcLoveStage.CHOOSE == loveStage || RtcLoveStage.MATCH == loveStage) {
                    loveChoseMap = rtcLoveChooseProvider.getLoveChoseMap(roomId);
                }
            }
        }
        Map<Long, Long> battleScoreMap = null;
        Long battleId = rtcBattleRoomInProvider.getBattleId(roomId);
        if (battleId != null) {
            // 获取 PK 值
            battleScoreMap = rtcBattleRoomInProvider.getBattleScoreMap(battleId, seatUserIdMap);
        }

        // 竞拍房
        if (RoomTypeEnum.AUCTION.getValue() - roomType == 0) {
            // 麦序说明：0:主持；1:拍卖对象；2:嘉宾；3、4、5、6: 均是普通麦位
            maxSeats = RtcSeatCount.AUCTION_ROOM_MAX;
        }

        List<RtcSeat> seats = new ArrayList<>();
        for (int i = 0; i < maxSeats; i++) {
            RtcSeat rtcSeat = new RtcSeat();
            rtcSeat.setSeatNo(i);
            Long userId = seatUserIdMap.get(i);

            // 麦位锁
            if (CollectionUtil.isNotEmpty(seatLocks)) {
                rtcSeat.setSeatLocked(seatLocks.contains(i) ? BoolIntVal.True : BoolIntVal.False);
            } else {
                rtcSeat.setSeatLocked(BoolIntVal.False);
            }

            // 静音
            if (CollectionUtil.isNotEmpty(silencedSet)) {
                rtcSeat.setMicSilenced(silencedSet.contains(i) ? BoolIntVal.True : BoolIntVal.False);
            } else {
                rtcSeat.setMicSilenced(BoolIntVal.False);
            }

            // 禁麦
            if (CollectionUtil.isNotEmpty(disabledSet)) {
                rtcSeat.setMicDisabled(disabledSet.contains(i) ? BoolIntVal.True : BoolIntVal.False);
            } else {
                rtcSeat.setMicDisabled(BoolIntVal.False);
            }

            if (seatUserIdMap.size() > 0) {
                if (userId != null && userMap != null) {
                    rtcSeat.setUser(userMap.get(userId));
                }
            }

            // 心动值
            if (CollectionUtil.isNotEmpty(userScoreMap) && CollectionUtil.isNotEmpty(seatUserIdMap)) {
                Long seatUserId = seatUserIdMap.get(i);
                rtcSeat.setSeatScore(Optional.ofNullable(userScoreMap.get(seatUserId)).orElse(0L));
            } else {
                rtcSeat.setSeatScore(0L);
            }

            // PK 值
            if (battleScoreMap != null) {
                rtcSeat.setBattleScore(battleScoreMap.getOrDefault(userId, 0L));
            }

            // 老板位
            if (bossSeatSet != null) {
                if (bossSeatSet.contains(i)) {
                    rtcSeat.setBossSeat(BoolIntVal.True);
                } else {
                    rtcSeat.setBossSeat(BoolIntVal.False);
                }
            }

            if (countDownMap != null) {
                rtcSeat.setCountDownSeconds(Optional.ofNullable(countDownMap.get(i)).orElse(0));
            }

            // 相亲房
            if (RtcLoveStage.CHOOSE == loveStage) {
                if (loveChoseMap != null) {
                    rtcSeat.setLoveChoseStatus(loveChoseMap.get(rtcSeat.getSeatNo()) == null ? BoolIntVal.False : BoolIntVal.True);
                } else {
                    rtcSeat.setLoveChoseStatus(BoolIntVal.False);
                }
            } else if (RtcLoveStage.MATCH == loveStage) {
                if (loveChoseMap != null) {
                    Integer choseSeatNo = loveChoseMap.get(rtcSeat.getSeatNo());
                    if (choseSeatNo != null) {
                        rtcSeat.setLoveChoseStatus(BoolIntVal.True);
                        rtcSeat.setLoveChoseSeatNo(choseSeatNo);
                    } else {
                        rtcSeat.setLoveChoseStatus(BoolIntVal.False);
                    }
                }
            }

            seats.add(rtcSeat);
        }

        return seats;
    }

    /**
     * 获取用户麦位
     *
     * @param roomId
     * @return
     */
    @Override
    public List<Long> getSeatUserIds(Long roomId) {
        return rtcSeatProvider.getSeatUserIds(roomId);
    }

    /**
     * 获取麦位用户 id
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @Override
    public Long getSeatUserId(Long roomId, int seatNo) {
        return rtcSeatProvider.getSeatUserId(roomId, seatNo);
    }

    /**
     * 是否当前主持人
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean isCurrentHost(Long roomId, Long userId) {
        if (roomId == null || userId == null) {
            return false;
        }
        return userId.equals(rtcSeatProvider.getSeatUserId(roomId, 0));
    }

    /**
     * 判断位置是否空置
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @Override
    public boolean isSeatIdle(Long roomId, int seatNo) {
        return rtcSeatProvider.getSeatUserId(roomId, seatNo) == null;
    }

    /**
     * 锁定麦位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @Override
    public boolean lockSeat(Long roomId, Integer seatNo) {
        boolean locked = rtcSeatProvider.lockSeat(roomId, seatNo);
        if (locked) {
            // 发送消息
            RtcSeatLockMsg seatLockMsg = new RtcSeatLockMsg();
            seatLockMsg.setRoomId(roomId);
            seatLockMsg.setSeatNo(seatNo);
            seatLockMsg.setLocked(BoolIntVal.True);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, seatLockMsg);
        }
        return locked;
    }

    /**
     * 解锁麦位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @Override
    public boolean unlockSeat(Long roomId, Integer seatNo) {
        rtcSeatProvider.unlockSeat(roomId, seatNo);
        RtcSeatLockMsg seatLockMsg = new RtcSeatLockMsg();
        seatLockMsg.setRoomId(roomId);
        seatLockMsg.setSeatNo(seatNo);
        seatLockMsg.setLocked(BoolIntVal.False);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, seatLockMsg);
        return true;
    }

    /**
     * 锁定所有麦位
     *
     * @param roomId
     * @return
     */
    @Override
    public int lockSeatAll(Long roomId) {
        int lockNum = rtcSeatProvider.lockSeatAll(roomId);
        if (lockNum > 0) {
            // 发送消息
            RtcSeatLockMsg seatLockMsg = new RtcSeatLockMsg();
            seatLockMsg.setRoomId(roomId);
            seatLockMsg.setSeatNo(-1);
            seatLockMsg.setLocked(BoolIntVal.True);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, seatLockMsg);
        }
        return lockNum;
    }

    /**
     * 解锁所有麦位
     *
     * @param roomId
     * @return
     */
    @Override
    public boolean unlockSeatAll(Long roomId) {
        boolean unlocked = rtcSeatProvider.unlockAllSeat(roomId);
        if (unlocked) {
            // 发送消息
            RtcSeatLockMsg seatLockMsg = new RtcSeatLockMsg();
            seatLockMsg.setRoomId(roomId);
            seatLockMsg.setSeatNo(-1);
            seatLockMsg.setLocked(BoolIntVal.False);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, seatLockMsg);
        }
        return unlocked;
    }

    /**
     * 设置禁麦状态
     *
     * @param roomId
     * @param seatNo
     * @param disabled
     * @return
     */
    @Override
    public boolean setMicDisabled(Long roomId, int seatNo, boolean disabled) {
        boolean success;

        if (disabled) {
            // 发送消息
            success = rtcSeatProvider.setMicDisabled(roomId, seatNo);
        } else {
            success = rtcSeatProvider.setMicEnabled(roomId, seatNo);
        }

        if (success) {
            RtcMicDisabledMsg micDisabledMsg = new RtcMicDisabledMsg();
            micDisabledMsg.setRoomId(roomId);
            micDisabledMsg.setSeatNo(seatNo);
            micDisabledMsg.setDisabled(disabled ? BoolIntVal.True : BoolIntVal.False);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, micDisabledMsg);
        }

        return success;
    }

    /**
     * 设置静音状态
     *
     * @param roomId
     * @param seatNo
     * @param silenced
     * @return
     */
    @Override
    public boolean setMicSilenced(Long roomId, int seatNo, boolean silenced) {
        boolean success;

        if (silenced) {
            success = rtcSeatProvider.setMicSilenced(roomId, seatNo);
        } else {
            success = rtcSeatProvider.cancelMicSilenced(roomId, seatNo);
        }
        if (success) {
            RtcMicSilencedMsg micSilencedMsg = new RtcMicSilencedMsg();
            micSilencedMsg.setRoomId(roomId);
            micSilencedMsg.setSeatNo(seatNo);
            micSilencedMsg.setSilenced(silenced ? BoolIntVal.True : BoolIntVal.False);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, micSilencedMsg);
        }

        return success;

    }

    /**
     * 麦序用户上麦
     *
     * @param roomId
     * @return
     */
    @Override
    public int sitSeatInQueue(Long roomId, String appVer) {
        List<Long> userIds = rtcQueueProvider.getUserIds(roomId);
        int num = 0;
        if (CollectionUtil.isNotEmpty(userIds)) {
            for (Long userId : userIds) {
                if (sitSeat(roomId, userId, null, true, appVer) > 0) {
                    num++;
                }
            }
        }
        return num;
    }

    /**
     * 获取用户麦位
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public int getUserSeatNo(Long roomId, Long userId) {
        return rtcSeatProvider.getUserSeatNo(roomId, userId);
    }

    /**
     * 是否在麦位上
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isOnSeat(Long userId) {
        Long roomId = rtcUserService.getUserStayRoomId(userId);
        if (roomId == null || roomId == 0) {
            return false;
        }
        return getUserSeatNo(roomId, userId) > -1;
    }

    /**
     * 获取用户位置
     *
     * @param roomId
     * @param userIds
     * @return userId:seatNo
     */
    @Override
    public Map<Long, Integer> getUserSeats(Long roomId, List<Long> userIds) {
        return rtcSeatProvider.getUserSeats(roomId, userIds);
    }

    /**
     * 设置老板位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @Override
    public boolean setBossSeat(Long roomId, Integer seatNo) {
        if (rtcSeatProvider.setBossSeat(roomId, seatNo)) {
            RtcBossSeatMsg bossSeatMsg = new RtcBossSeatMsg();
            bossSeatMsg.setRoomId(roomId);
            bossSeatMsg.setSeatNo(seatNo);
            bossSeatMsg.setAction(BoolIntVal.True);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, bossSeatMsg);
            return true;
        }
        return false;
    }

    /**
     * 取消老板位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    @Override
    public boolean removeBossSeat(Long roomId, Integer seatNo) {
        if (rtcSeatProvider.removeBossSeat(roomId, seatNo)) {
            RtcBossSeatMsg bossSeatMsg = new RtcBossSeatMsg();
            bossSeatMsg.setRoomId(roomId);
            bossSeatMsg.setSeatNo(seatNo);
            bossSeatMsg.setAction(BoolIntVal.False);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, bossSeatMsg);
            return true;
        }
        return false;
    }

    /**
     * 设置倒计时
     *
     * @param roomId
     * @param seatNo
     * @param seconds
     * @return
     */
    @Override
    public boolean setCountDown(Long roomId, Integer seatNo, Integer seconds) {
        seconds = Optional.ofNullable(seconds).orElse(0);

        // 发送消息
        RtcCountDownMsg rtcCountDownMsg = new RtcCountDownMsg();
        rtcCountDownMsg.setRoomId(roomId);
        rtcCountDownMsg.setSeatNo(seatNo);
        rtcCountDownMsg.setSeconds(seconds);

        if (seconds > 0) {
            if (rtcSeatProvider.newCountDown(roomId, seatNo, seconds)) {
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, rtcCountDownMsg);
                return true;
            }
        } else {
            if (rtcSeatProvider.removeCountDown(roomId, seatNo)) {
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, rtcCountDownMsg);
                return true;
            }
        }
        return false;
    }

    /**
     * 获取麦位心动值列表
     *
     * @param roomId
     * @return
     */
    @Override
    public List<RtcSeatScore> getScoreList(Long roomId) {
        Map<Integer, Long> seatUserMap = rtcSeatProvider.getSeatUserMap(roomId);
        List<Long> seatUserIds = seatUserMap.values().stream().collect(Collectors.toList());
        List<RtcUserScore> userScoreList = rtcScoreProvider.getUserScoreList(roomId);
        List<RtcUserScore> seatUserScoreList = userScoreList.stream().filter(e -> seatUserIds.contains(e)).collect(Collectors.toList());
        List<RtcSeatScore> seatScores = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(seatUserScoreList)) {
            long curTime = System.currentTimeMillis();
            for (Map.Entry<Integer, Long> entry : seatUserMap.entrySet()) {

                List<Long> scoreValues = seatUserScoreList.stream().filter(e -> e.getUserId().equals(entry.getValue())).map(RtcUserScore::getScoreValue).collect(Collectors.toList());
                List<Long> timestamps = seatUserScoreList.stream().filter(e -> e.getUserId().equals(entry.getValue())).map(RtcUserScore::getTimestamp).collect(Collectors.toList());

                RtcSeatScore seatScore = new RtcSeatScore();
                seatScore.setSeatNo(entry.getKey());
                seatScore.setScoreValue(CollectionUtil.isEmpty(scoreValues) ? 0L : scoreValues.get(0));
                seatScore.setTimestamp(CollectionUtil.isEmpty(timestamps) ? curTime : timestamps.get(0));
                seatScores.add(seatScore);
            }
        }
        return seatScores;
    }

    /**
     * 获取麦位心动值列表
     *
     * @param roomId
     * @return
     */
    @Override
    public List<RtcUserScore> getUserScoreList(Long roomId) {
        return rtcScoreProvider.getUserScoreList(roomId);
    }

    /**
     * 获取麦上用户数
     *
     * @param roomId
     * @param containsSelf
     * @param userId
     * @return
     */
    @Override
    public int getSeatUserCount(Long roomId, boolean containsSelf, Long userId) {
        int seatUserCount = rtcSeatProvider.getSeatUserCount(roomId);
        if (!containsSelf) {
            if (rtcSeatProvider.getUserSeatNo(roomId, userId) > -1) {
                return seatUserCount - 1;
            }
        }
        return seatUserCount;
    }

    @Override
    public boolean handleCallbackNotify(RtcServiceCallback serviceCallback) {
        switch (serviceCallback.getEventType()) {
            case AgoraRealTimeEventType.ANCHOR_JOIN_CHANNEL:
            case ZegoServerCallbackType.StreamCreate:
                // 主播上麦
                break;
            case AgoraRealTimeEventType.ANCHOR_LEAVE_CHANNEL:
            case ZegoServerCallbackType.StreamClose:
                handleNotifyLeave(Convert.toLong(serviceCallback.getChannelName().replace("YL-", "")), serviceCallback.getUserID(), serviceCallback.getAppVer());
                break;
            default:
                return true;
        }

        return true;
    }

    /**
     * 发送表情
     *
     * @param roomId
     * @param userId
     * @param seatNo
     * @param expId
     * @return
     */
    @Override
    public boolean sendFacialExpression(Long roomId, Long userId, Integer seatNo, Integer expId) {
        FacialExpressionDO expressionDO = facialExpressionDAO.selectById(expId);
        if (expressionDO != null) {
            // 创建对象
            RtcFacialExpressionMsg facialExpressionMsg = new RtcFacialExpressionMsg();

            if (StringUtils.isNotBlank(expressionDO.getRandomData())) {
                FacialRandomData facialRandomData = JSON.parseObject(expressionDO.getRandomData(), FacialRandomData.class);
                if (facialRandomData != null && facialRandomData.getNumFrame() != null) {
                    // 获取随机数
                    int randomNum = RandomUtil.randomInt(facialRandomData.getNumMin(), facialRandomData.getNumMax() + 1);
                    facialExpressionMsg.setFinalFrame(facialRandomData.getNumFrame().get(String.valueOf(randomNum)));
                }
            }

            facialExpressionMsg.setRoomId(roomId);
            facialExpressionMsg.setExpId(expId);
            facialExpressionMsg.setSeatNo(seatNo);
            facialExpressionMsg.setUserId(userId);
            facialExpressionMsg.setExpUrl(expressionDO.getExpUrl());
            facialExpressionMsg.setPlayTimes(expressionDO.getPlayTimes());
            log.info("imService.sendRtcRoomMsg-开始");
            imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, facialExpressionMsg);
            log.info("imService.sendRtcRoomMsg-结束");
            return true;
        }

        return false;
    }

    /**
     * 强行踢出房间（包含下麦）
     *
     * @param userId
     * @return
     */
    @Override
    public String forceRemoveRoom(Long userId) {
        if (userId == null) {
            return "userId 为 null 不能强行踢出房间";
        }

        Long roomId = getUserRoomId(userId);
        if (roomId < 1) {
            return "用户不在房间，无需踢出房间";
        }

        String resultPrefix = "强行踢出房间操作结果：";
        String result = resultPrefix;

        // 判断是否在麦上
        if (rtcSeatProvider.getUserSeatNo(roomId, userId) > -1) {
            String appVersion = redisService.get(RedisUserKeys.getUserAppVer(userId));
            // 强制下麦
            if (leaveSeat(roomId, userId, true, appVersion) > 0) {
                result += "强制下麦成功!";
            }
        }
        boolean removeQueueUser = rtcQueueService.removeQueueUser(roomId, userId);
        if (removeQueueUser) {
            result += "移除麦序成功!";
        }

        // 发送拉黑消息
        LiveUserBlackMsg msg = new LiveUserBlackMsg();
        msg.setMasterUserName("管理员");
        imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, msg);

        return result.equals(resultPrefix) ? "用户不在房间内，无需踢出房间" : result;
    }

    //服务端回调的时候调用此函数
    private void handleNotifyLeave(Long roomId, Long userId, String AppVer) {
//        Long userId = userService.getUserIdByNo(Convert.toLong(userNo));
        if (userId != null && userId > 0) {
            // 判断用户是否还在麦上
            int seatNo = rtcSeatProvider.getUserSeatNo(roomId, userId);
            // 在麦上且不是房主
            if (seatNo > -1 && !rtcRoomDAO.selectUserId(roomId).equals(userId)) {
                // 强制下麦
                rtcSeatProvider.leaveSeat(roomId, seatNo, true, AppVer);
            }

//            if (AppVer.equals("1.8.0")) {
//                // 判断用户是否还在麦上
//                int seatNo = rtcSeatProvider.getUserSeatNo(roomId, userId);
//                // 在麦上且不是房主
//                if (seatNo > -1 && !rtcRoomDAO.selectUserId(roomId).equals(userId)) {
//                    // 强制下麦
//                    rtcSeatProvider.leaveSeat(roomId, seatNo, true,AppVer);
//                }
//            }
//            else {
//                QueueZegoVocieTimeoutMsg msg = new QueueZegoVocieTimeoutMsg();
//                msg.setAppVer(AppVer);
//                msg.setRoomID(roomId);
//                msg.setUserID(userId);
//
//                rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.ZEGO_CHECK_VOICE_HOVER, msg.serialize(), System.currentTimeMillis() + TimeConstants.SECOND_15_MILLS);
//            }
        }
    }

    private Long getUserRoomId(Long userId) {
        return Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_USER_ROOM, String.valueOf(userId)))
                .map(Convert::toLong).orElse(0L);
    }
}
