package com.starbuds.server.service.rtc.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.data.UnitTypeEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.rtc.RtcSkillOrderEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.RtcSkillOrderDO;
import com.starbuds.server.common.pojo.db.RtcSkillOrderReadHisDO;
import com.starbuds.server.common.pojo.db.SkillDO;
import com.starbuds.server.common.pojo.message.im.rtc.RtcSkillOrderMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcSkillOrderMsg;
import com.starbuds.server.common.pojo.rtc.RtcSeat;
import com.starbuds.server.common.pojo.rtc.vo.*;
import com.starbuds.server.common.pojo.user.UserBasicInfo;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.rtc.dao.RtcSkillOrderDAO;
import com.starbuds.server.service.rtc.dao.RtcSkillOrderReadHisDAO;
import com.starbuds.server.service.rtc.provider.RtcSkillOrderProvider;
import com.starbuds.server.common.pojo.rtc.qo.RtcSkillOrderPageListQO;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import com.starbuds.server.common.service.rtc.RtcSkillOrderService;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import org.apache.dubbo.config.annotation.Service;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 派单订单表
 */
@Service(version = "1.0.0")
@Slf4j
public class RtcSkillOrderServiceImpl implements RtcSkillOrderService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "orderId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private RtcSkillOrderProvider rtcSkillOrderProvider;

    @Resource
    private RtcSkillOrderDAO rtcSkillOrderDAO;

    @Resource
    private RtcSkillOrderReadHisDAO rtcSkillOrderReadHisDAO;

    @Resource
    RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RtcSeatService rtcSeatService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    ImService imService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public RtcSkillOrderDO create(RtcSkillOrderDO po) {
        return rtcSkillOrderProvider.create(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<RtcSkillOrderDO> list) {
        return rtcSkillOrderProvider.createBatch(list);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(RtcSkillOrderDO po) {
        return rtcSkillOrderProvider.createOrUpdate(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(RtcSkillOrderDO po) {
        return rtcSkillOrderProvider.updateById(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        return rtcSkillOrderProvider.update(uw);
    }

    @Override
    public RtcSkillOrderDO getById(Long id) {
        return rtcSkillOrderProvider.getById(id);
    }

    @Override
    public boolean exist(Long id) {
        return rtcSkillOrderProvider.exist(id);
    }

    @Override
    public PageList<RtcSkillOrderDO> getPageList(RtcSkillOrderPageListQO qo) {
        return rtcSkillOrderProvider.getPageList(qo);
    }

    @Override
    public Map<Long, RtcSkillOrderDO> getMapByIds(Collection ids) {
        return rtcSkillOrderProvider.getMapByIds(ids);
    }

    // 手动编码开始
    @Override
    public PageList<RtcSkillOrderDO> getPageListAsc(RtcSkillOrderPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<RtcSkillOrderDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(rtcSkillOrderDAO.selectPageListAsc(qw));
        page.setTotal(rtcSkillOrderDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始
    @Override
    public PageList<RtcSkillOrderDO> getUserPageList(RtcSkillOrderPageListQO qo, Long userId) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<RtcSkillOrderDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());

        String builder = "(bossUserId = " +
                userId +
                " or serviceUserId = " +
                userId +
                ")";
        qw.apply(builder);
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(rtcSkillOrderDAO.selectUserPageList(qw));
        page.setTotal(rtcSkillOrderDAO.selectUserPageListCount(qw));
        return page;
    }

    /**
     * 创建订单
     *
     * @param skillDO
     * @param roomId
     * @param gender
     * @param price
     * @param quantity
     * @param desc
     * @param bossUserId
     * @param userId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrder(SkillDO skillDO, Long roomId, Integer gender, Long price, Integer quantity, String desc, Long bossUserId, Long userId) {

        long curTime = System.currentTimeMillis();

        RtcSkillOrderDO skillOrderDO = new RtcSkillOrderDO();
        skillOrderDO.setOrderPrice(price);
        skillOrderDO.setOrderNum(quantity);
        skillOrderDO.setCreateTime(curTime);
        skillOrderDO.setOrderDesc(desc);
        skillOrderDO.setUpdateTime(curTime);
        skillOrderDO.setOperatorUserId(userId);
        skillOrderDO.setBossUserId(bossUserId);
        skillOrderDO.setServiceGender(gender);
        skillOrderDO.setSkillIcon(skillDO.getSkillIcon());
        skillOrderDO.setSkillName(skillDO.getSkillName());
        skillOrderDO.setSkillId(skillDO.getSkillId());
        skillOrderDO.setRoomId(roomId);
        skillOrderDO.setStatus(RtcSkillOrderEnum.WAIT.getValue());
        skillOrderDO.setSkillUnit(skillDO.getUnit());

        RtcSkillOrderDO newSkillOrder = create(skillOrderDO);

        if (newSkillOrder == null) {
            return false;
        }

        QueueRtcSkillOrderMsg createMsg = new QueueRtcSkillOrderMsg();
        createMsg.setOrderId(newSkillOrder.getOrderId());
        createMsg.setSkillId(skillDO.getSkillId());
        createMsg.setBossUserId(bossUserId);
        createMsg.setOperatorUserId(userId);
        createMsg.setRoomId(roomId);

        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_SKILL_ORDER_CREATE, createMsg.serialize());
        return true;
    }

    /**
     * 获取房间内的订单
     *
     * @return
     */
    @Override
    public PageList<RtcSkillOnlineOrderVO> getOrderListInRoom(RtcSkillOrderPageListQO pageListQO) {
        pageListQO.setStatus(RtcSkillOrderEnum.WAIT.getValue());

        // 获取分页信息
        PageList<RtcSkillOrderDO> pageList = getPageListAsc(pageListQO);

        List<RtcSkillOrderDO> list = pageList.getList();

        List<RtcSkillOnlineOrderVO> result = new ArrayList<>();

        // 下单人ids
        Set<Long> bossUserIds = list.stream().map(RtcSkillOrderDO::getBossUserId).collect(Collectors.toSet());

        // 批量获取用户信息
        Map<Long, UserBasicInfo> bossUserBaseInfoMap = userService.getBasicInfoMap(bossUserIds);
        // 批量获取用户房间信息
        Map<Long, Long> bossUserInRoom = rtcUserService.getUserRoomIdMap(bossUserIds);

        list.forEach(e -> {

            RtcSkillOnlineOrderVO orderVO = new RtcSkillOnlineOrderVO();
            BeanUtils.copyProperties(e, orderVO);

            RtcSkillOrderBossUserVO userVO = new RtcSkillOrderBossUserVO();
            userVO.setUserId(Optional.ofNullable(bossUserBaseInfoMap.get(e.getBossUserId())).map(UserBasicInfo::getUserId).orElse(null));
            userVO.setUserNo(Optional.ofNullable(bossUserBaseInfoMap.get(e.getBossUserId())).map(UserBasicInfo::getUserNo).orElse(null));
            userVO.setUserAvatar(Optional.ofNullable(bossUserBaseInfoMap.get(e.getBossUserId())).map(UserBasicInfo::getUserAvatar).orElse(null));
            userVO.setUserName(Optional.ofNullable(bossUserBaseInfoMap.get(e.getBossUserId())).map(UserBasicInfo::getUserName).orElse(null));
            userVO.setIsInRoom(Optional.ofNullable(bossUserInRoom.get(e.getBossUserId())).map(i -> i.equals(pageListQO.getRoomId()) ? BoolIntVal.True : BoolIntVal.False).orElse(0));

            orderVO.setUser(userVO);

            result.add(orderVO);
        });

        PageList<RtcSkillOnlineOrderVO> resultPageList = new PageList<>();
        BeanUtils.copyProperties(pageList, resultPageList);
        resultPageList.setList(result);
        return resultPageList;
    }

    /**
     * 获取房间内的订单数量
     *
     * @param roomId
     * @return
     */
    @Override
    public Integer getOrderInRoomCount(Long roomId) {
        return rtcSkillOrderProvider.getOrderInRoomCount(roomId);
    }

    /**
     * 获取大厅内的订单数量
     *
     * @return
     */
    @Override
    public Integer getOrderCreatedCount(Long userId) {
        return rtcSkillOrderProvider.getOrderCreatedCount(userId);
    }

    /**
     * 获取大厅内的订单数量
     *
     * @return
     */
    @Override
    public Integer getAllOrderCreatedCount(Long userId) {
        return rtcSkillOrderProvider.getAllOrderCreatedCount(userId);
    }

    /**
     * 获取最新发布的订单
     *
     * @return
     */
    @Override
    public RtcSkillOrderDO getLastOrderCreated() {

        return rtcSkillOrderProvider.getLastOrderCreated();
    }

    /**
     * 获取房间内待接单人列表
     *
     * @param order
     * @param roomId
     * @return
     */
    @Override
    public List<RtcSkillOrderSeatUserVO> getServiceUserList(RtcSkillOrderDO order, Long roomId) {

        List<RtcSeat> seats = rtcSeatService.getSeats(roomId);
        List<RtcSkillOrderSeatUserVO> result = new ArrayList<>(seats.size());

        seats.forEach(e -> {

            RtcSkillOrderSeatUserVO seatVO = new RtcSkillOrderSeatUserVO();
            RtcSkillOrderServiceUserVO userVO = new RtcSkillOrderServiceUserVO();

            // 麦位禁用状态
            int isDisabled = BoolIntVal.True;

            if (e.getUser() != null) {
                BeanUtils.copyProperties(e.getUser(), userVO);
                // 设置用户底部tag
                if (order.getBossUserId().equals(userVO.getUserId())) {
                    userVO.setUserTag("下单用户");
                } else {
                    // 判断用户是否有技能
                    boolean isMatchedSkill = familyService.isSkillAuth(order.getSkillId(), userVO.getUserId());

                    if (isMatchedSkill) {
                        userVO.setUserTag(order.getSkillName());
                        isDisabled = BoolIntVal.False;
                    } else {
                        userVO.setUserTag("无此技能");
                    }
                }

            }

            seatVO.setUser(userVO);
            seatVO.setSeatNo(e.getSeatNo());
            seatVO.setIsDisabled(isDisabled);

            result.add(seatVO);
        });

        return result;
    }

    /**
     * 结束订单
     *
     * @param serviceUserId
     * @param orderId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean finishOrder(Long serviceUserId, Long orderId) {

        RtcSkillOrderDO order = getById(orderId);

        if (order == null) {
            return false;
        }

        RtcSkillOrderDO updateOrder = new RtcSkillOrderDO();
        updateOrder.setOrderId(orderId);
        updateOrder.setStatus(RtcSkillOrderEnum.FINISH.getValue());
        updateOrder.setServiceUserId(serviceUserId);
        updateOrder.setFinishOrderTime(System.currentTimeMillis());

        boolean flag = false;
        if (updateById(updateOrder)) {

            flag = true;

            QueueRtcSkillOrderMsg createMsg = new QueueRtcSkillOrderMsg();
            createMsg.setOrderId(orderId);
            createMsg.setSkillId(order.getSkillId());
            createMsg.setBossUserId(order.getBossUserId());
            createMsg.setOperatorUserId(order.getOperatorUserId());
            createMsg.setRoomId(order.getRoomId());
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_SKILL_ORDER_FINISH, createMsg.serialize());
        }

        return flag;
    }

    /**
     * 获取订单详情
     *
     * @param order
     * @return
     */
    @Override
    public RtcSkillOrderVO getOrderDetails(RtcSkillOrderDO order) {
        Set<Long> ids = new HashSet<>();
        ids.add(order.getBossUserId());
        ids.add(order.getOperatorUserId());
        ids.add(order.getServiceUserId());
        Map<Long, UserBasicInfo> userBaseInfoMap = userService.getBasicInfoMap(ids);
        return translateDO2VO(order, userBaseInfoMap);
    }

    /**
     * 获取房间内派单记录
     *
     * @return
     */
    @Override
    public PageList<RtcSkillOrderVO> getHistoryListInRoom(RtcSkillOrderPageListQO pageListQO) {
        pageListQO.setStatus(RtcSkillOrderEnum.FINISH.getValue());

        PageList<RtcSkillOrderDO> pageList = getPageList(pageListQO);
        PageList<RtcSkillOrderVO> resultPageList = new PageList<>();
        List<RtcSkillOrderVO> resultList = new ArrayList<>();


        List<RtcSkillOrderDO> list = pageList.getList();
        List<Long> bossUserIds = list.stream().map(RtcSkillOrderDO::getBossUserId).collect(Collectors.toList());
        List<Long> operatorUserIds = list.stream().map(RtcSkillOrderDO::getOperatorUserId).collect(Collectors.toList());
        List<Long> serviceUserIds = list.stream().map(RtcSkillOrderDO::getServiceUserId).collect(Collectors.toList());

        List<Long> userIds = CollectionUtil.addAllIfNotContains(bossUserIds, operatorUserIds);
        List<Long> allUserIds = CollectionUtil.addAllIfNotContains(userIds, serviceUserIds);

        Map<Long, UserBasicInfo> userBaseInfoMap = userService.getBasicInfoMap(CollectionUtil.newHashSet(allUserIds));

        list.forEach(e -> resultList.add(translateDO2VO(e, userBaseInfoMap)));

        BeanUtils.copyProperties(pageList, resultPageList);
        resultPageList.setList(resultList);
        return resultPageList;
    }

    /**
     * 获取用户派单历史
     *
     * @param userId
     * @return
     */
    @Override
    public PageList<RtcSkillOrderVO> getHistoryList(Long userId, RtcSkillOrderPageListQO pageListQO) {
        pageListQO.setStatus(RtcSkillOrderEnum.FINISH.getValue());

        PageList<RtcSkillOrderDO> pageList = getUserPageList(pageListQO, userId);
        PageList<RtcSkillOrderVO> resultPageList = new PageList<>();
        List<RtcSkillOrderVO> resultList = new ArrayList<>();


        List<RtcSkillOrderDO> list = pageList.getList();
        List<Long> bossUserIds = list.stream().map(RtcSkillOrderDO::getBossUserId).collect(Collectors.toList());
        List<Long> operatorUserIds = list.stream().map(RtcSkillOrderDO::getOperatorUserId).collect(Collectors.toList());
        List<Long> serviceUserIds = list.stream().map(RtcSkillOrderDO::getServiceUserId).collect(Collectors.toList());

        List<Long> userIds = CollectionUtil.addAllIfNotContains(bossUserIds, operatorUserIds);
        List<Long> allUserIds = CollectionUtil.addAllIfNotContains(userIds, serviceUserIds);

        Map<Long, UserBasicInfo> userBaseInfoMap = userService.getBasicInfoMap(CollectionUtil.newHashSet(allUserIds));

        list.forEach(e -> resultList.add(translateDO2VO(e, userBaseInfoMap)));

        BeanUtils.copyProperties(pageList, resultPageList);
        resultPageList.setList(resultList);
        return resultPageList;
    }

    /**
     * 关闭订单
     *
     * @param orderId
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean closeOrder(Long orderId) {

        RtcSkillOrderDO order = getById(orderId);

        if (order == null) {
            return false;
        }

        RtcSkillOrderDO updateOrder = new RtcSkillOrderDO();
        updateOrder.setOrderId(orderId);
        updateOrder.setStatus(RtcSkillOrderEnum.CLOSE.getValue());

        boolean flag = false;
        if (updateById(updateOrder)) {

            flag = true;

            QueueRtcSkillOrderMsg createMsg = new QueueRtcSkillOrderMsg();
            createMsg.setOrderId(orderId);
            createMsg.setSkillId(order.getSkillId());
            createMsg.setBossUserId(order.getBossUserId());
            createMsg.setOperatorUserId(order.getOperatorUserId());
            createMsg.setRoomId(order.getRoomId());
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_SKILL_ORDER_CLOSE, createMsg.serialize());
        }

        return flag;
    }


    /**
     * 查询待接单的订单+已完成的订单 总共100条
     */
    @Override
    public PageList<RtcSkillOrderDO> getListInStatus(RtcSkillOrderPageListQO qo) {

        return rtcSkillOrderProvider.getListInStatus(qo);
    }


    /**
     * 插入已读记录
     */
    @Override
    public void addReadHis(Long userId, List<Long> orderIds) {
        List<RtcSkillOrderReadHisDO> list = new ArrayList<>();
        orderIds.forEach(val -> {
            RtcSkillOrderReadHisDO hisDO = new RtcSkillOrderReadHisDO();
            hisDO.setHisId(identityService.getId());
            hisDO.setOrderId(val);
            hisDO.setUserId(userId);
            hisDO.setStatus(1);
            hisDO.setCreateTime(System.currentTimeMillis());
            hisDO.setUpdateTime(System.currentTimeMillis());
            list.add(hisDO);
        });
        rtcSkillOrderReadHisDAO.insertBatch(list);


    }

    /**
     * 刷新用户未读消息数
     */
    @Override
    public void resetUnReadCount(Long userId) {
        RtcSkillOrderMsg orderMsg = new RtcSkillOrderMsg();
        orderMsg.setOrderCount(0);
        imService.sendRtcRoomMsg(ImTargetType.USER, null, userId, null, orderMsg);
        imService.sendCustomImMsg(null, userId, orderMsg, null);
    }


    private RtcSkillOrderVO translateDO2VO(RtcSkillOrderDO order, Map<Long, UserBasicInfo> userBaseInfoMap) {
        RtcSkillOrderVO orderVO = new RtcSkillOrderVO();
        BeanUtils.copyProperties(order, orderVO);

        RtcSkillOrderUserVO bossUserVO = new RtcSkillOrderUserVO();
        RtcSkillOrderUserVO operatorUserVO = new RtcSkillOrderUserVO();
        RtcSkillOrderUserVO serviceUserVO = new RtcSkillOrderUserVO();

        if (userBaseInfoMap.get(order.getBossUserId()) != null) {
            BeanUtils.copyProperties(userBaseInfoMap.get(order.getBossUserId()), bossUserVO);
        }

        if (userBaseInfoMap.get(order.getOperatorUserId()) != null) {
            BeanUtils.copyProperties(userBaseInfoMap.get(order.getOperatorUserId()), operatorUserVO);
        }

        if (userBaseInfoMap.get(order.getServiceUserId()) != null) {
            BeanUtils.copyProperties(userBaseInfoMap.get(order.getServiceUserId()), serviceUserVO);
        }

        orderVO.setBossUser(bossUserVO);
        orderVO.setOperatorUser(operatorUserVO);
        orderVO.setServiceUser(serviceUserVO);
        orderVO.setSkillUnitDesc(UnitTypeEnum.toEnum(order.getSkillUnit()).getDesc());
        return orderVO;
    }
}