package com.starbuds.server.service.rtc.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.TriggerActionTypeEnum;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.rtc.WishStatusEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.rtc.vo.WishRewardVO;
import com.starbuds.server.common.pojo.util.PageQO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.rtc.WishService;
import com.starbuds.server.common.service.rtc.WishStatService;
import com.starbuds.server.service.rtc.dao.WishRewardHisDAO;
import com.starbuds.server.service.rtc.dao.WishUserTaskDAO;
import com.starbuds.server.service.rtc.provider.WishUserTaskProvider;
import com.starbuds.server.common.pojo.rtc.qo.WishUserTaskPageListQO;

import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.starbuds.server.common.service.rtc.WishUserTaskService;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 许愿用户任务表，用户打开许愿任务时创建当天记录
 */
@Service(version = "1.0.0")
@Slf4j
public class WishUserTaskServiceImpl implements WishUserTaskService {

    @Resource
    private WishUserTaskProvider wishUserTaskProvider;

    @Reference(version = "1.0.0")
    private WishService wishService;

    @Resource
    private WishUserTaskDAO wishUserTaskDAO;

    @Resource
    private WishRewardHisDAO wishRewardHisDAO;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Reference(version = "1.0.0")
    private WishStatService wishStatService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    WalletService walletService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public WishUserTaskDO create(WishUserTaskDO po) {
        return wishUserTaskProvider.create(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<WishUserTaskDO> list) {
        return wishUserTaskProvider.createBatch(list);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(WishUserTaskDO po) {
        return wishUserTaskProvider.createOrUpdate(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(WishUserTaskDO po) {
        return wishUserTaskProvider.updateById(po);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        return wishUserTaskProvider.update(uw);
    }

    @Override
    public WishUserTaskDO getById(Long id) {
        return wishUserTaskProvider.getById(id);
    }

    @Override
    public boolean exist(Long id) {
        return wishUserTaskProvider.exist(id);
    }

    @Override
    public PageList<WishUserTaskDO> getPageList(WishUserTaskPageListQO qo) {
        return wishUserTaskProvider.getPageList(qo);
    }

    @Override
    public Map<Long, WishUserTaskDO> getMapByIds(Collection ids) {
        return wishUserTaskProvider.getMapByIds(ids);
    }


    // 手动编码开始

    /**
     * 查询用户当天许愿任务
     */
    @Override
    public List<WishUserTaskDO> getUserCurrentDayList(Long userId) {
        return wishUserTaskProvider.getUserCurrentDayList(userId);
    }

    /**
     * 添加用户当天任务
     */
    @Override
    public List<WishUserTaskDO> addCurrentDayTaskIfNotExist(Long userId) {

        List<WishUserTaskDO> userCurrentDayTasks = getUserCurrentDayList(userId);
        List<WishUserTaskDO> newTasks = new ArrayList<>();
        Map<Long, WishUserTaskDO> userCurrentDayMap = userCurrentDayTasks.stream().collect(Collectors.toMap(WishUserTaskDO::getWishId, Function.identity(), (key1, key2) -> key2));
        List<WishDO> wishs = wishService.getAvailableList();
        wishs.forEach(val -> {
            if (!userCurrentDayMap.containsKey(val.getWishId())) {
                WishUserTaskDO taskDO = new WishUserTaskDO();
                BeanUtils.copyProperties(val, taskDO);

                taskDO.setWishTaskId(identityService.getId());
                taskDO.setCreateTime(System.currentTimeMillis());
                taskDO.setUserId(userId);
                taskDO.setCurrentNum(0);
                taskDO.setStatus(WishStatusEnum.CLOSE.getValue());
                newTasks.add(taskDO);
            }
        });

        if (isDev()) {
            log.info("新添加用户当天任务:{}", JSON.toJSONString(newTasks));
        }

        // 用户当前拥有的任务
        userCurrentDayTasks.addAll(newTasks);

        if (newTasks.size() > 0) {
            if (!wishUserTaskProvider.createBatch(newTasks)) {
                log.error("用户当天新任务添加失败:{}", JSON.toJSONString(newTasks));
            }
        }

        userCurrentDayTasks.sort(Comparator.comparing(WishUserTaskDO::getSort));
        return userCurrentDayTasks;
    }

    /**
     * 更新许愿任务状态
     */
    @Override
    public boolean updateWishTaskStatus(Long wishTaskId, Long userId, Integer status) {
        // 更新任务状态
        boolean flag = false;
        if (status.equals(BoolIntVal.True)) {
            // 如果是开启，需要更新开启时间
            flag = wishUserTaskProvider.updateWishTaskOpenStatus(wishTaskId, userId, System.currentTimeMillis());
        } else {
            flag = wishUserTaskProvider.updateWishTaskStatus(wishTaskId, userId, status);
        }

        // 统计开关记录
        addWishStat(wishTaskId, userId, status);
        return flag;
    }


    /**
     * 更新许愿任务为完成
     */
    @Override
    public boolean updateWishTaskComplete(Long wishTaskId, Long userId) {
        // 更新任务状态

        boolean flag = wishUserTaskDAO.updateWishTaskComplete(wishTaskId, userId, System.currentTimeMillis());

        // 统计开关记录
        addWishStat(wishTaskId, userId, WishStatusEnum.COMPLETE.getValue());
        return flag;
    }

    /**
     * 更新许愿任务进度
     */
    @Override
    public boolean updateWishTask(Long wishTaskId, Long userId, Integer sendGiftNum) {
        // 更新任务进度
        return wishUserTaskDAO.updateWishTask(wishTaskId, userId, sendGiftNum);
    }

    /**
     * 获取用户开启的心愿任务
     */
    @Override
    public WishUserTaskDO getOpenedWishTask(Long userId) {
        return wishUserTaskProvider.getOpenedWishTask(userId);
    }

    /**
     * 获取个人许愿完成的列表
     */
    @Override
    public PageList<WishUserTaskDO> getWishCompleteList(PageQO qo, Long userId) {

        QueryWrapper qw = new QueryWrapper();
        qw.eq("userId", userId);
        qw.eq("status", WishStatusEnum.COMPLETE.getValue());

        PageList<WishUserTaskDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(wishUserTaskDAO.selectPageList(qw));
        page.setTotal(wishUserTaskDAO.selectCount(qw));

        return page;
    }

    /**
     * 批量获取用户开启的许愿任务
     */
    @Override
    public Map<Long, WishUserTaskDO> getOpenedMapByIds(Collection userIds) {
        return wishUserTaskDAO.selectOpenedMapByIds(userIds);
    }

    /**
     * 重新拷贝新的许愿任务
     */
    @Override
    public WishUserTaskDO copyNewTask(WishUserTaskDO userTask) {
        WishUserTaskDO task = new WishUserTaskDO();
        BeanUtils.copyProperties(userTask, task);

        task.setWishTaskId(null);
        task.setStatus(WishStatusEnum.OPEN.getValue());
        task.setCurrentNum(0);
        task.setCloseTime(null);
        task.setCompleteTime(null);
        task.setOpenTime(System.currentTimeMillis());

        return wishUserTaskProvider.create(task);
    }

    /**
     * 将已经绑定的用户任务删除
     */
    @Override
    public boolean deleteTaskByWishId(Long wishId) {

        return wishUserTaskDAO.deleteTaskByWishId(wishId);
    }

    /**
     * 发放奖励
     *
     * @return
     */
    @Override
    public boolean reward(List<WishRewardVO> reward, WishUserTaskDO userTask) {
        Long refId = identityService.getId();
        if (isDev()) {
            log.info("reward:{},userTask:{},许愿任务完成发放奖励", JSON.toJSONString(reward), JSON.toJSONString(userTask));
        }

        List<WishRewardHisDO> hisList = new ArrayList<>();
        reward.forEach(val -> {
            // 创建领取记录
            WishRewardHisDO hisDO = new WishRewardHisDO();
            hisDO.setHisId(identityService.getId());
            hisDO.setRewardType(val.getRewardType());
            hisDO.setWishTaskId(userTask.getWishTaskId());
            hisDO.setWishId(userTask.getWishId());
            hisDO.setCreateTime(System.currentTimeMillis());
            hisDO.setRewardId(val.getRewardId());
            hisDO.setRewardNum(val.getRewardNum());
            hisDO.setUpdateTime(System.currentTimeMillis());
            hisDO.setStatus(BoolIntVal.False);
            hisList.add(hisDO);
        });

        if (isDev()) {
            log.info("hisList:{}", JSON.toJSONString(hisList));
        }

        wishRewardHisDAO.insertBatch(hisList);

        // 领取动作
        for (WishRewardVO item : reward) {
            BagItemTypeEnum triggerActionType = BagItemTypeEnum.toEnum(item.getRewardType());
            if (triggerActionType == null) {
                return false;
            }
            String remarks = String.format("许愿任务%s领取奖励", userTask.getGiftName());
            switch (triggerActionType) {
                case GOLDEN_COIN:
                    walletService.addCoin(userTask.getUserId(), WalletTypeEnum.GOLD_COIN.getValue(), item.getRewardNum().longValue(), remarks, refId);
                    break;
                default:
                    bagService.addItem(userTask.getUserId(), item.getRewardId(), item.getRewardType(), item.getRewardNum(), item.getDuration().equals(-1L) ? -1L : System.currentTimeMillis() + item.getDuration(), remarks, refId);
                    break;
            }

            // 更新领取记录
            WishRewardHisDO hisDO = new WishRewardHisDO();
            hisDO.setStatus(BoolIntVal.True);
            UpdateWrapper uw = new UpdateWrapper();
            uw.setPo(hisDO);
            QueryWrapper qw = new QueryWrapper();
            qw.eq("wishTaskId", userTask.getWishTaskId());
            qw.eq("rewardId", item.getRewardId());
            uw.setQw(qw);
            wishRewardHisDAO.update(uw);
        }

        return true;
    }

    /**
     * 统计许愿任务
     *
     * @param wishTaskId
     * @param userId
     * @param status
     */
    private void addWishStat(Long wishTaskId, Long userId, Integer status) {
        WishStatDO statDO = new WishStatDO();
        statDO.setUserId(userId);
        statDO.setWishTaskId(wishTaskId);
        statDO.setType(status);
        wishStatService.create(statDO);
    }

}