package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.rtc.AgroaRole;
import com.starbuds.server.common.pojo.rtc.RtcAuthInfo;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.rtc.agora.AccessToken;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * 声网 RTC 服务
 *
 * @author Lay
 * @date 2020/7/31
 */
@Component
@Slf4j
public class AgoraRtcProvider {

    @Reference(version = "1.0.0")
    UserService userService;

    @Value("${agora.appId}")
    private String appId;

    @Value("${agora.appCertificate}")
    private String appCertificate;

    /**
     * 获取授权信息
     *
     * @param roomId
     * @param userId
     * @return
     */
    public RtcAuthInfo getAuthInfo(Long roomId, Long userId) {
        Long userNo = userService.getUserNo(userId);
        if (userNo != null && userNo > 0) {
            String channelName = String.format("YL-%d", roomId);
            RtcAuthInfo rtcAuthInfo = new RtcAuthInfo();
            rtcAuthInfo.setChannelName(channelName);
            rtcAuthInfo.setToken(buildToken(channelName, String.valueOf(userNo), AgroaRole.Role_Admin, System.currentTimeMillis() + TimeConstants.DAY_MILLS));
            rtcAuthInfo.setUserNo(Convert.toInt(userNo));
            return rtcAuthInfo;
        }
        return null;
    }

    private String buildToken(String channelName, String account, AgroaRole role, long expireTime) {
        AccessToken builder = new AccessToken(appId, appCertificate, channelName, account);
        int privilegeTs = Convert.toInt(expireTime / 1000, 0);
        builder.addPrivilege(AccessToken.Privileges.kJoinChannel, privilegeTs);
        if (role == AgroaRole.Role_Publisher || role == AgroaRole.Role_Subscriber || role == AgroaRole.Role_Admin) {
            builder.addPrivilege(AccessToken.Privileges.kPublishAudioStream, privilegeTs);
            builder.addPrivilege(AccessToken.Privileges.kPublishVideoStream, privilegeTs);
            builder.addPrivilege(AccessToken.Privileges.kPublishDataStream, privilegeTs);
        }

        try {
            return builder.build();
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }


    public Long getRoomIdByChannelName(String channelName) {
        return Convert.toLong(channelName.replace("YL-", ""));
    }
}
