package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.lang.UUID;
import cn.hutool.crypto.SecureUtil;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.pojo.rtc.AliRtcAuthInfo;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Arrays;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/7/7
 */
@Component
public class AliRtcProvider {
    @Value("${aliyun.rtc.appId}")
    private String appId;

    @Value("${aliyun.rtc.appKey}")
    private String appKey;

    /**
     * 获取授权信息
     *
     * @param roomId
     * @param userId
     * @return
     */
    public AliRtcAuthInfo getAuthInfo(Long roomId, Long userId) {

        String nonce = String.format("AK-%s", UUID.fastUUID().toString(true));

        Long timestamp = System.currentTimeMillis() / 1000L + TimeConstants.DAY_MILLS / 1000L;

        AliRtcAuthInfo authInfo = new AliRtcAuthInfo();
        authInfo.setAppId(appId);
        authInfo.setConferenceId(String.valueOf(roomId));
        authInfo.setNonce(nonce);
        authInfo.setTimeStamp(timestamp);
        authInfo.setUserId(String.valueOf(userId));
        authInfo.setServers(Arrays.asList("https://rgslb.rtc.aliyuncs.com"));
        authInfo.setToken(getToken(authInfo));
        return authInfo;
    }

    private String getToken(AliRtcAuthInfo authInfo) {
        return SecureUtil.sha256(String.format("%s%s%s%s%s%d", authInfo.getAppId(), appKey, authInfo.getConferenceId(), authInfo.getUserId(), authInfo.getNonce(), authInfo.getTimeStamp()));
    }
}
