package com.starbuds.server.service.rtc.provider;

import com.starbuds.server.common.pojo.db.FamilyDO;
import com.starbuds.server.common.pojo.rtc.bo.FamilyRatio;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.rtc.dao.FamilyDao;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;

import static java.math.BigDecimal.ZERO;

/**
 * 家族
 *
 * @author aoe
 * @date 2021/11/5
 */
@Slf4j
@Component
public class FamilyProvider {

    @Resource
    private FamilyDao familyDao;

    /**
     * 家族分成比例信息
     *
     * @param familyId
     * @return
     */
    public FamilyRatio getFamilyRatio(Long familyId) {
        log.info(LogUtil.devLog("家族分成比例信息 familyId={}"), familyId);
        FamilyDO f = familyDao.selectFamilyRatio(familyId);
        if (f == null) {
            return null;
        }
        FamilyRatio bo = new FamilyRatio();
        bo.setFamilyId(familyId);
        Double userRatio = f.getUserRatio();
        Double familyRatio = f.getFamilyRatio();
        Double userSingleRtcRatio = f.getUserSingleRtcRatio();
        Double familySingleRtcRatio = f.getFamilySingleRtcRatio();
        bo.setUserRatio(userRatio == null ? ZERO : BigDecimal.valueOf(userRatio));
        bo.setFamilyRatio(familyRatio == null ? ZERO : BigDecimal.valueOf(familyRatio));
        bo.setUserSingleRtcRatio(userSingleRtcRatio == null ? ZERO : BigDecimal.valueOf(userSingleRtcRatio));
        bo.setFamilySingleRtcRatio(familySingleRtcRatio == null ? ZERO : BigDecimal.valueOf(familySingleRtcRatio));
        return bo;
    }
}
