package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.media.BattleStatusEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rtc.RtcRoomBattleCrossCmdType;
import com.starbuds.server.common.constant.rtc.RtcRoomBattleType;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.RtcBattleRoomCrossDO;
import com.starbuds.server.common.pojo.db.RtcBattleRoomCrossVipScoreDO;
import com.starbuds.server.common.pojo.db.RtcRoomDO;
import com.starbuds.server.common.pojo.message.im.rtc.RtcBattleRoomCrossCmdMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcBattleRoomCrossStatusMsg;
import com.starbuds.server.common.pojo.rtc.RtcBattleStatus;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.rtc.dao.RtcBattleRoomCrossDAO;
import com.starbuds.server.service.rtc.dao.RtcBattleRoomCrossVipScoreDAO;
import com.starbuds.server.service.rtc.dao.RtcRoomDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 房间内 PK 服务
 *
 * @author Lay
 * @date 2021/5/11
 */
@Component
@Slf4j
public class RtcBattleRoomCrossProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    RtcBattleRoomCrossDAO rtcBattleRoomCrossDAO;

    @Resource
    RtcBattleRoomCrossVipScoreDAO rtcBattleRoomCrossVipScoreDAO;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Resource
    RtcBattleRoomPropertyProvider rtcBattleRoomPropertyProvider;

    @Resource
    RtcBattleProvider rtcBattleProvider;

    /**
     * 左侧阵营
     */
    private static final int LEFT_CAMP = -1;

    /**
     * 右侧阵营
     */
    private static final int RIGHT_CAMP = 1;

    /**
     * 增加积分
     *
     * @param roomId
     * @param battleId
     * @param vipUserId
     * @param scoreValue
     */
    public void addScore(Long roomId, Long battleId, Long vipUserId, long scoreValue, long flowValue) {
        if (scoreValue < 1) {
            return;
        }

        RtcBattleStatus battleStatus = rtcBattleProvider.getBattleStatus(roomId);
        if (battleStatus == null || RtcRoomBattleType.ROOM_CROSS != battleStatus.getBattleType()) {
            return;
        }

        RtcBattleRoomCrossDO rtcBattleRoomCross = rtcBattleRoomCrossDAO.selectRoomIds(battleId);
        if (rtcBattleRoomCross == null) {
            return;
        }

        int camp = rtcBattleRoomCross.getRoomId().equals(roomId) ? LEFT_CAMP : RIGHT_CAMP;

        Long[] campVip = null;

        // 增加阵营积分
        String campKey = RedisRtcKeys.getRtcBattleRoomCrossCampScore(battleId);
        String flowKey = RedisRtcKeys.getRtcBattleRoomCrossCampFlow(battleId);

        String vipKey = RedisRtcKeys.getRtcBattleRoomCrossCampVipScore(battleId, camp);

        String vipFlowKey = RedisRtcKeys.getRtcBattleRoomCrossCampVipFlow(battleId, camp);

        // 增加主要贡献者积分
        redisService.zIncrementScore(vipKey, String.valueOf(vipUserId), scoreValue);
        // 增加主要贡献者流水
        redisService.zIncrementScore(vipFlowKey, String.valueOf(vipUserId), flowValue);
        // 增加阵营流水
        redisService.hashIncreaseLong(flowKey, String.valueOf(camp), flowValue);
        long campScore = redisService.hashIncreaseLong(campKey, String.valueOf(camp), scoreValue);

        // VIP 处理
        Set<Tuple> vipSet = redisService.zReverseRangeWithScores(vipKey, 0, 3);
        if (!CollectionUtils.isEmpty(vipSet)) {
            List<Tuple> vipList = new ArrayList<>(vipSet);
            vipList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
            campVip = new Long[vipList.size()];
            int index = 0;
            for (Tuple tuple : vipList) {
                campVip[index] = Convert.toLong(tuple.getElement());
                index++;
            }
        }

        // 同步积分 和 VIP
        if (LEFT_CAMP == camp) {
            rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(roomId, campScore, null, campVip, null);
            rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(battleStatus.getTargetRoomId(), campScore, null, campVip, null);
        } else {
            rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(roomId, null, campScore, null, campVip);
            rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(battleStatus.getTargetRoomId(), null, campScore, null, campVip);
        }
    }

    /**
     * 呼叫超时
     *
     * @param battleId
     */
    public void ringTimeout(Long battleId) {
        // 查询记录
        RtcBattleRoomCrossDO rtcBattleRoomCross = rtcBattleRoomCrossDAO.selectById(battleId);
        if (rtcBattleRoomCross == null) {
            return;
        }

        // 状态判断
        if (BattleStatusEnum.Ringing.getValue() != rtcBattleRoomCross.getBattleStatus()) {
            return;
        }

        if (rtcBattleRoomCrossDAO.updateStatus(battleId, BattleStatusEnum.Timeout.getValue(), BattleStatusEnum.Ringing.getValue(), System.currentTimeMillis()) > 0) {
            // 移除 Redis Keys
            String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(rtcBattleRoomCross.getRoomId());
            String targetBattleStatusKey = RedisRtcKeys.getRtcBattleStatus(rtcBattleRoomCross.getTargetRoomId());
            redisService.delKeys(Arrays.asList(battleStatusKey, targetBattleStatusKey));
            log.info("匹配超时 battleId {}, roomId {}, targetRoomId {}, battleStatusKey {}, targetBattleStatusKey {}", battleId, rtcBattleRoomCross.getRoomId(), rtcBattleRoomCross.getTargetRoomId(), battleStatusKey, targetBattleStatusKey);

            // 放回匹配池
            addToMatchPool(Arrays.asList(rtcBattleRoomCross.getRoomId(), rtcBattleRoomCross.getTargetRoomId()));

            // 发送消息
            RtcBattleRoomCrossCmdMsg cmdMsg = new RtcBattleRoomCrossCmdMsg();
            cmdMsg.setBattleId(battleId);
            cmdMsg.setRoomId(rtcBattleRoomCross.getRoomId());
            cmdMsg.setTargetRoomId(rtcBattleRoomCross.getTargetRoomId());
            cmdMsg.setCmdType(RtcRoomBattleCrossCmdType.Timeout);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, rtcBattleRoomCross.getRoomId(), cmdMsg);
        }
    }

    /**
     * PK 结束
     *
     * @param battleId
     */
    public boolean battleOver(Long battleId) {
        RtcBattleRoomCrossDO rtcBattleRoomCross = rtcBattleRoomCrossDAO.selectById(battleId);
        if (rtcBattleRoomCross == null || BattleStatusEnum.Battling.getValue() != rtcBattleRoomCross.getBattleStatus()) {
            log.error("跨房 PK 结束失败，未找到记录或状态不符 battleId:{}", battleId);
            return false;
        }

        // 更新数据库
        if (rtcBattleRoomCrossDAO.updateStatus(battleId, BattleStatusEnum.Over.getValue(), BattleStatusEnum.Battling.getValue(), System.currentTimeMillis()) > 0) {

            String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(rtcBattleRoomCross.getRoomId());
            String targetBattleStatusKey = RedisRtcKeys.getRtcBattleStatus(rtcBattleRoomCross.getTargetRoomId());
            String battleListenStatusKey = RedisRtcKeys.getRtcBattleListenStatus(rtcBattleRoomCross.getRoomId());
            String targetBattleListenStatusKey = RedisRtcKeys.getRtcBattleListenStatus(rtcBattleRoomCross.getTargetRoomId());

            // 移除缓存
            redisService.delKeys(Arrays.asList(battleStatusKey, targetBattleStatusKey, battleListenStatusKey, targetBattleListenStatusKey));

            // 结算并发送消息
            String campKey = RedisRtcKeys.getRtcBattleRoomCrossCampScore(battleId);
            String flowKey = RedisRtcKeys.getRtcBattleRoomCrossCampFlow(battleId);
            Map<Object, Object> campScoreMap = redisService.hashGetAll(campKey);
            Map<Object, Object> campFlowMap = redisService.hashGetAll(flowKey);
            long leftCampScore = Convert.toLong(campScoreMap.get(String.valueOf(LEFT_CAMP)), 0L);
            long rightCampScore = Convert.toLong(campScoreMap.get(String.valueOf(RIGHT_CAMP)), 0L);

            long leftCampFlow = Convert.toLong(campFlowMap.get(String.valueOf(LEFT_CAMP)), 0L);
            long rightCampFlow = Convert.toLong(campFlowMap.get(String.valueOf(RIGHT_CAMP)), 0L);

            // PK 结果
            int battleResult;
            if (leftCampScore > rightCampScore) {
                battleResult = LEFT_CAMP;
            } else if (rightCampScore > leftCampScore) {
                battleResult = RIGHT_CAMP;
            } else {
                battleResult = 0;
            }

            // 更新结果
            rtcBattleRoomCrossDAO.updateResult(battleId, battleResult, leftCampScore, rightCampScore, leftCampFlow, rightCampFlow);

            // 拉取两边贡献榜前5
            String leftVipKey = RedisRtcKeys.getRtcBattleRoomCrossCampVipScore(battleId, LEFT_CAMP);
            Set<Tuple> leftVipSet = redisService.zReverseRangeWithScores(leftVipKey, 0, -1);

            String leftVipFlowKey = RedisRtcKeys.getRtcBattleRoomCrossCampVipFlow(battleId, LEFT_CAMP);
            Set<Tuple> leftVipFlowSet = redisService.zReverseRangeWithScores(leftVipFlowKey, 0, -1);

            List<Tuple> leftVipList = null;
            Tuple leftMax = null;
            if (!CollectionUtils.isEmpty(leftVipSet)) {
                leftVipList = new ArrayList<>(leftVipSet);
                leftVipList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
                leftMax = leftVipList.get(0);
            } else {
                leftVipList = Collections.emptyList();
            }

            String rightVipKey = RedisRtcKeys.getRtcBattleRoomCrossCampVipScore(battleId, RIGHT_CAMP);
            Set<Tuple> rightVipSet = redisService.zReverseRangeWithScores(rightVipKey, 0, -1);

            String rightVipFlowKey = RedisRtcKeys.getRtcBattleRoomCrossCampVipFlow(battleId, RIGHT_CAMP);
            Set<Tuple> rightVipFlowSet = redisService.zReverseRangeWithScores(rightVipFlowKey, 0, -1);
            List<Tuple> rightVipList = null;
            Tuple rightMax = null;
            if (!CollectionUtils.isEmpty(rightVipSet)) {
                rightVipList = new ArrayList<>(rightVipSet);
                rightVipList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
                rightMax = rightVipList.get(0);
            } else {
                rightVipList = Collections.emptyList();
            }

            // 持久化
            List<RtcBattleRoomCrossVipScoreDO> vipScoreDOList = new ArrayList<>();
            if (!CollectionUtils.isEmpty(leftVipSet)) {
                for (Tuple tuple : leftVipSet) {
                    RtcBattleRoomCrossVipScoreDO vipScoreDO = new RtcBattleRoomCrossVipScoreDO();
                    vipScoreDO.setBattleId(battleId);
                    vipScoreDO.setUserId(Convert.toLong(tuple.getElement()));
                    vipScoreDO.setCamp(LEFT_CAMP);
                    vipScoreDO.setScore(Convert.toLong(tuple.getScore()));
                    vipScoreDO.setRoomId(rtcBattleRoomCross.getRoomId());
                    Double flow = leftVipFlowSet.stream().filter(Objects::nonNull).filter(s -> s.getElement().equals(tuple.getElement())).findFirst().orElse(new Tuple()).getScore();
                    vipScoreDO.setFlow(flow.longValue());
                    vipScoreDOList.add(vipScoreDO);
                }
            }
            if (!CollectionUtils.isEmpty(rightVipSet)) {
                for (Tuple tuple : rightVipSet) {
                    RtcBattleRoomCrossVipScoreDO vipScoreDO = new RtcBattleRoomCrossVipScoreDO();
                    vipScoreDO.setBattleId(battleId);
                    vipScoreDO.setUserId(Convert.toLong(tuple.getElement()));
                    vipScoreDO.setCamp(RIGHT_CAMP);
                    vipScoreDO.setScore(Convert.toLong(tuple.getScore()));
                    vipScoreDO.setRoomId(rtcBattleRoomCross.getTargetRoomId());
                    Double flow = rightVipFlowSet.stream().filter(Objects::nonNull).filter(s -> s.getElement().equals(tuple.getElement())).findFirst().orElse(new Tuple()).getScore();
                    vipScoreDO.setFlow(flow.longValue());
                    vipScoreDOList.add(vipScoreDO);
                }
            }

            if (!CollectionUtils.isEmpty(vipScoreDOList)) {
                rtcBattleRoomCrossVipScoreDAO.batchInsert(vipScoreDOList);
            }

            // MVP 判定
            Long mvpUserId = null;
            Long mvpScore = null;
            if (battleResult == LEFT_CAMP) {
                // 取左阵营最高
                if (leftMax != null) {
                    mvpUserId = Convert.toLong(leftMax.getElement());
                    mvpScore = Convert.toLong(leftMax.getScore());
                }
            } else if (battleResult == RIGHT_CAMP) {
                // 取右阵营最高
                if (rightMax != null) {
                    mvpUserId = Convert.toLong(rightMax.getElement());
                    mvpScore = Convert.toLong(rightMax.getScore());
                }
            } else {
                // 平局 取最高
                if (leftMax != null) {
                    mvpUserId = Convert.toLong(leftMax.getElement());
                    mvpScore = Convert.toLong(leftMax.getScore());
                }

                if (rightMax != null) {
                    if (leftMax == null || rightMax.getScore() > leftMax.getScore()) {
                        mvpUserId = Convert.toLong(rightMax.getElement());
                        mvpScore = Convert.toLong(rightMax.getScore());
                    }
                }
            }

            // 移除缓存
            redisService.delKeys(Arrays.asList(leftVipKey, rightVipKey, campKey, flowKey, leftVipFlowKey, rightVipFlowKey));

            int leftVipSize = Math.min(leftVipList.size(), 5);
            int rightVipSize = Math.min(rightVipList.size(), 5);

            // 用户数据加载
            List<Long> userIds = new ArrayList<>();
            if (leftVipSize > 0) {
                userIds.addAll(leftVipList.subList(0, leftVipSize).stream().map(t -> Convert.toLong(t.getElement())).collect(Collectors.toList()));
            }
            if (rightVipSize > 0) {
                userIds.addAll(rightVipList.subList(0, rightVipSize).stream().map(t -> Convert.toLong(t.getElement())).collect(Collectors.toList()));
            }
            Map<Long, UserCardVO> userMap;
            if (!CollectionUtils.isEmpty(userIds)) {
                userMap = userService.getNameAndAvatarByIds(userIds).stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
            } else {
                userMap = new HashMap<>();
            }

            RtcBattleRoomCrossStatusMsg statusMsg = new RtcBattleRoomCrossStatusMsg();
            statusMsg.setBattleId(battleId);
            statusMsg.setRoomId(rtcBattleRoomCross.getRoomId());
            statusMsg.setTargetRoomId(rtcBattleRoomCross.getTargetRoomId());
            statusMsg.setBattleStatus(BattleStatusEnum.Over.getValue());
            statusMsg.setBattleResult(battleResult);
            statusMsg.setEnableListen(BoolIntVal.False);

            // 拼装用户信息
            if (mvpUserId != null) {
                RtcBattleRoomCrossStatusMsg.BattleVip mvpUser = new RtcBattleRoomCrossStatusMsg.BattleVip();
                UserCardVO mvpUserCard = new UserCardVO();
                BeanUtils.copyProperties(userMap.get(mvpUserId), mvpUserCard);
                mvpUser.setUserCardVO(mvpUserCard);
                mvpUser.setScore(mvpScore);
                statusMsg.setMvpUser(mvpUser);
            }

            if (leftVipSize > 0) {
                List<RtcBattleRoomCrossStatusMsg.BattleVip> leftCampVipList = new ArrayList<>();
                for (int i = 0; i < leftVipSize; i++) {
                    Tuple tuple = leftVipList.get(i);
                    RtcBattleRoomCrossStatusMsg.BattleVip battleVip = new RtcBattleRoomCrossStatusMsg.BattleVip();
                    battleVip.setUserCardVO(userMap.get(Convert.toLong(tuple.getElement())));
                    battleVip.setScore(Convert.toLong(tuple.getScore()));
                    leftCampVipList.add(battleVip);
                }
                statusMsg.setLeftCampVip(leftCampVipList);
            }

            if (rightVipSize > 0) {
                List<RtcBattleRoomCrossStatusMsg.BattleVip> rightCampVipList = new ArrayList<>();
                for (int i = 0; i < rightVipSize; i++) {
                    Tuple tuple = rightVipList.get(i);
                    RtcBattleRoomCrossStatusMsg.BattleVip battleVip = new RtcBattleRoomCrossStatusMsg.BattleVip();
                    battleVip.setUserCardVO(userMap.get(Convert.toLong(tuple.getElement())));
                    battleVip.setScore(Convert.toLong(tuple.getScore()));
                    rightCampVipList.add(battleVip);
                }
                statusMsg.setRightCampVip(rightCampVipList);
            }

            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, rtcBattleRoomCross.getRoomId(), statusMsg);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, rtcBattleRoomCross.getTargetRoomId(), statusMsg);

            // 返回匹配池
            addToMatchPool(Arrays.asList(rtcBattleRoomCross.getRoomId(), rtcBattleRoomCross.getTargetRoomId()));

            // 清除房间属性
            rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(rtcBattleRoomCross.getRoomId(), 0L, 0L, new Long[0], new Long[0]);
            rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(rtcBattleRoomCross.getTargetRoomId(), 0L, 0L, new Long[0], new Long[0]);

            return true;
        }
        return false;
    }

    /**
     * 匹配 PK 对手
     *
     * @param roomId
     * @return
     */
    public Long getRandomBattleRoomId(Long roomId) {
        // 先把自己移除
        removeFromMatchPool(roomId);
        // 匹配一个房间
        Long targetRoomId = randomMatchRoomId(roomId, 0);
        if (targetRoomId == null) {
            addToMatchPool(roomId);
        }
        return targetRoomId;
    }

    private Long randomMatchRoomId(Long roomId, int times) {
        if (times > 5) {
            return null;
        }
        // 随机获取一个PK房间 ID
        log.info("随机匹配[{}] 第{}次", roomId, times);
        Long targetRoomId = Optional.ofNullable(redisService.sRandomMember(RedisRtcKeys.RTC_BATTLE_ROOM_CROSS_POOL)).map(Convert::toLong).orElse(null);
        // 小浣熊跨房PK测试
        if (isDev() && roomId - 454304443924480L == 0) {
            targetRoomId = 454860264701952L;
        }
        if (targetRoomId != null) {
            log.info("匹配到目标 {} vs {}", roomId, targetRoomId);
            // 判断是否在 PK 中
            RtcBattleStatus targetBattleStatus = new RtcBattleStatus(0L, BattleStatusEnum.Matched, RtcRoomBattleType.ROOM_CROSS, roomId);
            RtcBattleStatus battleStatus = new RtcBattleStatus(0L, BattleStatusEnum.Matched, RtcRoomBattleType.ROOM_CROSS, targetRoomId);
            boolean battleStatusSet = redisService.setIfAbsent(RedisRtcKeys.getRtcBattleStatus(roomId), battleStatus.toString());
            boolean targetBattleStatusSet = redisService.setIfAbsent(RedisRtcKeys.getRtcBattleStatus(targetRoomId), targetBattleStatus.toString());
            if (battleStatusSet && targetBattleStatusSet) {
                log.info("匹配成功 {} vs {}", roomId, targetRoomId);
                return targetRoomId;
            } else {
                log.info("匹配失败 battleStatusSet {}, targetBattleStatusSet {}", battleStatusSet, targetBattleStatusSet);
                if (battleStatusSet) {
                    redisService.delKey(RedisRtcKeys.getRtcBattleStatus(roomId));
                }

                if (targetBattleStatusSet) {
                    redisService.delKey(RedisRtcKeys.getRtcBattleStatus(targetRoomId));
                    // 丢回池子
                    addToMatchPool(targetRoomId);
                }

                // 重新找一个
                return randomMatchRoomId(roomId, times + 1);
            }
        }
        log.info("匹配失败");
        return null;
    }

    public void setMatchEnabled(Long roomId, boolean enabled) {
        if (enabled) {
            rtcRoomDAO.updateBattleMatchEnabled(roomId, BoolIntVal.True);
            // 加入匹配池子
            redisService.sAdd(RedisRtcKeys.RTC_BATTLE_ROOM_CROSS_POOL, String.valueOf(roomId));
        } else {
            rtcRoomDAO.updateBattleMatchEnabled(roomId, BoolIntVal.False);
            // 从匹配池移除
            removeFromMatchPool(roomId);
        }
    }

    public Map<Long, Integer> getMatchEnabled(List<Long> roomIds) {
        if (CollectionUtil.isEmpty(roomIds)) {
            return Collections.emptyMap();
        }

        List<RtcRoomDO> list = rtcRoomDAO.selectBattleMatchEnabled(roomIds);
        Map<Long, Integer> enableMap = new HashMap<>();
        for (RtcRoomDO room : list) {
            enableMap.put(room.getRoomId(), Optional.ofNullable(room.getBattleMatchEnabled()).orElse(BoolIntVal.False));
        }
        return enableMap;
    }

    public List<Long> getMatchPoolRoomIds() {
        Set<String> members = redisService.sMembers(RedisRtcKeys.RTC_BATTLE_ROOM_CROSS_POOL);
        if (CollectionUtil.isNotEmpty(members)) {
            return members.stream().map(Convert::toLong).collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 将房间加入匹配池
     *
     * @param roomId
     */
    public void addToMatchPool(Long roomId) {
        addToMatchPool(Collections.singletonList(roomId));
    }

    public void addToMatchPool(List<Long> roomIds) {
        if (CollectionUtil.isEmpty(roomIds)) {
            return;
        }

        // 判断是否开启了允许匹配
        Map<Long, Integer> enabledMap = getMatchEnabled(roomIds);
        if (CollectionUtil.isNotEmpty(enabledMap)) {
            List<String> toBeAdded = enabledMap.entrySet().stream().filter(t -> t.getValue() == BoolIntVal.True).map(t -> String.valueOf(t.getKey())).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(toBeAdded)) {
                redisService.sAdd(RedisRtcKeys.RTC_BATTLE_ROOM_CROSS_POOL, toBeAdded.toArray(new String[0]));
            }
        }
    }

    public void removeFromMatchPool(Long roomId) {
        removeFromMatchPool(Collections.singletonList(roomId));
    }

    public void removeFromMatchPool(List<Long> roomIds) {
        redisService.sRemove(RedisRtcKeys.RTC_BATTLE_ROOM_CROSS_POOL, roomIds.stream().map(String::valueOf).toArray(String[]::new));
    }
}
