package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.starbuds.server.common.pojo.rtc.BattleCampVip;
import com.starbuds.server.common.pojo.user.UserSimpleInfo;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.utils.LogUtil;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.logging.log4j.util.Strings;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.*;

import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 房间 PK 属性服务
 *
 * @author Lay
 * @date 2021/5/11
 */
@Component
public class RtcBattleRoomPropertyProvider {

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    /**
     * PK 状态
     */
    private static final String BATTLE_STATUS = "battleStatus";

    /**
     * 左阵营积分
     */
    private static final String LEFT_CAMP_SCORE = "leftCampScore";

    /**
     * 右阵营积分
     */
    private static final String RIGHT_CAMP_SCORE = "rightCampScore";

    /**
     * 左阵营主要贡献者
     */
    private static final String LEFT_CAMP_VIP = "leftCampVip";

    private static final String LEFT_CAMP_VIP_USERS = "leftCampVipUsers";

    /**
     * 右阵营主要贡献者
     */
    private static final String RIGHT_CAMP_VIP = "rightCampVip";
    private static final String RIGHT_CAMP_VIP_USERS = "rightCampVipUsers";

    public void setBattleStatus(Long roomId, int battleStatus) {
        imService.setRoomProperty(roomId, BATTLE_STATUS, String.valueOf(battleStatus));
    }

    @Async
    public void setCampScoreAndCampVip(Long roomId, Long leftCampScore, Long rightCampScore, Long[] leftCampVip, Long[] rightCampVip) {
        Map<String, String> props = new HashMap<>();
        if (leftCampScore != null && leftCampScore >= 0) {
            props.put(LEFT_CAMP_SCORE, String.valueOf(leftCampScore));
        }
        if (rightCampScore != null && rightCampScore >= 0) {
            props.put(RIGHT_CAMP_SCORE, String.valueOf(rightCampScore));
        }

        String emptyData = "{}";
        if (leftCampVip != null) {
            if (leftCampVip.length > 0) {
                // 排名处理
                Map<Long, Integer> campRank = new HashMap<>();
                for (int i = 0; i < leftCampVip.length; i++) {
                    campRank.put(leftCampVip[i], i + 1);
                }

                props.put(LEFT_CAMP_VIP, StrUtil.join(",", leftCampVip));
                List<Long> userIds = Arrays.asList(leftCampVip);
                Map<Long, UserSimpleInfo> users = userSearchService.getSimpleInfo(userIds);
                Map<Long, BattleCampVip> campVips = new HashMap<>();
                for (Map.Entry<Long, UserSimpleInfo> entry : users.entrySet()) {
                    BattleCampVip battleCampVip = new BattleCampVip(entry.getValue());
                    battleCampVip.setRank(campRank.get(entry.getKey()));
                    campVips.put(entry.getKey(), battleCampVip);
                }
                props.put(LEFT_CAMP_VIP_USERS, toJSON(campVips));
            } else {
                props.put(LEFT_CAMP_VIP, emptyData);
                props.put(LEFT_CAMP_VIP_USERS, emptyData);
            }
        }

        if (rightCampVip != null) {
            if (rightCampVip.length > 0) {
                // 排名处理
                Map<Long, Integer> campRank = new HashMap<>();
                for (int i = 0; i < rightCampVip.length; i++) {
                    campRank.put(rightCampVip[i], i + 1);
                }

                props.put(RIGHT_CAMP_VIP, StrUtil.join(",", rightCampVip));
                List<Long> userIds = Arrays.asList(rightCampVip);
                Map<Long, UserSimpleInfo> users = userSearchService.getSimpleInfo(userIds);
                Map<Long, BattleCampVip> campVips = new HashMap<>();
                for (Map.Entry<Long, UserSimpleInfo> entry : users.entrySet()) {
                    BattleCampVip battleCampVip = new BattleCampVip(entry.getValue());
                    battleCampVip.setRank(campRank.get(entry.getKey()));
                    campVips.put(entry.getKey(), battleCampVip);
                }
                props.put(RIGHT_CAMP_VIP_USERS, toJSON(campVips));
            } else {
                props.put(RIGHT_CAMP_VIP, emptyData);
                props.put(RIGHT_CAMP_VIP_USERS, emptyData);
            }
        }

        if (props.size() > 0) {
            if (isDev()) {
                LogUtil.devLog("清除房间属性-跨房PK " + JSON.toJSONString(props, true));
            }
            imService.setRoomProperties(roomId, props);
        }
    }

    private static String toJSON(Object obj) {
        return JSON.toJSONString(obj, SerializerFeature.WriteNonStringKeyAsString, SerializerFeature.DisableCircularReferenceDetect);
    }
}
