package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.activity.TriggerEventEnum;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.activity.RtcLoveMatchEventParam;
import com.starbuds.server.common.pojo.message.im.rtc.RtcLoveNoticeMsg;
import com.starbuds.server.common.pojo.rtc.RtcLoveChose;
import com.starbuds.server.common.pojo.rtc.RtcLoveMatch;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.TriggerService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.rtc.dao.RtcRoomDAO;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 相亲模式选人服务
 *
 * @author Lay
 * @date 2020/7/22
 */
@Component
public class RtcLoveChooseProvider {
    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    TriggerService triggerService;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Resource
    RtcSeatProvider rtcSeatProvider;

    @Resource
    RtcScoreProvider rtcScoreProvider;

    private static final List<Integer> RED_SEAT_NO = Arrays.asList(1, 2, 5, 6);

    /**
     * 初始化
     *
     * @param roomId
     */
    public void init(Long roomId) {
        redisService.delKey(RedisRtcKeys.getRtcLoveChoose(roomId));
    }

    /**
     * 选人
     *
     * @param roomId
     * @param seatNo
     * @param targetSeatNo
     * @return
     */
    public boolean chooseLove(Long roomId, int seatNo, int targetSeatNo) {
        if (seatNo > 0) {
            redisService.hashPut(RedisRtcKeys.getRtcLoveChoose(roomId), String.valueOf(seatNo), String.valueOf(targetSeatNo));
        }
        return true;
    }

    /**
     * 获取心动对象
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public int getChose(Long roomId, int seatNo) {
        Object targetSeatNo = redisService.hashGet(RedisRtcKeys.getRtcLoveChoose(roomId), String.valueOf(seatNo));
        if (targetSeatNo != null) {
            return Convert.toInt(targetSeatNo);
        }
        return -1;
    }

    public Map<Integer, Integer> getLoveChoseMap(Long roomId) {
        Map<Object, Object> map = redisService.hashGetAll(RedisRtcKeys.getRtcLoveChoose(roomId));
        if (map != null && map.size() > 0) {
            Map<Integer, Integer> loveChoseMap = new HashMap<>();
            for (Map.Entry<Object, Object> entry : map.entrySet()) {
                loveChoseMap.put(Convert.toInt(entry.getKey()), Convert.toInt(entry.getValue()));
            }
            return loveChoseMap;
        }
        return null;
    }

    /**
     * 获取真爱配对
     *
     * @param roomId
     * @return
     */
    public List<RtcLoveMatch> getMatchList(Long roomId) {
        Map<Integer, Integer> loveChoseMap = getLoveChoseMap(roomId);
        if (loveChoseMap != null) {
            List<Integer> choseList = new ArrayList<>();
            for (Map.Entry<Integer, Integer> entry : loveChoseMap.entrySet()) {
                choseList.add(entry.getKey() * 10 + entry.getValue());
            }

            // 升序
            Collections.sort(choseList);

            // 查找反转是否存在，存在则配对成功
            Set<Integer> matched = new HashSet<>();
            for (Integer val : choseList) {
                // 已配对不重复处理
                if (matched.contains(val)) {
                    continue;
                }

                Integer reverseVal = reverseInt(val);
                if (choseList.contains(reverseVal)) {
                    // 配对成功
                    matched.add(reverseVal);
                }
            }

            // 判断是否有匹配成功的
            if (CollectionUtil.isNotEmpty(matched)) {
                List<RtcLoveMatch> list = new ArrayList<>();
                List<Long> userIds = new ArrayList<>();
                // 麦位用户缓存
                Map<Integer, Long> seatUserMap = Optional.ofNullable(rtcSeatProvider.getSeatUserMap(roomId)).orElse(new HashMap<>());

                Map<Long, Long> userScoreMap = Optional.ofNullable(rtcScoreProvider.getUserScoreMap(roomId)).orElse(new HashMap<>());

                for (Integer cmbSeatNo : matched) {
                    int blueSeatNo = cmbSeatNo / 10;
                    int redSeatNo = cmbSeatNo - blueSeatNo * 10;
                    // 红蓝互换
                    if (RED_SEAT_NO.contains(blueSeatNo)) {
                        int tmp = blueSeatNo;
                        blueSeatNo = redSeatNo;
                        redSeatNo = tmp;
                    }

                    // 缓存用户 id
                    Long redUserId = seatUserMap.get(redSeatNo);
                    Long blueUserId = seatUserMap.get(blueSeatNo);
                    if (redUserId != null && blueUserId != null) {
                        // 加入 id 缓存
                        userIds.add(redUserId);
                        userIds.add(blueUserId);

                        // 创建对象
                        RtcLoveMatch loveMatch = new RtcLoveMatch();
                        loveMatch.setBlueUserId(blueUserId);
                        loveMatch.setBlueSeatNo(blueSeatNo);
                        loveMatch.setRedUserId(redUserId);
                        loveMatch.setRedSeatNo(redSeatNo);

                        // 计算心动值总额
                        loveMatch.setTotalScore(Optional.ofNullable(userScoreMap.get(blueUserId)).orElse(0L) + Optional.ofNullable(userScoreMap.get(redUserId)).orElse(0L));

                        // 加入列表
                        list.add(loveMatch);
                    }
                }

                // 获取用户资料并填充
                if (CollectionUtil.isNotEmpty(list)) {
                    Map<Long, UserCardVO> userMap = userService.getNameAndAvatarByIds(userIds).stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
                    for (RtcLoveMatch loveMatch : list) {
                        // 填充
                        UserCardVO blueUser = userMap.get(loveMatch.getBlueUserId());
                        loveMatch.setBlueUserName(blueUser.getUserName());
                        loveMatch.setBlueUserAvatar(blueUser.getUserAvatar());

                        UserCardVO redUser = userMap.get(loveMatch.getRedUserId());
                        loveMatch.setRedUserName(redUser.getUserName());
                        loveMatch.setRedUserAvatar(redUser.getUserAvatar());
                    }
                }

                return list;
            }
        }
        return Collections.emptyList();
    }

    /**
     * 获取选择列表
     *
     * @param roomId
     * @return
     */
    public List<RtcLoveChose> getChoseList(Long roomId) {
        Map<Integer, Integer> loveChoseMap = getLoveChoseMap(roomId);
        if (CollectionUtil.isNotEmpty(loveChoseMap)) {
            Map<Integer, Long> seatUserMap = rtcSeatProvider.getSeatUserMap(roomId);
            List<RtcLoveChose> list = new ArrayList<>();
            for (Map.Entry<Integer, Integer> entry : loveChoseMap.entrySet()) {
                RtcLoveChose loveChose = new RtcLoveChose();
                loveChose.setSeatNo(entry.getKey());
                loveChose.setUserId(seatUserMap.get(entry.getKey()));
                loveChose.setTargetSeatNo(entry.getValue());
                loveChose.setTargetUserId(seatUserMap.get(entry.getValue()));
                list.add(loveChose);
            }
            return list;
        }
        return Collections.emptyList();
    }

    /**
     * 牵手通知
     *
     * @param roomId
     * @param matchList
     */
    @Async
    public void notifyAsync(Long roomId, List<RtcLoveMatch> matchList) {
        String roomName = rtcRoomDAO.selectRoomName(roomId);
        for (RtcLoveMatch loveMatch : matchList) {
            if (loveMatch.getTotalScore() > 100000L) {
                RtcLoveNoticeMsg loveNoticeMsg = new RtcLoveNoticeMsg();
                loveNoticeMsg.setRoomId(roomId);
                loveNoticeMsg.setRoomName(roomName);
                loveNoticeMsg.setRtcLoveMatch(loveMatch);
                imService.sendRtcRoomMsg(ImTargetType.ALL, null, null, null, loveNoticeMsg);
            }

            // 触发事件
            RtcLoveMatchEventParam eventParam = new RtcLoveMatchEventParam();
            eventParam.setRedUserId(loveMatch.getRedUserId());
            eventParam.setBlueUserId(loveMatch.getBlueUserId());
            eventParam.setTotalScore(loveMatch.getTotalScore());
            triggerService.emitEvent(TriggerEventEnum.RTC_LOVE_MATCH, eventParam);
        }
    }

    /**
     * 整数反转
     *
     * @param x
     * @return
     */
    private Integer reverseInt(Integer x) {
        long result = 0;
        while (x != 0) {
            result = result * 10 + x % 10;
            x = x / 10;
        }
        if (result > Integer.MAX_VALUE || result < Integer.MIN_VALUE) {
            return 0;
        }
        return Convert.toInt(result);
    }
}
