package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.RtcLoveScoreHisAssistsDO;
import com.starbuds.server.common.pojo.db.RtcLoveScoreHisDO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.service.rtc.dao.RtcLoveScoreHisAssistsDAO;
import com.starbuds.server.service.rtc.dao.RtcLoveScoreHisDAO;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 相亲房心动值持久化服务
 *
 * @author Lay
 * @date 2020/9/18
 */
@Component
public class RtcLoveScorePersistProvider {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    RtcLoveScoreHisDAO rtcLoveScoreHisDAO;

    @Resource
    RtcLoveScoreHisAssistsDAO rtcLoveScoreHisAssistsDAO;

    @Async
    public void persistAsync(Long roomId, Long beginTime, Map<Long, Set<Tuple>> userAssistsMap) {
        Long endTime = System.currentTimeMillis();
        Long totalScore = 0L;
        Long hisId = identityService.getId();

        if (userAssistsMap.size() > 0) {
            List<RtcLoveScoreHisAssistsDO> list = new ArrayList<>();

            for (Map.Entry<Long, Set<Tuple>> entry : userAssistsMap.entrySet()) {
                for (Tuple tuple : entry.getValue()) {
                    Long assistsUserId = Convert.toLong(tuple.getElement());
                    Long amount = Convert.toLong(tuple.getScore());
                    totalScore += amount;
                    RtcLoveScoreHisAssistsDO assists = new RtcLoveScoreHisAssistsDO();
                    assists.setHisId(hisId);
                    assists.setUserId(entry.getKey());
                    assists.setAssistsAmount(amount);
                    assists.setAssistsUserId(assistsUserId);
                    list.add(assists);
                }
            }

            if (CollectionUtils.isNotEmpty(list)) {
                rtcLoveScoreHisAssistsDAO.batchInsert(list);
            }
        }

        RtcLoveScoreHisDO scoreHis = new RtcLoveScoreHisDO();
        scoreHis.setHisId(hisId);
        scoreHis.setRoomId(roomId);
        scoreHis.setTotalScore(totalScore);
        scoreHis.setBeginTime(beginTime);
        scoreHis.setEndTime(endTime);
        rtcLoveScoreHisDAO.insert(scoreHis);
    }
}
