package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rtc.RtcLoveStage;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.message.im.rtc.RtcVipSeatMsg;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Set;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/7/24
 */
@Component
public class RtcLoveVipProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    RtcSeatProvider rtcSeatProvider;

    @Resource
    RtcLoveStageProvider rtcLoveStageProvider;

    /**
     * VIP 席位最低金额
     */
    private static final long VIP_MIN_AMOUNT = 10000L;

    /**
     * 初始化
     *
     * @param roomId
     */
    public void init(Long roomId, String appVer) {
        // 删除 vip 金额缓存
        redisService.delKey(RedisRtcKeys.getRtcVipAmount(roomId));
        // 强制下麦
        rtcSeatProvider.leaveSeat(roomId, 9, true, appVer);
    }

    /**
     * 增加 vip 金额
     *
     * @param roomId
     * @param userId
     * @param coinAmount
     */
    public void addVipAmount(Long roomId, Long userId, Long coinAmount, String appVer) {
        int userSeat = rtcSeatProvider.getUserSeatNo(roomId, userId);

        // 判断阶段 心动选人之前可以抢夺 vip 并且不在麦上
        if (rtcLoveStageProvider.getStage(roomId) < RtcLoveStage.CHOOSE && userSeat != 0) {
            // 先获取最大金额
            Long vipAmountMax = getVipAmountMax(roomId);
            // 增加金额
            Long vipAmount = Convert.toLong(redisService.zIncrementScore(RedisRtcKeys.getRtcVipAmount(roomId), String.valueOf(userId), Convert.toDouble(coinAmount)));

            // 超过最低金额可以上 vip 席位
            if (vipAmount >= VIP_MIN_AMOUNT) {
                if (vipAmount > vipAmountMax) {
                    if (userSeat != 9) {
                        // 强制下麦
                        rtcSeatProvider.leaveSeat(roomId, 9, true, appVer);

                        // 上 vip 席位
                        if (userSeat == -1) {
                            // 抱上麦
                            rtcSeatProvider.sitSeat(roomId, userId, 9, true, appVer);
                        } else {
                            RtcVipSeatMsg vipSeatMsg = new RtcVipSeatMsg();
                            vipSeatMsg.setRoomId(roomId);
                            vipSeatMsg.setVisible(BoolIntVal.False);
                            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, vipSeatMsg);
                        }
                    }

                }
            }
        }
    }

    /**
     * 获取最大 vip 金额
     *
     * @param roomId
     * @return
     */
    public long getVipAmountMax(Long roomId) {
        Set<Tuple> tupleSet = redisService.zReverseRangeWithScores(RedisRtcKeys.getRtcVipAmount(roomId), 0, 0);
        if (CollectionUtil.isNotEmpty(tupleSet)) {
            Tuple tuple = tupleSet.iterator().next();
            return Convert.toLong(tuple.getScore());
        }
        return 0;
    }

    /**
     * 获取 vip 用户 id
     *
     * @param roomId
     * @return
     */
    public Long getVipUserId(Long roomId) {
        Set<Tuple> tupleSet = redisService.zReverseRangeWithScores(RedisRtcKeys.getRtcVipAmount(roomId), 0, 0);
        if (CollectionUtil.isNotEmpty(tupleSet)) {
            Tuple tuple = tupleSet.iterator().next();
            return Convert.toLong(tuple.getElement());
        }
        return null;
    }
}
