package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.service.data.RedisService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/2
 */
@Component
public class RtcPopularityProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    public void init(Long roomId) {
        remove(roomId);
    }

    public void remove(Long roomId) {
        // 移除
        redisService.zRemove(RedisRtcKeys.RTC_ROOM_POPULARITY, String.valueOf(roomId));
    }

    public Long increase(Long roomId, Long value) {
        return change(roomId, Math.abs(value));
    }

    public Long decrease(Long roomId, Long value) {
        return change(roomId, -Math.abs(value));
    }

    private Long change(Long roomId, Long value) {
        if (value < 0 && Optional.ofNullable(redisService.zScore(RedisRtcKeys.RTC_ROOM_POPULARITY, String.valueOf(roomId))).orElse(0D) < 1) {
            return 0L;
        }
        Long newPopularity = Convert.toLong(redisService.zIncrementScore(RedisRtcKeys.RTC_ROOM_POPULARITY, String.valueOf(roomId), Convert.toDouble(value)));
        if (newPopularity > Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_POPULARITY_MAX, String.valueOf(roomId))).map(Convert::toLong).orElse(0L)) {
            redisService.hashPut(RedisRtcKeys.RTC_POPULARITY_MAX, String.valueOf(roomId), String.valueOf(newPopularity));
        }
        return newPopularity;
    }

    public Long get(Long roomId) {
        return Optional.ofNullable(redisService.zScore(RedisRtcKeys.RTC_ROOM_POPULARITY, String.valueOf(roomId))).map(Convert::toLong).orElse(0L);
    }

    public Long getMax(Long roomId) {
        return Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_POPULARITY_MAX, String.valueOf(roomId))).map(Convert::toLong).orElse(0L);
    }
}
