package com.starbuds.server.service.rtc.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.rtc.RtcAuctionImKey;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.RtcRoomAuctionDO;
import com.starbuds.server.common.pojo.db.RtcRoomAuctionHisDO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.mq.QueueRtcAuctionBidMsg;
import com.starbuds.server.common.pojo.rtc.qo.RtcRoomAuctionPageListQO;
import com.starbuds.server.common.pojo.rtc.vo.RtcRoomAuctionBidRankVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.rtc.dao.RtcRoomAuctionDAO;
import com.starbuds.server.service.rtc.dao.RtcRoomDAO;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;
import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 拍卖房记录
 */
@Component
@Slf4j
public class RtcRoomAuctionProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "rtcRoomAuctionId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private RtcRoomAuctionDAO rtcRoomAuctionDAO;

    @Resource
    private RtcRoomDAO rtcRoomDAO;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Resource
    private RtcRoomAuctionHisProvider rtcRoomAuctionHisProvider;

    @Resource
    private RocketProducer rocketProducer;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private ImService imService;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public RtcRoomAuctionDO create(RtcRoomAuctionDO po) {
        po.setRtcRoomAuctionId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = rtcRoomAuctionDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("拍卖房记录创建成功 {}", json);
            return po;
        }
        log.warn("拍卖房记录创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<RtcRoomAuctionDO> list) {
        boolean flag = rtcRoomAuctionDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(RtcRoomAuctionDO po) {
        boolean flag = rtcRoomAuctionDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(RtcRoomAuctionDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getRtcRoomAuctionId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = rtcRoomAuctionDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public RtcRoomAuctionDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return rtcRoomAuctionDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return rtcRoomAuctionDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<RtcRoomAuctionDO> getPageList(RtcRoomAuctionPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<RtcRoomAuctionDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(rtcRoomAuctionDAO.selectPageList(qw));
        page.setTotal(rtcRoomAuctionDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, RtcRoomAuctionDO> getMapByIds(Collection ids) {
        return rtcRoomAuctionDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 获得正在被竞拍的用户id
     *
     * @param roomId
     * @return 大于0才是正常id
     */
    public long getAuctionUserId(Long roomId) {
        String auctionUserKey = RedisRtcKeys.getRtcAuctionRoomUser(roomId);
        String auctionUserIdStr = redisService.get(auctionUserKey);
        return NumberUtils.toLong(auctionUserIdStr, 0);
    }

    /**
     * 是否竞拍中
     *
     * @param roomId
     * @return true 竞拍中
     */
    public boolean isBidding(Long roomId) {
        return getAuctionUserId(roomId) > 0;
    }

    /**
     * 投标
     *
     * @param roomId            房间id
     * @param orderId           赠送礼物订单id
     * @param rtcRoomAuctionId  拍卖房记录id
     * @param startTime         起拍时间
     * @param bidItemId         拍卖物品id
     * @param sendUserId        发送礼物用户id
     * @param receiveUserId     接收礼物用户id
     * @param giftId            礼物id
     * @param giftPrice         礼物价格
     * @param quantity          礼物数量
     * @param startingGiftPrice 起拍礼物价格
     */
    public void bid(Long roomId, Long orderId, Long rtcRoomAuctionId, Long startTime, Long bidItemId,
                    Long sendUserId, Long receiveUserId,
                    Long giftId, Long giftPrice, Integer quantity, Long startingGiftPrice) {
        if (!checkBid(roomId, orderId, rtcRoomAuctionId,
                sendUserId, receiveUserId,
                giftId, giftPrice, quantity, startingGiftPrice)) {
            return;
        }

        Long giftTotalPrice = giftPrice * quantity;
        RtcRoomAuctionHisDO his = new RtcRoomAuctionHisDO();
        his.setOrderId(orderId);
        his.setRtcRoomAuctionId(rtcRoomAuctionId);
        his.setSendUserId(sendUserId);
        his.setReceiveUserId(receiveUserId);
        his.setGiftId(giftId);
        his.setGiftPrice(giftPrice);
        his.setGiftCount(quantity);
        his.setGiftTotalPrice(giftTotalPrice);
        // 距离起拍时间的间隔时间
        Long timeGap = System.currentTimeMillis() - startTime;
        his.setTimeGap(timeGap);
        his = rtcRoomAuctionHisProvider.create(his);

        if (his == null) {
            String msg = String.format("投标失败，插入 RtcRoomAuctionHisDO 失败 orderId = %s, rtcRoomAuctionId = %s,\n" +
                            "                sendUserId = %s, receiveUserId = %s,\n" +
                            "                giftId = %s, giftPrice = %s, quantity = %s, startingGiftPrice = %s",
                    orderId, rtcRoomAuctionId,
                    sendUserId, receiveUserId,
                    giftId, giftPrice, quantity, startingGiftPrice);
            log.error(msg);
            return;
        }

        // 保存排名
        BigDecimal bidScore = new BigDecimal("" + giftTotalPrice);
        String sendUserIdKey = sendUserId + "";
        redisService.zIncrementScore(RedisRtcKeys.getRtcAuctionRoomRank(roomId), sendUserIdKey, bidScore.doubleValue());
        redisService.hashPut(RedisRtcKeys.getRtcAuctionRoomRankTime(roomId), sendUserIdKey, "" + System.currentTimeMillis());

        List<RtcRoomAuctionBidRankVO> bidRankTops = getBidRankTops(roomId);
        if (!CollectionUtils.isEmpty(bidRankTops)) {
            boolean flag = bidRankTops.stream()
                    .filter(Objects::nonNull)
                    .map(rank -> rank.getUserId())
                    .filter(Objects::nonNull)
                    .anyMatch(userId -> userId - sendUserId == 0);
            if (flag) {
                imService.setRoomProperty(roomId, RtcAuctionImKey.BID_RANK_TOPS_KEY, JSON.toJSONString(bidRankTops));
            }
        }
        if (isDev()) {
            log.info(LogUtil.devLog("竞拍增加分值 userId = {} +{} roomId = {} 竞拍排名 = {}"),
                    sendUserId, bidScore.toPlainString(), roomId, JSON.toJSONString(bidRankTops, true));
        }

        QueueRtcAuctionBidMsg bidMsg = new QueueRtcAuctionBidMsg();
        BeanUtils.copyProperties(his, bidMsg);
        bidMsg.setRoomId(roomId);
        bidMsg.setBidItemId(bidItemId);
        bidMsg.setGiftId(giftId);
        // 丢出拍卖中-投标消息到消息队列
        boolean sendQueueRtcAuctionEndMsg = rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_AUCTION_BID, bidMsg.serialize());
        if (!sendQueueRtcAuctionEndMsg) {
            log.error("拍卖中：丢出拍卖-投标消息到消息队列失败 {}", bidMsg);
        }
        if (isDev()) {
            log.info(LogUtil.devLog("给鹿角的消息 {}"), JSON.toJSONString(bidMsg, true));
        }
    }

    /**
     * 获得竞拍拍卖列表
     *
     * @param roomId
     * @return
     */
    public List<RtcRoomAuctionBidRankVO> getBidRankTops(Long roomId) {
        long initLimit = 2;
        long limit = initLimit;
        String rtcAuctionRoomRankKey = RedisRtcKeys.getRtcAuctionRoomRank(roomId);
        Set<Tuple> tuples = redisService.zReverseRangeWithScores(rtcAuctionRoomRankKey, 0, limit);
        if (CollectionUtils.isEmpty(tuples)) {
            return Collections.emptyList();
        }

        // 有重复数据时，取出所有重复数据比对
        List<Double> scoreList = tuples.stream().map(Tuple::getScore).collect(Collectors.toList());
        boolean containsDuplicateData = containsDuplicateData(scoreList);
        if (containsDuplicateData) {
            Double min = scoreList.stream().min(Double::compareTo).orElse(0D);
            Double max = scoreList.stream().max(Double::compareTo).orElse(Double.valueOf("" + limit));
            limit = redisService.zCount(rtcAuctionRoomRankKey, min, max) - 1;
            if (isDev()) {
                log.info("拍卖出现相同分值 scoreList = {} min = {} max = {} limit = {}",
                        JSON.toJSONString(scoreList), min, max, limit);
            }
        }
        if (limit > initLimit) {
            tuples = redisService.zReverseRangeWithScores(rtcAuctionRoomRankKey, 0, limit);
        }

        List<Tuple> list = new ArrayList<>(tuples);
        long now = System.currentTimeMillis();
        list.forEach(tuple -> tuple.setScore(tuple.getScore() + getLastSendGiftTime(roomId, NumberUtils.toLong(tuple.getElement(), 0), now)));
        list.sort(Comparator.comparingDouble(Tuple::getScore).reversed());

        List<RtcRoomAuctionBidRankVO> tops = list.stream()
                .limit(3)
                .filter(Objects::nonNull)
                .filter(tuple -> tuple.getScore() != null)
                .map(tuple -> {
                    RtcRoomAuctionBidRankVO vo = new RtcRoomAuctionBidRankVO();
                    vo.setUserId(NumberUtils.toLong(tuple.getElement(), 0));
                    vo.setTotalPrice(tuple.getScore().longValue());
                    return vo;
                })
                .filter(vo -> vo.getUserId() > 0)
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(tops)) {
            return Collections.emptyList();
        }

        List<Long> userIds = tops.stream()
                .map(vo -> vo.getUserId())
                .collect(Collectors.toList());
        Map<Long, UserDO> userMap = userService.getUserMapByIds(userIds);

        tops = tops.stream()
                .map(vo -> {
                    UserDO user = userMap.get(vo.getUserId());
                    if (user == null) {
                        return null;
                    }
                    vo.setUserAvatar(user.getUserAvatar());
                    vo.setUserName(user.getUserName());
                    return vo;
                })
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        for (int i = 0; i < tops.size(); i++) {
            tops.get(i).setRank(i + 1);
        }

        return tops;
    }

    /**
     * 竞拍中最后一次送礼物的时间
     *
     * @param roomId
     * @param userId
     * @return
     */
    private double getLastSendGiftTime(Long roomId, Long userId, long now) {
        Object o = redisService.hashGet(RedisRtcKeys.getRtcAuctionRoomRankTime(roomId), "" + userId);
        long lastTime = NumberUtils.toLong("" + o, 0);
        if (lastTime == 0) {
            return 0;
        }
        long time = now - lastTime;
        String timeStr = StringUtils.leftPad(time + "", 13, "0");
        return NumberUtils.toDouble("0." + timeStr);
    }

    /**
     * 检查投标参数
     *
     * @param roomId
     * @param orderId
     * @param rtcRoomAuctionId
     * @param sendUserId
     * @param receiveUserId
     * @param giftId
     * @param giftPrice
     * @param quantity
     * @param startingGiftPrice
     * @return
     */
    private boolean checkBid(Long roomId, Long orderId, Long rtcRoomAuctionId,
                             Long sendUserId, Long receiveUserId,
                             Long giftId, Long giftPrice, Integer quantity, Long startingGiftPrice) {
        boolean flag = true;
        if (roomId == null || orderId == null || rtcRoomAuctionId == null
                || sendUserId == null || receiveUserId == null
                || giftId == null || giftPrice == null || quantity == null || startingGiftPrice == null) {
            flag = false;
        }

        if (Boolean.TRUE != flag) {
            String msg = String.format("投标失败，参数异常 roomId = %s, orderId = %s, rtcRoomAuctionId = %s,\n" +
                            "                sendUserId = %s, receiveUserId = %s,\n" +
                            "                giftId = %s, giftPrice = %s, quantity = %s, startingGiftPrice = %s",
                    roomId, orderId, rtcRoomAuctionId,
                    sendUserId, receiveUserId,
                    giftId, giftPrice, quantity, startingGiftPrice);
            log.error(msg);
        }
        return flag;
    }

    /**
     * 获取最火的竞拍房ID
     *
     * @return
     */
    public Long getHotAuctionRoomId() {
        return rtcRoomDAO.getHotAuctionRoomId();
    }

    /**
     * 是否包含重复的元素
     *
     * @param collection
     * @return
     */
    private boolean containsDuplicateData(List<Double> collection) {
        if (CollectionUtils.isEmpty(collection)) {
            return false;
        }
        return collection.stream()
                .filter(Objects::nonNull)
                .collect(Collectors.groupingBy(Function.identity(), Collectors.counting()))
                .entrySet()
                .stream()
                .anyMatch(entry -> entry.getValue() > 1);
    }
}