package com.starbuds.server.service.rtc.provider;

import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rtc.SingleRtcRoomAnchorStatusEnum;
import com.starbuds.server.common.pojo.data.MedalShinyStatus;
import com.starbuds.server.common.pojo.message.im.live.LiveUserJoinMsg;
import com.starbuds.server.common.pojo.message.im.rtc.SingleRtcLiveUserStatusMsg;
import com.starbuds.server.common.service.data.MountsService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserMedalService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * @author aoe
 * @date 2021/10/9
 */
@Component
@Slf4j
public class RtcRoomProvider {

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private MountsService mountsService;

    @Reference(version = "1.0.0")
    private ImService imService;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Reference(version = "1.0.0")
    private UserMedalService userMedalService;

    /**
     * 发送进入直播间消息
     *
     * @param curTime
     * @param joinUserId
     * @param roomId
     */
    public void sendUserJoinInRoomMsg(Long curTime, Long joinUserId, Long roomId) {
        // 判断是否超管
        if (userService.isOfficial(joinUserId) &&
                redisService.setIfAbsent(RedisRtcKeys.getRtcRoomJoin(roomId, joinUserId), String.valueOf(curTime), TimeConstants.SECOND_15_MILLS, TimeUnit.MILLISECONDS)) {
            return;
        }
        LiveUserJoinMsg liveUserJoinMsg = new LiveUserJoinMsg();
        liveUserJoinMsg.setMounts(mountsService.getUserEnabledMounts(joinUserId));
        MedalShinyStatus medalShinyStatus = userMedalService.getMedalShinyStatus(joinUserId);
        liveUserJoinMsg.setMedalShinyStatus(medalShinyStatus);
        log.info("发送进入直播间消息-开始");
        imService.sendRtcRoomMsg(ImTargetType.GROUP, joinUserId, null, roomId, liveUserJoinMsg);
        log.info("发送进入直播间消息-结束");
    }

    /**
     * 发送个播主播返回直播间消息
     *
     * @param joinUserId
     * @param roomId
     */
    public void sendAnchorBackRoomMsg(Long joinUserId, Long roomId) {
        SingleRtcLiveUserStatusMsg statusMsg = new SingleRtcLiveUserStatusMsg();
        statusMsg.setStatus(SingleRtcRoomAnchorStatusEnum.ONLINE.getValue());
        statusMsg.setUserId(joinUserId);
        statusMsg.setRoomId(roomId);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, joinUserId, null, roomId, statusMsg);
    }
}
