package com.starbuds.server.service.rtc.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.RtcRoomSuggestDO;
import com.starbuds.server.service.rtc.dao.RtcRoomSuggestDAO;
import com.starbuds.server.common.pojo.rtc.qo.RtcRoomSuggestPageListQO;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 推荐语聊房间
 */
@Component
@Slf4j
public class RtcRoomSuggestProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "rtcRoomSuggestId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private RtcRoomSuggestDAO rtcRoomSuggestDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public RtcRoomSuggestDO create(RtcRoomSuggestDO po) {
        po.setRtcRoomSuggestId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        boolean flag = rtcRoomSuggestDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("推荐语聊房间创建成功 {}", json);
            return po;
        }
        log.warn("推荐语聊房间创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<RtcRoomSuggestDO> list) {
        boolean flag = rtcRoomSuggestDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(RtcRoomSuggestDO po) {
        boolean flag = rtcRoomSuggestDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(RtcRoomSuggestDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getRtcRoomSuggestId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = rtcRoomSuggestDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public RtcRoomSuggestDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return rtcRoomSuggestDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return rtcRoomSuggestDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<RtcRoomSuggestDO> getPageList(RtcRoomSuggestPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<RtcRoomSuggestDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(rtcRoomSuggestDAO.selectPageList(qw));
        page.setTotal(rtcRoomSuggestDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, RtcRoomSuggestDO> getMapByIds(Collection ids) {
        return rtcRoomSuggestDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}