package com.starbuds.server.service.rtc.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.WishUserTaskGiftHisDO;
import com.starbuds.server.service.rtc.dao.WishUserTaskGiftHisDAO;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 许愿任务用户送礼表
 */
@Component
@Slf4j
public class WishUserTaskGiftHisProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private WishUserTaskGiftHisDAO wishUserTaskGiftHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public WishUserTaskGiftHisDO create(WishUserTaskGiftHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = wishUserTaskGiftHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("许愿任务用户送礼表创建成功 {}", json);
            return po;
        }
        log.warn("许愿任务用户送礼表创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<WishUserTaskGiftHisDO> list) {
        boolean flag = wishUserTaskGiftHisDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(WishUserTaskGiftHisDO po) {
        boolean flag = wishUserTaskGiftHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(WishUserTaskGiftHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = wishUserTaskGiftHisDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public WishUserTaskGiftHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return wishUserTaskGiftHisDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return wishUserTaskGiftHisDAO.selectCount(qw) > 0;
    }


    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, WishUserTaskGiftHisDO> getMapByIds(Collection ids) {
        return wishUserTaskGiftHisDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}