package com.starbuds.server.service.rtc.rocket.factory;

import com.starbuds.server.common.pojo.media.GiftItemVO;
import com.starbuds.server.common.pojo.message.im.rtc.GiftSettleMsg;
import com.starbuds.server.common.pojo.message.im.rtc.PrivateChatGiftSettleMsg;
import com.starbuds.server.common.pojo.message.im.rtc.VoiceGiftSettleMsg;
import com.starbuds.server.common.pojo.mq.QueueBaseGiftPaidMsg;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.user.UserService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.Optional;

/**
 * 礼物结算消息工厂
 *
 * @author aoe
 * @date 2020/12/22
 */
@Service
public class GiftSettleMsgFactory {

    @Reference(version = "1.0.0")
    private UserService userService;

    @Reference(version = "1.0.0")
    private GiftService giftService;

    /**
     * 创建语音速配礼物支付消息
     *
     * @param msg
     * @param hisId
     * @return
     */
    public VoiceGiftSettleMsg buildVoiceGiftSettleMsg(QueueBaseGiftPaidMsg msg, Long hisId) {
        VoiceGiftSettleMsg giftSettleMsg = new VoiceGiftSettleMsg();
        GiftSettleMsg settleMsg = getGiftSettleMsg(msg, giftSettleMsg);
        BeanUtils.copyProperties(settleMsg, giftSettleMsg);
        giftSettleMsg.setHisId(hisId);
        giftSettleMsg.setReceiveUserId(settleMsg.getTargetId());
        return giftSettleMsg;
    }

    /**
     * 创建私聊礼物结算消息
     *
     * @param msg
     * @return
     */
    public PrivateChatGiftSettleMsg buildPrivateChatGiftSettleMsg(QueueBaseGiftPaidMsg msg) {
        PrivateChatGiftSettleMsg giftSettleMsg = new PrivateChatGiftSettleMsg();
        BeanUtils.copyProperties(getGiftSettleMsg(msg, giftSettleMsg), giftSettleMsg);
        giftSettleMsg.setReward(msg.isReward());
        return giftSettleMsg;
    }

    /**
     * 获得礼物结算消息
     *
     * @param msg
     * @param settleMsg
     * @return
     */
    private GiftSettleMsg getGiftSettleMsg(QueueBaseGiftPaidMsg msg, GiftSettleMsg settleMsg) {
        settleMsg.setGiftId(msg.getGiftId());
        settleMsg.setQuantity(msg.getQuantity());
        settleMsg.setGiftType(msg.getGiftType());
        settleMsg.setGiftPriceType(msg.getGiftPriceType());
        settleMsg.setGiftPrice(msg.getGiftPrice());
        settleMsg.setTargetId(msg.getReceiveUserId());
        settleMsg.setSenderUserId(msg.getSenderUserId());

        UserCardVO receiveCard = userService.getCard(msg.getReceiveUserId());
        UserCardVO senderCard = userService.getCard(msg.getSenderUserId());
        settleMsg.setReceiveUserName(Optional.ofNullable(receiveCard).map(UserCardVO::getUserName).orElse(""));
        settleMsg.setReceiveUserAvatar(Optional.ofNullable(receiveCard).map(UserCardVO::getUserAvatar).orElse(""));
        settleMsg.setSenderUserName(Optional.ofNullable(senderCard).map(UserCardVO::getUserName).orElse(""));
        settleMsg.setSenderUserAvatar(Optional.ofNullable(senderCard).map(UserCardVO::getUserAvatar).orElse(""));
        settleMsg.setFromBag(msg.isFromBag());

        GiftItemVO giftItemVO = giftService.getGift(msg.getGiftId());
        if (giftItemVO != null) {
            settleMsg.setGiftName(giftItemVO.getGiftName());
            settleMsg.setGiftIcon(giftItemVO.getGiftIcon());
        }
        settleMsg.setChannelId(msg.getChannelId());
        settleMsg.setTargetUserName(receiveCard.getUserName());
        settleMsg.setTargetAvatar(receiveCard.getUserAvatar());
        return settleMsg;
    }
}
