package com.starbuds.server.service.rtc.rocket.handler;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.rtc.RtcRoomBattleType;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.LiveService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.service.rtc.provider.RtcBattleRoomCrossProvider;
import com.starbuds.server.service.rtc.provider.RtcBattleRoomInProvider;
import com.starbuds.server.service.rtc.provider.RtcSeatProvider;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Optional;

/**
 * 延时消息处理
 *
 * @author Lay
 * @date 2020/7/10
 */
@Component
public class DelayMsgHandler {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    LiveService liveService;

    @Resource
    RtcBattleRoomInProvider rtcBattleRoomInProvider;

    @Resource
    RtcBattleRoomCrossProvider rtcBattleRoomCrossProvider;

    @Resource
    RtcSeatProvider rtcSeatProvider;

    /**
     * RTC 用户状态检查
     *
     * @param msg
     */
    public void handleRtcCheckStatus(QueueRtcCheckStatusMsg msg) {
        Long roomId = msg.getRoomId();
        Long userId = msg.getUserId();
        String appVer = msg.getAppVer();
        if (Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_CHECK_TIME, RtcSeatProvider.getCheckKey(msg.getRoomId(), msg.getUserId()))).map(Convert::toLong).orElse(0L).equals(msg.getTimestamp())) {
            if (!rtcSeatProvider.isUserInChannel(msg.getRoomId(), msg.getUserId())) {
                // 下麦
                rtcSeatProvider.leaveSeat(roomId, userId, true, appVer);
            } else {
                // 判断用户是否在房间
                if (!rtcUserService.isUserInRoom(roomId, userId)) {
                    // 继续监控
                    rtcSeatProvider.sendCheckStatusMsg(roomId, userId, appVer);
                }
            }
        }
    }

    /**
     * PK 呼叫超时
     *
     * @param msg
     */
    public void handleRtcRoomBattleRingTimeout(QueueRtcBattleRingTimeoutMsg msg) {
        rtcBattleRoomCrossProvider.ringTimeout(msg.getBattleId());
    }

    /**
     * 房间 PK 结束
     *
     * @param msg
     */
    public void handleRtcRoomBattleTimeUp(QueueRtcRoomBattleTimeUpMsg msg) {
        // 判断类型
        if (RtcRoomBattleType.ROOM_IN == msg.getBattleType()) {
            handleRtcBattleRoomInTimeUp(msg);
        } else if (RtcRoomBattleType.ROOM_CROSS == msg.getBattleType()) {
            handleRtcBattleRoomCrossTimeUp(msg);
        }
    }

    /**
     * 房间内 PK 结束
     *
     * @param msg
     */
    private void handleRtcBattleRoomInTimeUp(QueueRtcRoomBattleTimeUpMsg msg) {
        Long roomId = msg.getRoomId();

        // 判断状态
        Long battleId = rtcBattleRoomInProvider.getBattleId(roomId);
        if (battleId != null && battleId.equals(msg.getBattleId())) {
            rtcBattleRoomInProvider.battleOver(roomId, msg.getBattleId());
        }
    }

    /**
     * 跨房 PK 结束
     *
     * @param msg
     */
    private void handleRtcBattleRoomCrossTimeUp(QueueRtcRoomBattleTimeUpMsg msg) {
        rtcBattleRoomCrossProvider.battleOver(msg.getBattleId());
    }

    /**
     * 检查即构主播是否还在麦位上推流
     *
     * @param msg
     */
    public void handleZegoCheckVoice(QueueZegoVocieTimeoutMsg msg) {
        Long userID = msg.getUserID();
        Long roomID = msg.getRoomID();
        String appVer = msg.getAppVer();
        if (liveService.getRemoteUserExistChannel(roomID.toString(), userID, appVer)) {
            return;
        }

        // 判断用户是否还在麦上
        int seatNo = rtcSeatProvider.getUserSeatNo(roomID, userID);
        // 在麦上
        if (seatNo > -1) {
            // 强制下麦
            rtcSeatProvider.leaveSeat(roomID, seatNo, true, appVer);
        }
    }
}
