package com.starbuds.server.service.rtc.rocket.handler;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.data.DataMsgHitBussinessEnum;
import com.starbuds.server.common.constant.data.NoticeScopeTypeEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.constant.media.BattleStatusEnum;
import com.starbuds.server.common.constant.media.GiftTypeEnum;
import com.starbuds.server.common.constant.message.im.ContentType;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rtc.*;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.pojo.data.MedalShinyStatus;
import com.starbuds.server.common.pojo.data.NoticeConfigVO;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.gift.GiftReceiver;
import com.starbuds.server.common.pojo.media.GiftComboConfigData;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.im.rtc.*;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.common.pojo.rtc.RtcBattleStatus;
import com.starbuds.server.common.pojo.rtc.RtcRoom;
import com.starbuds.server.common.pojo.rtc.vo.WishRewardVO;
import com.starbuds.server.common.pojo.user.SkillUser;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.LotteryActivityService;
import com.starbuds.server.common.service.data.*;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.*;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.user.FamilyService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserMedalService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.rtc.dao.RtcRoomDAO;
import com.starbuds.server.service.rtc.dao.RtcRoomJoinHisDAO;
import com.starbuds.server.service.rtc.dao.RtcSkillOrderReadHisDAO;
import com.starbuds.server.service.rtc.provider.*;
import com.starbuds.server.service.rtc.rocket.RocketConfig;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import com.starbuds.server.service.rtc.rocket.factory.GiftSettleMsgFactory;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 普通消息处理
 *
 * @author Lay
 * @date 2020/7/10
 */
@Component
@Slf4j
public class NormalMsgHandler {

    @Value("${spring.application.name}")
    String applicationName;

    @Value("${kuaiyu.sendMsgMaxAppVersion}")
    private int sendMsgMaxAppVersion;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    MountsService mountsService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    GiftService giftService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    LotteryActivityService lotteryActivityService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    WishUserTaskService wishUserTaskService;

    @Reference(version = "1.0.0")
    WishStatService wishStatService;

    @Reference(version = "1.0.0")
    WishUserTaskGiftHisService wishUserTaskGiftHisService;

    @Reference(version = "1.0.0")
    RtcRoomSearchService rtcRoomSearchService;

    @Resource
    RtcHeatV2Provider rtcHeatV2Provider;

    @Resource
    RtcPopularityProvider rtcPopularityProvider;

    @Resource
    RtcScoreProvider rtcScoreProvider;

    @Resource
    RtcRankProvider rtcRankProvider;

    @Resource
    RtcSeatProvider rtcSeatProvider;

    @Resource
    RtcLoveVipProvider rtcLoveVipProvider;

    @Resource
    RtcLoveStageProvider rtcLoveStageProvider;

    @Resource
    RtcQueueProvider rtcQueueProvider;

    @Resource
    RtcLoveScoreProvider rtcLoveScoreProvider;

    @Resource
    RtcBattleProvider rtcBattleProvider;

    @Resource
    RtcBattleRoomInProvider rtcBattleRoomInProvider;

    @Resource
    RtcBattleRoomCrossProvider rtcBattleRoomCrossProvider;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Resource
    RtcRoomJoinHisDAO rtcRoomJoinHisDAO;

    @Resource
    private RtcSkillOrderReadHisDAO rtcSkillOrderReadHisDAO;

    @Resource
    NoticeProvider noticeProvider;

    @Resource
    TrueLoveRankProvider trueLoveRankProvider;

    @Resource
    private GiftSettleMsgFactory giftSettleMsgFactory;

    @Resource
    private RtcRoomAuctionProvider rtcRoomAuctionProvider;

    @Resource
    private RtcSkillOrderProvider rtcSkillOrderProvider;

    @Reference(version = "1.0.0")
    DataMsgHisService dataMsgHisService;

    @Reference(version = "1.0.0")
    RtcRoomLiveOfflineHisService rtcRoomLiveOfflineHisService;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    private RtcRoomProvider rtcRoomProvider;

    @Reference(version = "1.0.0")
    private UserMedalService userMedalService;

    @PostConstruct
    private void init() {
        log.info(LogUtil.devLog("sendMsgMaxAppVersion={}"), sendMsgMaxAppVersion);
    }

    /**
     * 用户加入房间
     *
     * @param msg
     */
    public void handleRtcUserJoin(QueueRtcUserJoinMsg msg) {
        Long roomId = msg.getRoomId();
        Long joinUserId = msg.getJoinUserId();

        // 排查Bug：用户在房间内，在线列表中看不到
        log.info("rong-userJoin-{}-{}-{}-receive", joinUserId, roomId, msg.getAppVer());

        // 缓存直播间资料
        rtcUserService.cacheUserProfile(joinUserId, roomId);

        // 热度处理
        rtcHeatV2Provider.addHeat(roomId, 100);

        // 缓存直播间人数
        rtcPopularityProvider.increase(roomId, 1L);

        // 加入访问记录
        rtcUserService.addVisitHis(roomId, joinUserId, msg.getTimestamp());
        Long curTime = msg.getTimestamp();
        // 发送进入直播间消息
        // 2.0.0之后的版本进入语音房会主动调用接口
        String appVersion = redisService.get(RedisUserKeys.getUserAppVer(joinUserId));
        if (NumberUtils.toInt(StringUtils.replace(appVersion, ".", "")) - sendMsgMaxAppVersion <= 0) {
            rtcRoomProvider.sendUserJoinInRoomMsg(curTime, joinUserId, roomId);
        }

        // 判断是否今日首次进房
        int todayTimes = rtcUserService.getRtcUserJoinDayBefore(msg.getJoinUserId(), null, msg.getTimestamp());
        if (todayTimes == 0) {
            // 进房统计
            statPlatformService.addPlatformStat(msg.getChannelId(), StatPlatformTypeEnum.JOIN_RTC_ROOM_USERS, BigDecimal.ONE);
        }

        // 相亲模式
        if (RoomTypeEnum.LOVE.getValue() == rtcRoomDAO.selectRoomType(roomId)) {
            if (rtcLoveStageProvider.getStage(roomId) < RtcLoveStage.CHOOSE) {
                Long vipUserId = rtcLoveVipProvider.getVipUserId(roomId);
                if (vipUserId != null && vipUserId.equals(joinUserId)) {
                    rtcSeatProvider.sitSeat(roomId, joinUserId, 9, true, msg.getAppVer());
                }
            }
        }

        // 创建进房记录
        String hisId = SecureUtil.md5(String.format("%d:%d:%d:%d", joinUserId, roomId, BoolIntVal.True, curTime));
        RtcRoomJoinHisDO rtcRoomJoinHis = new RtcRoomJoinHisDO();
        rtcRoomJoinHis.setHisId(hisId);
        rtcRoomJoinHis.setRoomId(roomId);
        rtcRoomJoinHis.setUserId(joinUserId);
        rtcRoomJoinHis.setChannelId(msg.getChannelId());
        rtcRoomJoinHis.setJoinType(BoolIntVal.True);
        rtcRoomJoinHis.setHisCreateTime(curTime);
        rtcRoomJoinHisDAO.insert(rtcRoomJoinHis);

        // 头饰检查
        headwearService.checkEnabledHeadwearStatus(joinUserId);


        // 个播房主进房消息
        if (RtcBusinessTypeEnum.RTC_SINGLE.getValue() == rtcRoomDAO.selectRoomBusinessType(roomId)) {
            if (joinUserId.equals(rtcRoomDAO.selectUserId(roomId))) {
                rtcRoomService.sendAnchorBackRoomMsg(joinUserId, roomId);
            }
        }
    }

    /**
     * 用户离开房间
     *
     * @param msg
     */
    public void handleRtcUserExit(QueueRtcUserExitMsg msg) {

        if (isDev()) {
            LogUtil.devLog("用户离开房间收到mq消息:" + JSON.toJSONString(msg));
        }

        Long roomId = msg.getRoomId();
        Long exitUserId = msg.getExitUserId();

        // 移除直播间资料
        if (rtcUserService.removeUserProfile(exitUserId, roomId)) {
            // 减热度
            rtcHeatV2Provider.addHeat(roomId, -100);

            // 缓存直播间人数
            rtcPopularityProvider.decrease(roomId, 1L);

            // 移除麦序
            rtcQueueProvider.removeUser(roomId, exitUserId);

            // 创建出房记录
            Long curTime = msg.getTimestamp();
            String hisId = SecureUtil.md5(String.format("%d:%d:%d:%d", exitUserId, roomId, BoolIntVal.True, curTime));
            RtcRoomJoinHisDO rtcRoomJoinHis = new RtcRoomJoinHisDO();
            rtcRoomJoinHis.setHisId(hisId);
            rtcRoomJoinHis.setRoomId(roomId);
            rtcRoomJoinHis.setUserId(exitUserId);
            rtcRoomJoinHis.setChannelId(msg.getChannelId());
            rtcRoomJoinHis.setJoinType(BoolIntVal.False);
            rtcRoomJoinHis.setHisCreateTime(curTime);
            rtcRoomJoinHisDAO.insert(rtcRoomJoinHis);

            // 个播房发送主播离线消息
            rtcRoomService.dealSingleRoomOffline(roomId, exitUserId);
        }
    }

    @Resource
    RocketConfig rocketConfig;


    /**
     * 礼物支付
     *
     * @param msg
     */
    public void handleRtcGiftPaid(QueueRtcGiftPaidMsg msg) {

        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.RTC_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        if (isDev()) {
            log.info("礼物支付 {}", JSON.toJSONString(msg, true));
        }
        // 查询礼物
        GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);

        // 查询房间
        RtcRoom rtcRoom = rtcRoomDAO.selectById(msg.getRoomId());

        // 是否相亲房
        boolean isLoveRoom = RoomTypeEnum.LOVE.getValue() == rtcRoomDAO.selectRoomType(msg.getRoomId());

        // 目标用户
        List<Long> userIds = new ArrayList<>(msg.getTargetUserSeat().keySet());

        // 连击处理
        boolean isFromLottery = msg.isFromLottery();
        ComboData comboData = null;
        if (!isFromLottery) {
            comboData = processCombo(msg.getSenderUserId(), msg.getRoomId(), msg.getGiftId(), giftDO.getGiftPrice(), msg.getQuantity(), BoolIntVal.True == msg.getAllSeat() ? Collections.singletonList((long) -userIds.size()) : userIds);
        }

        // 发送直播间消息
        RtcGiftSettleMsg rtcGiftSettleMsg = new RtcGiftSettleMsg();
        rtcGiftSettleMsg.setRoomId(msg.getRoomId());
        rtcGiftSettleMsg.setGiftId(msg.getGiftId());
        rtcGiftSettleMsg.setQuantity(msg.getQuantity());
        rtcGiftSettleMsg.setSenderSeatNo(rtcSeatProvider.getUserSeatNo(msg.getRoomId(), msg.getSenderUserId()));
        rtcGiftSettleMsg.setTargetSeats(new ArrayList<>(msg.getTargetUserSeat().values()));
        List<UserCardVO> targetUsers = userService.getNameAndAvatarByIds(userIds);
        if (targetUsers.size() > 1) {
            Collections.shuffle(targetUsers);
        }
        Optional<UserCardVO> randomTargetUser = targetUsers.stream().findFirst();
        if (randomTargetUser.isPresent()) {
            rtcGiftSettleMsg.setRandomTargetUserAvatar(randomTargetUser.get().getUserAvatar());
        }
        rtcGiftSettleMsg.setTargetUsers(targetUsers.stream().collect(Collectors.toMap(UserCardVO::getUserId, UserCardVO::getUserName)));
        rtcGiftSettleMsg.setAllSeat(msg.getAllSeat());
        if (isFromLottery) {
            rtcGiftSettleMsg.setIsCombo(false);
            rtcGiftSettleMsg.setShowInComment(true);
            rtcGiftSettleMsg.setCombo(1L);
        } else {
            rtcGiftSettleMsg.setIsCombo(true);
            rtcGiftSettleMsg.setCombo(comboData.getCombo());
            rtcGiftSettleMsg.setComboKeyHash(SecureUtil.md5(comboData.getComboKey()));
            rtcGiftSettleMsg.setIsComboTimeout(false);
            rtcGiftSettleMsg.setComboPeriodSec(COMBO_TIMEOUT);
            rtcGiftSettleMsg.setComboAnimationId(comboData.getAnimationId());
            rtcGiftSettleMsg.setComboAnimationLevel(comboData.getLevel());

            // 弹幕礼物策略
            boolean showInComment = false;
            if (BoolIntVal.True == Optional.ofNullable(giftDO.getIsComment()).orElse(BoolIntVal.False)) {
                boolean comboFirst = comboData.getCombo() == 1 && BoolIntVal.True == Optional.ofNullable(giftDO.getCommentFirstCombo()).orElse(BoolIntVal.False);
                Integer commentComboMultiple = Optional.ofNullable(giftDO.getCommentComboMultiple()).orElse(0);
                boolean comboMultiple = commentComboMultiple > 0 && comboData.getCombo() % commentComboMultiple == 0;
                if (comboFirst || comboMultiple) {
                    showInComment = true;
                }
            }
            rtcGiftSettleMsg.setShowInComment(showInComment);
        }

        MedalShinyStatus medalShinyStatus = userMedalService.getMedalShinyStatus(msg.getSenderUserId());
        rtcGiftSettleMsg.setMedalShinyStatus(medalShinyStatus);
        // 发送礼物消息
        imService.sendRtcRoomMsg(ImTargetType.GROUP, msg.getSenderUserId(), null, msg.getRoomId(), rtcGiftSettleMsg);

        // 热度处理
        if (giftDO.getHeatValue() != null && giftDO.getHeatValue() > 0 && Optional.ofNullable(giftDO.getHeatDuration()).orElse(0L) > 0) {
            long heatValue = (long) giftDO.getHeatValue() * msg.getQuantity();
            rtcHeatV2Provider.addHeatItem(msg.getRoomId(), heatValue, msg.getTimestamp() + giftDO.getHeatDuration());
        }

        // 心动值处理
        Long score = giftDO.getRtcScore() * msg.getQuantity();
        for (Map.Entry<Long, Integer> entry : msg.getTargetUserSeat().entrySet()) {
            rtcScoreProvider.addScore(msg.getRoomId(), entry.getKey(), score);
            if (isLoveRoom) {
                rtcLoveScoreProvider.addScore(msg.getRoomId(), entry.getKey(), score, msg.getSenderUserId());
            }
        }

        // PK 值处理
        Long roomId = msg.getRoomId();
        RtcBattleStatus battleStatus = rtcBattleProvider.getBattleStatus(roomId);
        if (battleStatus != null && BattleStatusEnum.Battling == battleStatus.getBattleStatus()) {
            long scoreValue = giftDO.getBattleValue() * msg.getQuantity();
            long flowValue = giftDO.getGiftPrice() * msg.getQuantity();
            if (RtcRoomBattleType.ROOM_IN == battleStatus.getBattleType()) {
                rtcBattleRoomInProvider.batchAddScore(roomId, battleStatus.getBattleId(), msg.getSenderUserId(), msg.getTargetUserSeat(), scoreValue, flowValue);
            } else if (RtcRoomBattleType.ROOM_CROSS == battleStatus.getBattleType()) {
                rtcBattleRoomCrossProvider.addScore(roomId, battleStatus.getBattleId(), msg.getSenderUserId(), scoreValue * msg.getTargetUserSeat().size(), flowValue);
            }
        }

        // 星钻榜处理
        if (GiftTypeEnum.COMMON.getValue() == giftDO.getGiftType()) {
            Long coinAmount = msg.getTargetUserSeat().size() * msg.getQuantity() * msg.getGiftPrice();
            rtcRankProvider.addDiamondRankAmount(msg.getRoomId(), msg.getSenderUserId(), coinAmount);
            // 星尘榜
            for (Long userId : msg.getTargetUserSeat().keySet()) {
                rtcRankProvider.addDustRankAmount(msg.getRoomId(), userId, msg.getQuantity() * msg.getGiftPrice());
            }

            // 相亲房
            if (isLoveRoom) {
                // 增加金额
                rtcLoveVipProvider.addVipAmount(msg.getRoomId(), msg.getSenderUserId(), coinAmount, msg.getAppVer());
            }

            // 拍卖房
            if (RoomTypeEnum.toEnum(rtcRoom.getRoomType()) == RoomTypeEnum.AUCTION) {
                doAuctionRoom(roomId, msg.getTargetUserSeat().keySet(), giftDO, msg.getQuantity(), msg.getOrderId(), msg.getSenderUserId());
            }
        }

        // 真爱榜礼物
        if (giftService.isTrueLoveGift(msg.getGiftId())) {
            List<GiftLoveHisDO> list = new ArrayList<>();

            for (Long userId : msg.getTargetUserSeat().keySet()) {
                GiftLoveHisDO hisDO = new GiftLoveHisDO();
                hisDO.setSenderUserId(msg.getSenderUserId());
                hisDO.setReceiveUserId(userId);
                hisDO.setCreateTime(curTime);
                hisDO.setGiftId(msg.getGiftId());
                hisDO.setQuantity(Convert.toLong(msg.getQuantity()));
                list.add(hisDO);
            }
            trueLoveRankProvider.batchInsert(list);
        }

        // 广播礼物判断
        if (BoolIntVal.True == giftDO.getIsNotice() && Optional.ofNullable(giftDO.getNoticeStyle()).orElse(0) > 0) {
            // 赠送者
            UserCardVO card = userService.getCard(msg.getSenderUserId());
            if (card != null) {
                if (CollectionUtil.isNotEmpty(rtcGiftSettleMsg.getTargetUsers())) {
                    // 收礼人
                    Map<Long, UserCardVO> cardMap = userService.getCardMap(new HashSet<>(userIds));
                    List<String> receiveUserNames = new ArrayList<>();
                    for (Long targetId : userIds) {
                        UserCardVO userCardVO = cardMap.get(targetId);
                        if (userCardVO == null) {
                            continue;
                        }
                        receiveUserNames.add(userCardVO.getUserName());
                    }
                    RtcGiftNoticeMsg rtcGiftNoticeMsg = new RtcGiftNoticeMsg();
                    rtcGiftNoticeMsg.setGiftId(msg.getGiftId());
                    rtcGiftNoticeMsg.setGiftName(giftDO.getGiftName());
                    rtcGiftNoticeMsg.setRoomId(msg.getRoomId());
                    rtcGiftNoticeMsg.setRoomName(rtcRoom.getRoomName());
                    rtcGiftNoticeMsg.setQuantity(msg.getQuantity());
                    rtcGiftNoticeMsg.setSenderUserName(card.getUserName());
                    rtcGiftNoticeMsg.setSenderUserAvatar(card.getUserAvatar());
                    rtcGiftNoticeMsg.setReceiveUserId(BoolIntVal.True == msg.getAllSeat() ? -1L : userIds.get(0));
                    rtcGiftNoticeMsg.setReceiveUserName(BoolIntVal.True == msg.getAllSeat() ? "全麦" : CollectionUtil.join(receiveUserNames, "，"));

                    rtcGiftNoticeMsg.setNoticeStyle(giftDO.getNoticeStyle());
                    // 首次显示连击
                    if (comboData != null) {
                        rtcGiftNoticeMsg.setCombo(comboData.getCombo());
                    }
                    rtcGiftNoticeMsg.setContent(String.format("%s 送出 %sx%d给%s", card.getUserName(), giftDO.getGiftName(), msg.getQuantity(), rtcGiftNoticeMsg.getReceiveUserName()));
                    log.info("rtcGiftNoticeMsg {}", JSON.toJSONString(rtcGiftNoticeMsg));
                    imService.sendRtcRoomMsg(ImTargetType.ALL, msg.getSenderUserId(), null, msg.getRoomId(), rtcGiftNoticeMsg);
                }
            }
        }

        if (giftDO.getGiftType().equals(GiftTypeEnum.LOTTERY.getValue())) {
            // 抽奖礼物
            String lotteryId = giftDO.getLotteryId();
            if (StringUtils.isNotEmpty(lotteryId)) {
                // 抽奖并赠送
                lotteryActivityService.lottery(lotteryId, msg.getSenderUserId(), msg.getQuantity(), msg.getChannelId(),
                        msg.getRoomId(), msg.getTargetUserSeat(), msg.getOrderId());
            }
        }

        // 处理许愿用户任务逻辑
        dealWishTask(userIds, msg.getGiftId(), msg.getQuantity(), msg.getSenderUserId(), 1);

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }


    /**
     * 许愿任务处理
     */
    public void dealWishTask(List<Long> userIds, Long giftId, Integer sendGiftNum, Long senderUserId, Integer type) {
        if (isDev()) {
            log.info("userIds:{},开始处理许愿任务", JSON.toJSONString(userIds));
        }
        // 许愿礼物任务判断
        Map<Long, WishUserTaskDO> taskMap = wishUserTaskService.getOpenedMapByIds(userIds);
        if (isDev()) {
            log.info("taskMap:{},获取进行中的许愿任务", JSON.toJSONString(taskMap));
        }
        userIds.forEach(val -> {
            if (taskMap.containsKey(val)) {

//                WishUserTaskDO userTask = taskMap.get(val);
//                if (userTask == null) {
//                    userTask = new WishUserTaskDO();
//                }
                WishUserTaskDO userTask = JSON.parseObject(JSON.toJSONString(taskMap.get(val)), WishUserTaskDO.class);
                if (userTask == null) {
                    userTask = new WishUserTaskDO();
                }
                if (isDev()) {
                    log.info("userTask:{},获取进行中的许愿任务2", JSON.toJSONString(userTask));
                }
                // 送的礼物跟任务里礼物相同时，进行任务操作
                if (giftId.equals(userTask.getGiftId())) {

                    // 处理任务进度
                    boolean completeFlag = false;
                    Integer restNum = userTask.getGiftNum() - userTask.getCurrentNum();
                    if (sendGiftNum >= restNum) {
                        // 任务完成
                        completeFlag = true;
                    }

                    if (isDev()) {
                        log.info("completeFlag:{},获取进行中的许愿任务3", completeFlag);
                    }

                    // 记录送礼记录his
                    WishUserTaskGiftHisDO hisDO = new WishUserTaskGiftHisDO();
                    hisDO.setGiftNum(sendGiftNum);
                    hisDO.setGiftId(giftId);
                    hisDO.setWishTaskId(userTask.getWishTaskId());
                    hisDO.setCreateTime(System.currentTimeMillis());
                    hisDO.setReceiveUserId(userTask.getUserId());
                    hisDO.setSendUserId(senderUserId);
                    hisDO.setSendType(type);
                    wishUserTaskGiftHisService.create(hisDO);
                    if (isDev()) {
                        log.info("hisDO:{},插入送礼记录", JSON.toJSONString(hisDO));
                    }
                    if (completeFlag) {
                        wishTaskComplete(userTask);
                    } else {
                        wishUserTaskService.updateWishTask(userTask.getWishTaskId(), val, sendGiftNum);
                    }
                }
            }
        });
    }


    /**
     * 许愿任务完成
     *
     * @param userTask
     */
    private void wishTaskComplete(WishUserTaskDO userTask) {
        // 如果完成，要修改状态为完成，
        // 如果完成，要发奖励，并且系统通知，并且重新生成任务
        if (isDev()) {
            log.info("userTask:{},许愿任务完成处理逻辑", JSON.toJSONString(userTask));
        }
        // 修改状态为完成
        wishUserTaskService.updateWishTaskComplete(userTask.getWishTaskId(), userTask.getUserId());

        // 并添加统计
        WishStatDO statDO = new WishStatDO();
        statDO.setUserId(userTask.getUserId());
        statDO.setWishTaskId(userTask.getWishTaskId());
        statDO.setType(WishStatusEnum.COMPLETE.getValue());
        wishStatService.create(statDO);
        if (isDev()) {
            log.info("statDO:{},许愿任务完成添加统计", JSON.toJSONString(statDO));
        }

        List<WishRewardVO> reward = JSON.parseArray(userTask.getWishReward(), WishRewardVO.class);

        wishUserTaskService.reward(reward, userTask);
        if (isDev()) {
            log.info("reward:{},许愿任务完成发放奖励", JSON.toJSONString(reward));
        }

        // 个人自定义消息
        WishTaskCompleteMsg msg = new WishTaskCompleteMsg();
        msg.setGiftName(userTask.getGiftName());
        msg.setRewards(reward);
        imService.sendCustomImMsg(null, userTask.getUserId(), msg, null);
        if (isDev()) {
            log.info("msg:{},许愿任务完成个人自定义消息", JSON.toJSONString(msg));
        }
        // 个人系统通知
        SystemMsg systemMsg = new SystemMsg();
        systemMsg.setContentType(ContentType.TEXT);
        systemMsg.setTitle("许愿通知");
        List<String> rewardTxts = reward.stream().map(val -> val.getRewardName() + "*" + val.getRewardNum()).collect(Collectors.toList());
        String body = String.format("完成%s任务，奖励%s", userTask.getGiftName(), StringUtils.join(rewardTxts, ","));
        systemMsg.setBody(body);
        systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
        systemMsg.setModuleId(userTask.getUserId().toString());
        imService.sendCustomImMsg(null, userTask.getUserId(), systemMsg, null);
        if (isDev()) {
            log.info("systemMsg:{},许愿任务完成个人系统通知", JSON.toJSONString(systemMsg));
        }
        // 重新生成任务
        WishUserTaskDO newTaskDO = wishUserTaskService.copyNewTask(userTask);

        if (isDev()) {
            log.info("newTaskDO:{},许愿任务完成重新生成任务", JSON.toJSONString(newTaskDO));
        }
    }

    /**
     * 处理语聊房麦下送礼物支付消息
     *
     * @param msg
     */
    public void handleRtcGiftUnderMaiPaid(QueueRtcGiftUnderMaiPaidMsg msg) {

        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.RTC_UNDER_MAI_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }


        // 查询礼物
        GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);
        if (giftDO == null) {
            log.error("未找到礼物");
            return;
        }

        // 查询房间
        RtcRoom rtcRoom = rtcRoomDAO.selectById(msg.getRoomId());
        if (rtcRoom == null) {
            log.error("未找到房间");
            return;
        }

        // 是否相亲房
        boolean isLoveRoom = RoomTypeEnum.LOVE.getValue() == rtcRoomDAO.selectRoomType(msg.getRoomId());

        // 目标用户
        List<Long> userIds = msg.getReceiverUsers().stream().map(GiftReceiver::getUserId).collect(Collectors.toList());

        // 发送直播间消息
        RtcGiftSettleMsg rtcGiftSettleMsg = new RtcGiftSettleMsg();
        rtcGiftSettleMsg.setRoomId(msg.getRoomId());
        rtcGiftSettleMsg.setGiftId(msg.getGiftId());
        rtcGiftSettleMsg.setQuantity(msg.getQuantity());
        rtcGiftSettleMsg.setIsCombo(false);
        rtcGiftSettleMsg.setShowInComment(true);
        rtcGiftSettleMsg.setCombo(1L);

        // 麦下送礼物
        // 麦下送礼物座位号
        Integer underMaiSeatNo = -1;
        rtcGiftSettleMsg.setSenderSeatNo(underMaiSeatNo);
        rtcGiftSettleMsg.setTargetSeats(Collections.EMPTY_LIST);

        // 接收人
        Set<GiftReceiver> receiverUsers = msg.getReceiverUsers();
        Long score = giftDO.getRtcScore() * msg.getQuantity();
        for (GiftReceiver receiver : receiverUsers) {
            rtcScoreProvider.addScore(msg.getRoomId(), receiver.getUserId(), score);
        }

        // userId:userName
        Map<Long, String> targetUsers = receiverUsers.stream()
                .collect(Collectors.toMap(GiftReceiver::getUserId, GiftReceiver::getUserName));
        rtcGiftSettleMsg.setTargetUsers(targetUsers);
        rtcGiftSettleMsg.setAllSeat(BoolIntVal.False);

        MedalShinyStatus medalShinyStatus = userMedalService.getMedalShinyStatus(msg.getSenderUserId());
        rtcGiftSettleMsg.setMedalShinyStatus(medalShinyStatus);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, msg.getSenderUserId(), null, msg.getRoomId(), rtcGiftSettleMsg);

        // 热度处理
        if (giftDO.getHeatValue() != null && giftDO.getHeatValue() > 0 && Optional.ofNullable(giftDO.getHeatDuration()).orElse(0L) > 0) {
            long heatValue = (long) giftDO.getHeatValue() * msg.getQuantity();
            rtcHeatV2Provider.addHeatItem(msg.getRoomId(), heatValue, msg.getTimestamp() + giftDO.getHeatDuration());
        }

        // 删除心动值处理

        // 星钻榜处理
        if (GiftTypeEnum.COMMON.getValue() == giftDO.getGiftType()) {
            Long coinAmount = msg.getReceiverUsers().size() * msg.getQuantity() * msg.getGiftPrice();
            rtcRankProvider.addDiamondRankAmount(msg.getRoomId(), msg.getSenderUserId(), coinAmount);
            // 星尘榜
            msg.getReceiverUsers().forEach(receiver -> {
                rtcRankProvider.addDustRankAmount(msg.getRoomId(), receiver.getUserId(), msg.getQuantity() * msg.getGiftPrice());
            });

            // 相亲房
            if (isLoveRoom) {
                // 增加金额
                rtcLoveVipProvider.addVipAmount(msg.getRoomId(), msg.getSenderUserId(), coinAmount, msg.getAppVer());
            }

            // 拍卖房
            if (RoomTypeEnum.toEnum(rtcRoom.getRoomType()) == RoomTypeEnum.AUCTION) {
                Set<Long> receiverSet = msg.getReceiverUsers().stream()
                        .map(receiver -> receiver.getUserId())
                        .collect(Collectors.toSet());
                doAuctionRoom(msg.getRoomId(), receiverSet, giftDO, msg.getQuantity(), msg.getOrderId(), msg.getSenderUserId());
            }
        }

        // 真爱榜礼物
        if (giftService.isTrueLoveGift(msg.getGiftId())) {
            List<GiftLoveHisDO> list = new ArrayList<>();
            for (GiftReceiver receiver : msg.getReceiverUsers()) {
                GiftLoveHisDO hisDO = new GiftLoveHisDO();
                hisDO.setSenderUserId(msg.getSenderUserId());
                hisDO.setReceiveUserId(receiver.getUserId());
                hisDO.setCreateTime(curTime);
                hisDO.setGiftId(msg.getGiftId());
                hisDO.setQuantity(Convert.toLong(msg.getQuantity()));
                list.add(hisDO);
            }
            trueLoveRankProvider.batchInsert(list);
        }

        // 广播礼物判断
        if (BoolIntVal.True == giftDO.getIsNotice()) {
            // 赠送者
            UserCardVO card = userService.getCard(msg.getSenderUserId());
            // 通知配置
            NoticeConfigVO noticeConfig = noticeProvider.getNoticeConfig(giftDO.getNoticeId(), msg.getQuantity(), msg.getQuantity() * giftDO.getGiftPrice());
            Map<Long, UserCardVO> cardMap = userService.getCardMap(new HashSet<>(userIds));
            if (card != null && noticeConfig != null) {
                msg.getReceiverUsers().forEach(receiver -> {
                    UserCardVO userCardVO = cardMap.get(receiver.getUserId());
                    RtcGiftNoticeMsg rtcGiftNoticeMsg = new RtcGiftNoticeMsg();
                    rtcGiftNoticeMsg.setGiftId(msg.getGiftId());
                    rtcGiftNoticeMsg.setGiftName(giftDO.getGiftName());
                    rtcGiftNoticeMsg.setRoomId(msg.getRoomId());
                    rtcGiftNoticeMsg.setRoomName(rtcRoom.getRoomName());
                    rtcGiftNoticeMsg.setQuantity(msg.getQuantity());
                    rtcGiftNoticeMsg.setSenderUserName(card.getUserName());
                    rtcGiftNoticeMsg.setSenderUserAvatar(card.getUserAvatar());
                    rtcGiftNoticeMsg.setReceiveUserId(receiver.getUserId());
                    rtcGiftNoticeMsg.setReceiveUserName(Optional.ofNullable(userCardVO).map(UserCardVO::getUserName).orElse(""));
                    rtcGiftNoticeMsg.setNoticeStyle(giftDO.getNoticeStyle());
                    rtcGiftNoticeMsg.setValueAnimationId(noticeConfig == null ? null : noticeConfig.getFloatScreenAnimationId());
                    rtcGiftNoticeMsg.setContent(String.format("%s 送出 %sx%d给%s", card.getUserName(), giftDO.getGiftName(), msg.getQuantity(), rtcGiftNoticeMsg.getReceiveUserName()));
                    rtcGiftNoticeMsg.setFloatScreenStayTime(noticeConfig == null ? null : noticeConfig.getFloatScreenStayTime());
                    rtcGiftNoticeMsg.setReceiverAvatar(userService.getUserAvatar(userIds.get(0)));
                    boolean isNoticeAll = Optional.ofNullable(noticeConfig).map(e -> e.getNoticeScope() == NoticeScopeTypeEnum.ALL.getValue()).orElse(Boolean.FALSE);
                    imService.sendRtcRoomMsg(ImTargetType.ALL, msg.getSenderUserId(), null, isNoticeAll ? 0 : msg.getRoomId(), rtcGiftNoticeMsg);
                });
            }
        }

        if (giftDO.getGiftType().equals(GiftTypeEnum.LOTTERY.getValue())) {
            // 抽奖礼物
            String lotteryId = giftDO.getLotteryId();
            if (StringUtils.isNotEmpty(lotteryId)) {
                Map<Long, Integer> targetUserSeat = new HashMap<>();
                msg.getReceiverUsers().forEach(receiver -> {
                    targetUserSeat.put(receiver.getUserId(), underMaiSeatNo);
                });
                // 抽奖并赠送
                lotteryActivityService.lottery(lotteryId, msg.getSenderUserId(), msg.getQuantity(), msg.getChannelId(),
                        msg.getRoomId(), targetUserSeat, msg.getOrderId());
            }
        }

        // 跨房 PK 处理
        Long roomId = msg.getRoomId();
        RtcBattleStatus battleStatus = rtcBattleProvider.getBattleStatus(roomId);
        if (battleStatus != null && BattleStatusEnum.Battling == battleStatus.getBattleStatus()) {
            long scoreValue = giftDO.getBattleValue() * msg.getQuantity();
            long flowValue = giftDO.getGiftPrice() * msg.getQuantity();
            if (RtcRoomBattleType.ROOM_CROSS == battleStatus.getBattleType()) {
                rtcBattleRoomCrossProvider.addScore(roomId, battleStatus.getBattleId(), msg.getSenderUserId(), scoreValue, flowValue);
            }
        }

        // 处理许愿用户任务逻辑
        dealWishTask(userIds, msg.getGiftId(), msg.getQuantity(), msg.getSenderUserId(), 2);

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 用户在线状态改变
     *
     * @param msg
     */
    public void handleUserOnlineChange(QueueUserOnlineChangeMsg msg) {
        if (isDev()) {
            log.info("rtc消费-用户在线状态改变mq消息:{}", JSON.toJSONString(msg));
        }
        if (BoolIntVal.False == msg.getOnlineStatus()) {
            Long roomId = rtcUserService.getUserStayRoomId(msg.getUserId());

            if (isDev()) {
                log.info("rtc消费-用户所在房间roomId:{}", roomId);
            }

            if (roomId > 0) {
                QueueRtcUserExitMsg userExitMsg = new QueueRtcUserExitMsg();
                userExitMsg.setChannelId(msg.getChannelId());
                userExitMsg.setRoomId(roomId);
                userExitMsg.setExitUserId(msg.getUserId());

                if (isDev()) {
                    log.info("rtc消费-用户准备退出:{}", JSON.toJSONString(userExitMsg));
                }

                handleRtcUserExit(userExitMsg);
            }
        }
    }

    /**
     * 处理语音速配礼物支付消息
     *
     * @param msg
     */
    public void handleVoiceGiftPaid(QueueVoiceGiftPaidMsg msg) {

        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.VOICE_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        // 发送直播礼物结算间消息
        VoiceGiftSettleMsg settleMsg = giftSettleMsgFactory.buildVoiceGiftSettleMsg(msg, msg.getHisId());
        imService.sendChatGroupMsg(ImTargetType.GROUP, null, null, Convert.toStr(msg.getHisId()), settleMsg);

        if (msg.getGiftType().equals(GiftTypeEnum.LOTTERY.getValue())) {
            GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);
            // 抽奖礼物
            String lotteryId = giftDO.getLotteryId();
            if (StringUtils.isNotEmpty(lotteryId)) {
                lotteryActivityService.lotteryForVoiceChat(lotteryId, msg.getSenderUserId(), msg.getQuantity(), msg.getChannelId(), msg.getHisId(), msg.getReceiveUserId());
            }
        }

        // 真爱榜礼物
        if (giftService.isTrueLoveGift(msg.getGiftId())) {
            GiftLoveHisDO hisDO = new GiftLoveHisDO();
            hisDO.setSenderUserId(msg.getSenderUserId());
            hisDO.setReceiveUserId(msg.getReceiveUserId());
            hisDO.setCreateTime(System.currentTimeMillis());
            hisDO.setGiftId(msg.getGiftId());
            hisDO.setQuantity(Convert.toLong(msg.getQuantity()));
            trueLoveRankProvider.batchInsert(Collections.singletonList(hisDO));
        }

        // 处理许愿用户任务逻辑
        dealWishTask(Collections.singletonList(msg.getReceiveUserId()), msg.getGiftId(), msg.getQuantity(), msg.getSenderUserId(), 3);

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 处理私聊送礼物支付消息
     *
     * @param msg
     */
    public void handlePrivateChatGiftPaid(QueueBaseGiftPaidMsg msg) {
        //检查是否重复消费
        String appName = applicationName;
        String rockGropId = rocketConfig.getProps().get(PropertyKeyConst.GROUP_ID).toString();
        Long orderId = msg.getOrderId();
        String key = RocketTag.PRIVATE_CHAT_GIFT_PAID + "." + appName + "." + rockGropId + "." + orderId + "." + msg.getGiftId();
        long curTime = System.currentTimeMillis();
        if (dataMsgHisService.checkMsgIsExe(key, DataMsgHitBussinessEnum.RTCGIFTPAID.getValue())) {
            //消息已经被消费
            return;
        }

        log.info("私聊送礼物支付消息: {}", JSON.toJSONString(msg));

        // 发送私聊礼物结算间消息
        PrivateChatGiftSettleMsg settleMsg = giftSettleMsgFactory.buildPrivateChatGiftSettleMsg(msg);
        imService.sendCustomImMsg(msg.getSenderUserId(), msg.getReceiveUserId(), settleMsg, null);

        if (msg.getGiftType().equals(GiftTypeEnum.LOTTERY.getValue())) {
            GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);
            // 抽奖礼物
            String lotteryId = giftDO.getLotteryId();
            if (StringUtils.isNotEmpty(lotteryId)) {
                lotteryActivityService.lotteryForPrivateChat(lotteryId, msg.getSenderUserId(), msg.getQuantity(), msg.getChannelId(), msg.getOrderId(), msg.getReceiveUserId());
            }
        }

        // 真爱榜礼物
        if (giftService.isTrueLoveGift(msg.getGiftId())) {
            GiftLoveHisDO hisDO = new GiftLoveHisDO();
            hisDO.setSenderUserId(msg.getSenderUserId());
            hisDO.setReceiveUserId(msg.getReceiveUserId());
            hisDO.setCreateTime(System.currentTimeMillis());
            hisDO.setGiftId(msg.getGiftId());
            hisDO.setQuantity(Convert.toLong(msg.getQuantity()));
            trueLoveRankProvider.batchInsert(Collections.singletonList(hisDO));
        }

        // 处理许愿用户任务逻辑
        dealWishTask(Collections.singletonList(msg.getReceiveUserId()), msg.getGiftId(), msg.getQuantity(), msg.getSenderUserId(), 4);

        //更新消息执行状态
        dataMsgHisService.updateByKey(1, curTime, key);
    }

    /**
     * 连击过期
     *
     * @param msg
     */
    public void handleRtcGiftComboTimeout(QueueRtcGiftComboTimeoutMsg msg) {
        // 查询礼物消息
        GiftDO giftDO = giftService.getGift(msg.getGiftId(), null);
        log.info("飘屏连击过期消息处理 : {}", JSON.toJSONString(msg, true));
        // 是否全麦
        boolean isAllSeat = false;
        // 用户数量
        int userNum = msg.getTargetIds().size();
        if (userNum == 1 && msg.getTargetIds().get(0) < 0) {
            isAllSeat = true;
            userNum = (int) Math.abs(msg.getTargetIds().get(0));
        }

        // 弹幕礼物策略
        if (BoolIntVal.True == Optional.ofNullable(giftDO.getIsComment()).orElse(BoolIntVal.False) && msg.getCombo() > 1) {
            Integer commentComboMultiple = Optional.ofNullable(giftDO.getCommentComboMultiple()).orElse(0);
            if (commentComboMultiple > 0 && msg.getCombo() % commentComboMultiple != 0) {
                // 发送直播间消息
                RtcGiftSettleMsg rtcGiftSettleMsg = new RtcGiftSettleMsg();
                rtcGiftSettleMsg.setRoomId(msg.getRoomId());
                rtcGiftSettleMsg.setGiftId(msg.getGiftId());
                rtcGiftSettleMsg.setQuantity(msg.getQuantity());
                rtcGiftSettleMsg.setSenderSeatNo(rtcSeatProvider.getUserSeatNo(msg.getRoomId(), msg.getSenderUserId()));
                rtcGiftSettleMsg.setTargetSeats(null);
                rtcGiftSettleMsg.setIsComboTimeout(true);
                rtcGiftSettleMsg.setComboPeriodSec(COMBO_TIMEOUT);
                if (isAllSeat) {
                    rtcGiftSettleMsg.setTargetUsers(null);
                    rtcGiftSettleMsg.setAllSeat(BoolIntVal.True);
                } else {
                    rtcGiftSettleMsg.setTargetUsers(userService.getNameAndAvatarByIds(msg.getTargetIds()).stream().collect(Collectors.toMap(UserCardVO::getUserId, UserCardVO::getUserName)));
                    rtcGiftSettleMsg.setAllSeat(BoolIntVal.False);
                }
                rtcGiftSettleMsg.setCombo(msg.getCombo());
                rtcGiftSettleMsg.setComboKeyHash(SecureUtil.md5(RedisRtcKeys.getGiftComboKey(msg.getRoomId(), msg.getSenderUserId(), msg.getGiftId(), msg.getQuantity(), getComboKeyTargetsHash(msg.getTargetIds()))));
                imService.sendRtcRoomMsg(ImTargetType.GROUP, msg.getSenderUserId(), null, msg.getRoomId(), rtcGiftSettleMsg);
            }
        }

        // 通知配置
        NoticeConfigVO noticeConfig = noticeProvider.getNoticeConfig(giftDO.getNoticeId(), Convert.toInt(msg.getQuantity() * userNum * msg.getCombo()), msg.getQuantity() * giftDO.getGiftPrice() * msg.getCombo() * userNum);
        log.info("飘屏通知配置 : {}", JSON.toJSONString(noticeConfig, true));
        Long sendNum = msg.getQuantity() * userNum * msg.getCombo();
        if (noticeConfig != null) {
            // 送礼人
            UserCardVO card = userService.getCard(msg.getSenderUserId());
            // 飘屏
            RtcGiftNoticeMsg rtcGiftNoticeMsg = new RtcGiftNoticeMsg();
            rtcGiftNoticeMsg.setGiftId(msg.getGiftId());
            rtcGiftNoticeMsg.setGiftName(giftDO.getGiftName());
            rtcGiftNoticeMsg.setRoomId(msg.getRoomId());
            RtcRoomDO room = rtcRoomDAO.selectRoomById(msg.getRoomId());
            rtcGiftNoticeMsg.setRoomName(room.getRoomName());
            rtcGiftNoticeMsg.setQuantity(msg.getQuantity());
            rtcGiftNoticeMsg.setSenderUserName(card.getUserName());
            rtcGiftNoticeMsg.setSenderUserAvatar(card.getUserAvatar());
            if (isAllSeat) {
                rtcGiftNoticeMsg.setReceiveUserName("全麦");

                rtcGiftNoticeMsg.setReceiverAvatar(room.getRoomCover());
            } else {
                // 收礼人
                Map<Long, UserCardVO> cardMap = userService.getCardMap(new HashSet<>(msg.getTargetIds()));
                List<String> receiveUserNames = new ArrayList<>();
                for (Long targetId : msg.getTargetIds()) {
                    UserCardVO userCardVO = cardMap.get(targetId);
                    if (userCardVO == null) {
                        continue;
                    }
                    receiveUserNames.add(userCardVO.getUserName());
                }
                rtcGiftNoticeMsg.setReceiveUserName(CollectionUtil.join(receiveUserNames, "，"));
                rtcGiftNoticeMsg.setReceiverAvatar(userService.getUserAvatar(msg.getTargetIds().get(0)));
            }
            rtcGiftNoticeMsg.setNoticeStyle(giftDO.getNoticeStyle());
            rtcGiftNoticeMsg.setValueAnimationId(noticeConfig.getFloatScreenAnimationId());
            // rtcGiftNoticeMsg.setBarrageAnimationId(noticeConfig.getBarrageAnimationId());
            rtcGiftNoticeMsg.setCombo(msg.getCombo());
            rtcGiftNoticeMsg.setReceiveUserId(msg.getTargetIds().get(0));
            rtcGiftNoticeMsg.setContent(String.format("%s 送出 %sx%d给%s", card.getUserName(), giftDO.getGiftName(), sendNum, rtcGiftNoticeMsg.getReceiveUserName()));
            rtcGiftNoticeMsg.setFloatScreenStayTime(noticeConfig == null ? null : noticeConfig.getFloatScreenStayTime());

            boolean isNoticeAll = Optional.ofNullable(noticeConfig).map(e -> e.getNoticeScope() == NoticeScopeTypeEnum.ALL.getValue()).orElse(Boolean.FALSE);
            log.info("飘屏消息发送 : {}", JSON.toJSONString(rtcGiftNoticeMsg, true));
            imService.sendRtcRoomMsg(ImTargetType.ALL, msg.getSenderUserId(), null, isNoticeAll ? 0L : msg.getRoomId(), rtcGiftNoticeMsg);
        }
    }

    /**
     * 派单创建
     *
     * @param msg
     */
    public void handleRtcSkillOrderCreate(QueueRtcSkillOrderMsg msg) {
        if (isDev()) {
            log.info("处理派单创建消息,{}", JSON.toJSONString(msg));
        }
        RtcSkillOrderDO order = rtcSkillOrderProvider.getById(msg.getOrderId());

        // 派单大厅数量变更消息
        sendSkillOrderCountToHall(msg);

        // 发送房间订单数量变更消息
        Integer orderInRoomCount = rtcSkillOrderProvider.getOrderInRoomCount(msg.getRoomId());
        RtcSkillOrderInRoomMsg orderInRoomMsg = new RtcSkillOrderInRoomMsg();
        orderInRoomMsg.setOrderCount(orderInRoomCount);
        imService.sendRtcRoomMsg(ImTargetType.USER, null, msg.getOperatorUserId(), null, orderInRoomMsg);

        // 推送消息给有此技能的陪陪引导接单
        Integer userCount = familyService.countSkillUsersSimple(msg.getSkillId());
        Integer pageSize = 5000;
        int pageIndex = 1;
        if (userCount > pageSize) {

            int totalPage = (userCount + pageSize - 1) / pageSize;
            for (pageIndex = 1; pageIndex < totalPage; pageIndex++) {
                List<Long> peer = familyService.getSkillUserListSimple(pageIndex, pageSize, msg.getSkillId()).stream().map(SkillUser::getUserId).collect(Collectors.toList());
                batchSendSkillOrder(order, peer);
            }
        } else {
            List<Long> peer = familyService.getSkillUserListSimple(pageIndex, pageSize, msg.getSkillId()).stream().map(SkillUser::getUserId).collect(Collectors.toList());
            batchSendSkillOrder(order, peer);
        }

    }


    /**
     * 发送消息给陪陪
     *
     * @param order
     * @param serviceUserIds
     */
    private void batchSendSkillOrder(RtcSkillOrderDO order, List<Long> serviceUserIds) {

        // 给符合条件的陪陪发消息
        serviceUserIds.forEach(e -> {

            RtcSkillOrderDetailMsg msgToUser = new RtcSkillOrderDetailMsg();
            BeanUtils.copyProperties(order, msgToUser);

            imService.sendCustomImMsg(null, e, msgToUser, null);
        });
    }


    /**
     * 发送消息到所有人的派单大厅
     */
    private void sendSkillOrderCountToHall(QueueRtcSkillOrderMsg msg) {
        // 发送派单大厅数量变更消息

        Integer userCount = userService.countOnlineUser();
        Integer pageSize = 5000;
        int pageIndex = 0;
        if (userCount > pageSize) {

            int totalPage = (userCount + pageSize - 1) / pageSize;
            for (pageIndex = 0; pageIndex < totalPage; pageIndex++) {
                List<Long> peer = userService.selectPageListOnlineUser(pageIndex, pageSize);
                batchSendSkillOrderToHall(peer);
            }
        } else {

            List<Long> peer = userService.selectPageListOnlineUser(pageIndex, pageSize);
            batchSendSkillOrderToHall(peer);
        }
    }


    /**
     * 发送消息到所有人的派单大厅
     *
     * @param serviceUserIds
     */
    private void batchSendSkillOrderToHall(List<Long> serviceUserIds) {

        RtcSkillOrderDO lastOrderCreated = rtcSkillOrderProvider.getLastOrderCreated();

        String bossUserName = userService.getUserName(lastOrderCreated.getBossUserId());
        // 给符合条件的陪陪发消息
        serviceUserIds.forEach(e -> {

            Integer count = rtcSkillOrderProvider.getAllOrderCreatedCount(e);
            RtcSkillOrderMsg orderMsg = new RtcSkillOrderMsg();
            orderMsg.setOrderCount(count);
            orderMsg.setDesc(bossUserName + " 发布了 " + lastOrderCreated.getSkillName() + " 的派单需求");
            imService.sendRtcRoomMsg(ImTargetType.USER, null, e, null, orderMsg);
            imService.sendCustomImMsg(null, e, orderMsg, null);
        });
    }


    /**
     * 派单接单(结束)
     *
     * @param msg
     */
    public void handleRtcSkillOrderFinish(QueueRtcSkillOrderMsg msg) {
        if (isDev()) {
            log.info("处理派单接单消息,{}", JSON.toJSONString(msg));
        }
        /**
         * 发送普通消息
         */
        // 发送派单大厅数量变更消息
        sendSkillOrderCountToHall(msg);

        // 发送房间订单数量变更消息
        Integer orderInRoomCount = rtcSkillOrderProvider.getOrderInRoomCount(msg.getRoomId());
        RtcSkillOrderInRoomMsg orderInRoomMsg = new RtcSkillOrderInRoomMsg();
        orderInRoomMsg.setOrderCount(orderInRoomCount);
        // 发给现在的主持人
        Long userId = rtcSeatProvider.getSeatUserId(msg.getRoomId(), 0);
        imService.sendRtcRoomMsg(ImTargetType.USER, null, userId, null, orderInRoomMsg);

        /**
         * 发送系统消息
         */
        RtcSkillOrderDO order = rtcSkillOrderProvider.getById(msg.getOrderId());
        // 获取接单人昵称
        String serviceUserName = userService.getUserName(order.getServiceUserId());
        // 发送系统消息给下单用户
        SystemMsg systemBossUserMsg = new SystemMsg();
        systemBossUserMsg.setContentType(ContentType.TEXT);
        systemBossUserMsg.setTitle("派单通知");
        String body = String.format("你有一条【%s*%d】的订单已被%s接取，请等待沟通后参与服务！", order.getSkillName(), order.getOrderNum(), serviceUserName);
        systemBossUserMsg.setBody(body);
        systemBossUserMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
        systemBossUserMsg.setModuleType(ModuleTypeEnum.RTC_SKILL_ORDER_DETAILS.getDesc());
        systemBossUserMsg.setModuleId(Convert.toStr(msg.getOrderId()));
        imService.sendCustomImMsg(null, msg.getBossUserId(), systemBossUserMsg, null);

        // 发送系统消息给接单用户
        SystemMsg systemServiceUserMsg = new SystemMsg();
        systemServiceUserMsg.setContentType(ContentType.TEXT);
        systemServiceUserMsg.setTitle("派单通知");
        String bodyServiceUser = String.format("你接取了【%s*%d】的订单，请及时沟通后给予服务！", order.getSkillName(), order.getOrderNum());
        systemServiceUserMsg.setBody(bodyServiceUser);
        systemServiceUserMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
        systemServiceUserMsg.setModuleType(ModuleTypeEnum.RTC_SKILL_ORDER_DETAILS.getDesc());
        systemServiceUserMsg.setModuleId(Convert.toStr(msg.getOrderId()));
        imService.sendCustomImMsg(null, order.getServiceUserId(), systemServiceUserMsg, null);
    }

    /**
     * 派单关闭
     *
     * @param msg
     */
    public void handleRtcSkillOrderClose(QueueRtcSkillOrderMsg msg) {
        if (isDev()) {
            log.info("处理派单关闭消息,{}", JSON.toJSONString(msg));
        }

        // 发送派单大厅数量变更消息
        sendSkillOrderCountToHall(msg);

        // 发送房间订单数量变更消息
        Integer orderInRoomCount = rtcSkillOrderProvider.getOrderInRoomCount(msg.getRoomId());
        RtcSkillOrderInRoomMsg orderInRoomMsg = new RtcSkillOrderInRoomMsg();
        orderInRoomMsg.setOrderCount(orderInRoomCount);
        // 发给现在的主持人
        Long userId = rtcSeatProvider.getSeatUserId(msg.getRoomId(), 0);
        imService.sendRtcRoomMsg(ImTargetType.USER, null, userId, null, orderInRoomMsg);
    }

    /**
     * es更新rtc信息
     */
    public void handleRtcRoomESInfo(QueueRtcRoomInfoESMsg msg) {
        RtcRoomDO rtcRoomDO = new RtcRoomDO();
        rtcRoomDO.setRoomId(msg.getRoomId());
        rtcRoomDO.setRoomName(msg.getRoomName());
        rtcRoomDO.setUserId(msg.getUserId());
        rtcRoomDO.setRoomType(msg.getRoomType());
        rtcRoomDO.setRoomNo(msg.getRoomNo());
        rtcRoomDO.setCategoryId(msg.getCategoryId());
        rtcRoomDO.setRoomTitle(msg.getRoomTitle());
        rtcRoomDO.setRoomCover(msg.getRoomCover());
        rtcRoomDO.setRoomRule(msg.getRoomRule());
        rtcRoomDO.setRoomWelcome(msg.getRoomWelcome());
        rtcRoomDO.setRoomLockPass(msg.getRoomLockPass());
        rtcRoomDO.setRoomMicFree(msg.getRoomMicFree());
        rtcRoomDO.setRoomScoreEnabled(msg.getRoomScoreEnabled());
        rtcRoomDO.setRoomChatEnabled(msg.getRoomChatEnabled());
        rtcRoomDO.setStatus(msg.getStatus());
        rtcRoomDO.setFamilyId(msg.getFamilyId());
        rtcRoomDO.setRoomCreateTime(msg.getRoomCreateTime());

        rtcRoomSearchService.saveOrUpdate(rtcRoomDO);
    }

    /**
     * 处理feed点赞消息
     *
     * @param msg
     */
    public void handleFeedLike(QueueFeedLikeMsg msg) {
        // 系统消息
        SystemMsg systemMsg = new SystemMsg();
        systemMsg.setContentType(ContentType.TEXT);
        systemMsg.setTitle("新的点赞");
        String body = String.format("%s点赞了您的动态", userService.getUserName(msg.getOperatorUserId()));
        systemMsg.setBody(body);
        systemMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
        systemMsg.setModuleType(ModuleTypeEnum.USER_FEED.getDesc());
        systemMsg.setModuleId(Convert.toStr(msg.getFeedId()));
        imService.sendCustomImMsg(null, msg.getUserId(), systemMsg, null);
    }


    /**
     * 处理feed打赏消息
     *
     * @param msg
     */
    public void handleFeedSendGift(QueueFeedSendGiftMsg msg) {

        // 系统消息
        SystemMsg systemMsg = new SystemMsg();
        systemMsg.setContentType(ContentType.TEXT);
        systemMsg.setTitle("新的打赏");
        String body = String.format("%s打赏了您的动态", userService.getUserName(msg.getOperatorUserId()));
        systemMsg.setBody(body);
        systemMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
        systemMsg.setModuleType(ModuleTypeEnum.USER_FEED.getDesc());
        systemMsg.setModuleId(Convert.toStr(msg.getFeedId()));
        imService.sendCustomImMsg(null, msg.getUserId(), systemMsg, null);
    }

    private static final int COMBO_TIMEOUT = 5;

    private ComboData processCombo(long userId, long roomId, long giftId, long price, int quantity, List<Long> targetIds) {
        // 处理 combo
        String comboKey = RedisRtcKeys.getGiftComboKey(roomId, userId, giftId, quantity, getComboKeyTargetsHash(targetIds));

        long combo = redisService.increaseAndExpire(comboKey, 1, COMBO_TIMEOUT, TimeUnit.SECONDS);

        // 查询配置
        GiftComboConfigDO comboConfigDO = giftService.getCachedComboConfig(giftId);
        Long animationId = null;
        Integer level = 1;
        if (comboConfigDO != null) {
            List<GiftComboConfigData> dataList = JSON.parseArray(comboConfigDO.getConfigData(), GiftComboConfigData.class);
            Map<Long, Long> valueAnimMap = dataList.stream().collect(Collectors.toMap(GiftComboConfigData::getValue, GiftComboConfigData::getAnimationId));
            List<Long> values = new ArrayList<>(valueAnimMap.keySet());
            level = values.size();
            values.sort(Collections.reverseOrder());
            int userNum = targetIds.size();
            if (userNum == 1 && targetIds.get(0) < 0) {
                userNum = (int) Math.abs(targetIds.get(0));
            }
            long valueAmount = price * quantity * combo * userNum;
            for (Long value : values) {
                if (valueAmount >= value) {
                    animationId = valueAnimMap.get(value);
                    break;
                }
                level--;
            }
        }
        // 因为 RedisKeyExpireMsgListener 里面使用的是短的key, 那边的key经过处理的, 所以这里应该使用处理过的短key
        String simpleComboKey = comboKey.replace(RedisRtcKeys.RTC_GIFT_COMBO, "");

        if (combo == 1) {
            Long curTime = System.currentTimeMillis();
            redisService.hashPut(RedisRtcKeys.RTC_COMBO_TIME, simpleComboKey, String.valueOf(curTime));
            // 创建一个新的缓存
            redisService.increaseAndExpire(RedisRtcKeys.getRtcComboHis(comboKey, curTime), 1, COMBO_TIMEOUT * 2, TimeUnit.SECONDS);
            // 缓存目标用户
            redisService.set(RedisRtcKeys.getRtcComboTargets(comboKey, curTime), Strings.join(targetIds, ','), COMBO_TIMEOUT * 2, TimeUnit.SECONDS);
        } else {
            // 获取最新的缓存 key
            Long timestamp = Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_COMBO_TIME, simpleComboKey)).map(Convert::toLong).orElse(null);
            if (timestamp != null) {
                String comboHisKey = RedisRtcKeys.getRtcComboHis(comboKey, timestamp);
                String comboTargetsKey = RedisRtcKeys.getRtcComboTargets(comboKey, timestamp);
                redisService.increaseAndExpire(comboHisKey, 1, COMBO_TIMEOUT * 2, TimeUnit.SECONDS);
                // 重置过期时间
                redisService.expire(comboTargetsKey, COMBO_TIMEOUT * 2 * 1000L);
            }
        }
        return new ComboData(combo, comboKey, animationId, level);
    }

    private static String getComboKeyTargetsHash(List<Long> userIds) {
        Collections.sort(userIds);
        String str = Strings.join(userIds, ',');
        return SecureUtil.md5(str);
    }

    /**
     * 参与竞拍
     *
     * @param roomId
     * @param receiverUsers
     * @param gift
     * @param quantity
     * @param orderId
     * @param senderUserId
     */
    private void doAuctionRoom(Long roomId, Set<Long> receiverUsers, GiftDO gift, Integer quantity,
                               Long orderId, Long senderUserId) {
        Integer giftPriceType = gift.getGiftPriceType();
        if (giftPriceType == null) {
            if (isDev()) {
                log.warn(LogUtil.devLog("礼物价格类型为 null 无法参与竞拍 " + gift.getGiftId()));
            }
            return;
        }
        if (WalletTypeEnum.STAR_DIAMOND != WalletTypeEnum.toEnum(giftPriceType)) {
            if (isDev()) {
                log.warn(LogUtil.devLog("礼物价格类型不是钻石，无法参与竞拍 " + gift.getGiftId()));
            }
            return;
        }
        if (!rtcRoomAuctionProvider.isBidding(roomId)) {
            // 不在竞拍中
            if (isDev()) {
                log.warn(LogUtil.devLog("不在竞拍中，无法参与竞拍 roomId " + roomId));
            }
            return;
        }
        long auctionUserId = rtcRoomAuctionProvider.getAuctionUserId(roomId);
        if (!receiverUsers.contains(auctionUserId)) {
            return;
        }

        // 拍卖信息
        String auctionStr = redisService.get(RedisRtcKeys.getRtcAuctionRoomInfo(roomId));
        if (!JSON.isValidObject(auctionStr)) {
            log.error(String.format("参与竞拍失败！拍卖信息有误 roomId = %s auctionStr = %s", roomId, auctionStr));
            return;
        }
        RtcRoomAuctionDO auction = JSON.parseObject(auctionStr, RtcRoomAuctionDO.class);
        if (auction == null) {
            log.error(String.format("参与竞拍失败！拍卖信息有误，无法转换成竞拍对象 roomId = %s auctionStr = %s", roomId, auctionStr));
            return;
        }

        Long rtcRoomAuctionId = auction.getRtcRoomAuctionId();
        // 起拍时间
        Long startTime = auction.getCreateTime();
        Long startingGiftPrice = auction.getStartingGiftPrice();
        rtcRoomAuctionProvider.bid(roomId, orderId, rtcRoomAuctionId, startTime, auction.getBidItemId(),
                senderUserId, auctionUserId,
                gift.getGiftId(), gift.getGiftPrice(), quantity, startingGiftPrice);
    }

    @Data
    @AllArgsConstructor
    private static class ComboData implements Serializable {
        private Long combo;
        private String comboKey;
        private Long animationId;
        private Integer level;
    }
}
