package com.starbuds.server.service.rtc.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.service.rtc.rocket.handler.DelayMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 延迟消息监听
 *
 * @author Lay
 * @date 2020/7/10
 */
@Component
@Slf4j
public class DelayMsgListener implements MessageListener {
    @Resource
    DelayMsgHandler delayMsgHandler;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        try {
            log.info("收到延迟队列消息 {} {}", message.getTopic(), message.getTag());
            switch (message.getTag()) {
                case RocketTag.RTC_CHECK_STATUS:
                    delayMsgHandler.handleRtcCheckStatus(JSON.parseObject(message.getBody(), QueueRtcCheckStatusMsg.class));
                    break;
                case RocketTag.RTC_ROOM_BATTLE_TIME_UP:
                    delayMsgHandler.handleRtcRoomBattleTimeUp(JSON.parseObject(message.getBody(), QueueRtcRoomBattleTimeUpMsg.class));
                    break;
                case RocketTag.RTC_ROOM_BATTLE_RING_TIMEOUT:
                    delayMsgHandler.handleRtcRoomBattleRingTimeout(JSON.parseObject(message.getBody(), QueueRtcBattleRingTimeoutMsg.class));
                    break;
                case RocketTag.ZEGO_CHECK_VOICE_HOVER:
                    delayMsgHandler.handleZegoCheckVoice(JSON.parseObject(message.getBody(), QueueZegoVocieTimeoutMsg.class));
                    break;
                default:
                    break;
            }
            return Action.CommitMessage;
        } catch (Exception e) {
            log.error("消息处理失败", e);
            return Action.ReconsumeLater;
        }
    }
}
