package com.starbuds.server.service.rtc.rocket.listener;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Action;
import com.aliyun.openservices.ons.api.ConsumeContext;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.MessageListener;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.pojo.mq.*;
import com.starbuds.server.service.rtc.rocket.handler.NormalMsgHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 普通消息监听
 *
 * @author Lay
 * @date 2020/7/10
 */
@Component
@Slf4j
public class NormalMsgListener implements MessageListener {

    @Resource
    NormalMsgHandler normalMsgHandler;

    @Override
    public Action consume(Message message, ConsumeContext consumeContext) {
        try {
            log.info("收到普通队列消息 {} {}", message.getTopic(), message.getTag());
            switch (message.getTag()) {
                case RocketTag.RTC_USER_JOIN:
                    normalMsgHandler.handleRtcUserJoin(JSON.parseObject(message.getBody(), QueueRtcUserJoinMsg.class));
                    break;
                case RocketTag.RTC_USER_EXIT:
                    normalMsgHandler.handleRtcUserExit(JSON.parseObject(message.getBody(), QueueRtcUserExitMsg.class));
                    break;
                case RocketTag.RTC_GIFT_PAID:
                    normalMsgHandler.handleRtcGiftPaid(JSON.parseObject(message.getBody(), QueueRtcGiftPaidMsg.class));
                    break;
                case RocketTag.RTC_UNDER_MAI_GIFT_PAID:
                    // 语聊房麦下送礼物支付消息
                    normalMsgHandler.handleRtcGiftUnderMaiPaid(JSON.parseObject(message.getBody(), QueueRtcGiftUnderMaiPaidMsg.class));
                    break;
                case RocketTag.VOICE_GIFT_PAID:
                    normalMsgHandler.handleVoiceGiftPaid(JSON.parseObject(message.getBody(), QueueVoiceGiftPaidMsg.class));
                    break;
                case RocketTag.PRIVATE_CHAT_GIFT_PAID:
                    // 私聊送礼物支付
                    normalMsgHandler.handlePrivateChatGiftPaid(JSON.parseObject(message.getBody(), QueueBaseGiftPaidMsg.class));
                    break;
                case RocketTag.USER_ONLINE_CHANGE:
                    normalMsgHandler.handleUserOnlineChange(JSON.parseObject(message.getBody(), QueueUserOnlineChangeMsg.class));
                    break;
                case RocketTag.RTC_GIFT_COMBO_TIMEOUT:
                    normalMsgHandler.handleRtcGiftComboTimeout(JSON.parseObject(message.getBody(), QueueRtcGiftComboTimeoutMsg.class));
                    break;
                case RocketTag.RTC_SKILL_ORDER_CREATE:
                    // 派单创建
                    normalMsgHandler.handleRtcSkillOrderCreate(JSON.parseObject(message.getBody(), QueueRtcSkillOrderMsg.class));
                    break;
                case RocketTag.RTC_SKILL_ORDER_FINISH:
                    // 派单派单
                    normalMsgHandler.handleRtcSkillOrderFinish(JSON.parseObject(message.getBody(), QueueRtcSkillOrderMsg.class));
                    break;
                case RocketTag.RTC_SKILL_ORDER_CLOSE:
                    // 派单关闭
                    normalMsgHandler.handleRtcSkillOrderClose(JSON.parseObject(message.getBody(), QueueRtcSkillOrderMsg.class));
                    break;
                case RocketTag.RTC_ROOM_INFO_ES_SYNCH:
                    normalMsgHandler.handleRtcRoomESInfo(JSON.parseObject(message.getBody(), QueueRtcRoomInfoESMsg.class));
                    break;
                case RocketTag.FEED_LIKE:
                    // feed点赞
                    normalMsgHandler.handleFeedLike(JSON.parseObject(message.getBody(), QueueFeedLikeMsg.class));
                    break;
                case RocketTag.FEED_SEND_GIFT:
                    // feed打赏
                    normalMsgHandler.handleFeedSendGift(JSON.parseObject(message.getBody(), QueueFeedSendGiftMsg.class));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error("消息处理失败", e);
        }

        return Action.CommitMessage;
    }

}
