package com.starbuds.server.service.rtc.impl;

import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.pojo.rtc.bo.FamilyRatio;
import com.starbuds.server.common.pojo.rtc.bo.RtcRoomRatio;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import test.BaseTest;

import java.math.BigDecimal;

class RtcRoomServiceImplTest extends BaseTest {

    @Reference(version = "1.0.0")
    private RtcRoomService rtcRoomService;

    // 483988682571777, 483988682571778, 483988682571779
    // 语聊房id：分成0.6测试专用
    private Long ROOM_ID_HAS_RATIO = 483988682571777L;

    // 语聊房id：无分成测试专用
    private Long ROOM_ID_NO_RATIO = 483988682571778L;

    // 语聊房id：无家族无分类测试专用
    private Long ROOM_ID_NOTHING = 483988682571779L;

    // 个播房id：分成0.6测试专用
    private Long ROOM_ID_HAS_RATIO_SINGLE = 515743833653251L;

    // 个播房id：无分成测试专用
    private Long ROOM_ID_NO_RATIO_SINGLE = 515743833653252L;

    // mvn -Dtest=com.starbuds.server.service.rtc.impl.RtcRoomServiceImplTest#getRtcRoomRatio test
    // 语聊房分成
    @Test
    void getRtcRoomRatio() {
        // 分成0.6测试专用
        RtcRoomRatio rtcRoomRatio = rtcRoomService.getRtcRoomRatio(ROOM_ID_HAS_RATIO);
        Assertions.assertEquals(ROOM_ID_HAS_RATIO, rtcRoomRatio.getRoomId());
        Assertions.assertEquals(RtcBusinessTypeEnum.RTC_NORMAL, rtcRoomRatio.getBusinessType());
        Assertions.assertEquals(new BigDecimal("0.6"), rtcRoomRatio.getCategoryUserRatio());
        FamilyRatio familyRatio = rtcRoomRatio.getFamilyRatio();
        Assertions.assertEquals(new BigDecimal("0.25"), familyRatio.getFamilyRatio());
        Assertions.assertEquals(new BigDecimal("0.7"), familyRatio.getUserRatio());
        Assertions.assertEquals(new BigDecimal("0.15"), familyRatio.getFamilySingleRtcRatio());
        Assertions.assertEquals(new BigDecimal("0.7"), familyRatio.getUserSingleRtcRatio());

        // 无分成测试专用
        rtcRoomRatio = rtcRoomService.getRtcRoomRatio(ROOM_ID_NO_RATIO);
        Assertions.assertEquals(ROOM_ID_NO_RATIO, rtcRoomRatio.getRoomId());
        Assertions.assertEquals(RtcBusinessTypeEnum.RTC_NORMAL, rtcRoomRatio.getBusinessType());
        Assertions.assertEquals(null, rtcRoomRatio.getCategoryUserRatio());
        familyRatio = rtcRoomRatio.getFamilyRatio();
        Assertions.assertEquals(new BigDecimal("0.25"), familyRatio.getFamilyRatio());
        Assertions.assertEquals(new BigDecimal("0.7"), familyRatio.getUserRatio());
        Assertions.assertEquals(new BigDecimal("0.15"), familyRatio.getFamilySingleRtcRatio());
        Assertions.assertEquals(new BigDecimal("0.7"), familyRatio.getUserSingleRtcRatio());

        // 无家族无分类测试专用
        rtcRoomRatio = rtcRoomService.getRtcRoomRatio(ROOM_ID_NOTHING);
        Assertions.assertEquals(ROOM_ID_NOTHING, rtcRoomRatio.getRoomId());
        Assertions.assertEquals(RtcBusinessTypeEnum.RTC_NORMAL, rtcRoomRatio.getBusinessType());
        Assertions.assertEquals(null, rtcRoomRatio.getCategoryUserRatio());
        familyRatio = rtcRoomRatio.getFamilyRatio();
        Assertions.assertEquals(null, familyRatio);
    }

    // 个播房分成
    @Test
    void getRtcSingleRoomRatio() {
        // 分成0.6测试专用
        RtcRoomRatio rtcRoomRatio = rtcRoomService.getRtcRoomRatio(ROOM_ID_HAS_RATIO_SINGLE);
        Assertions.assertEquals(ROOM_ID_HAS_RATIO_SINGLE, rtcRoomRatio.getRoomId());
        Assertions.assertEquals(RtcBusinessTypeEnum.RTC_SINGLE, rtcRoomRatio.getBusinessType());
        Assertions.assertEquals(new BigDecimal("0.6"), rtcRoomRatio.getCategoryUserRatio());
        FamilyRatio familyRatio = rtcRoomRatio.getFamilyRatio();
        Assertions.assertEquals(new BigDecimal("0.25"), familyRatio.getFamilyRatio());
        Assertions.assertEquals(new BigDecimal("0.7"), familyRatio.getUserRatio());
        Assertions.assertEquals(new BigDecimal("0.15"), familyRatio.getFamilySingleRtcRatio());
        Assertions.assertEquals(new BigDecimal("0.7"), familyRatio.getUserSingleRtcRatio());

        // 无分成测试专用
        rtcRoomRatio = rtcRoomService.getRtcRoomRatio(ROOM_ID_NO_RATIO_SINGLE);
        Assertions.assertEquals(ROOM_ID_NO_RATIO_SINGLE, rtcRoomRatio.getRoomId());
        Assertions.assertEquals(RtcBusinessTypeEnum.RTC_SINGLE, rtcRoomRatio.getBusinessType());
        Assertions.assertEquals(null, rtcRoomRatio.getCategoryUserRatio());
        familyRatio = rtcRoomRatio.getFamilyRatio();
        Assertions.assertEquals(new BigDecimal("0.25"), familyRatio.getFamilyRatio());
        Assertions.assertEquals(new BigDecimal("0.7"), familyRatio.getUserRatio());
        Assertions.assertEquals(new BigDecimal("0.15"), familyRatio.getFamilySingleRtcRatio());
        Assertions.assertEquals(new BigDecimal("0.7"), familyRatio.getUserSingleRtcRatio());
    }
}