package com.starbuds.server.service.rtc.provider;

import org.apache.commons.lang3.StringUtils;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.jupiter.api.Test;
import org.springframework.test.util.ReflectionTestUtils;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.IntStream;
import java.util.stream.LongStream;

public class RtcRoomAuctionProviderTest {

    private RtcRoomAuctionProvider rtcRoomAuctionProvider;

    @BeforeClass
    public void beforeClass() {
        rtcRoomAuctionProvider = new RtcRoomAuctionProvider();
    }

    @Test
    public void getBidScoreTest() {
        Long price = 0L;
        // 单次分值：大于0；小于1
        Long[] gaps = new Long[]{1L, 86400L, 86400L, 86400L * 2, 86400L * 10, 86400L * 20, 86400L * 100};
        Arrays.stream(gaps)
                .forEach(gap -> {
                    BigDecimal score = getBidScore(price, gap);
                    System.out.println(score.toPlainString() + " " + score.doubleValue());
                    Assert.assertTrue(score.compareTo(BigDecimal.ONE) == -1);
                    Assert.assertTrue(score.compareTo(BigDecimal.ZERO) == 1);
                });

        System.out.println("---------------------------------------------------------------");
        Long[] gaps1 = new Long[]{272837L, 291912L};
        Arrays.stream(gaps1)
                .forEach(gap -> {
                    BigDecimal score = getBidScore(price, gap);
                    System.out.println(score.toPlainString() + " " + score.doubleValue());
                    Assert.assertTrue(score.compareTo(BigDecimal.ONE) == -1);
                    Assert.assertTrue(score.compareTo(BigDecimal.ZERO) == 1);
                });
        System.out.println("--------------------------------------------------------------- 3000.000000007392000");
        System.out.println("--------------------------------------------------------------- 2000.000000008230000");

        // 累计分值小于1
        // 持续100天，每10秒刷一次礼物
        BigDecimal total = IntStream.rangeClosed(1, 86400 * 100)
                .parallel()
                .boxed()
                .filter(gap -> gap % 10 == 0)
                .map(Integer::longValue)
                .map(gap -> getBidScore(price, gap))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        System.out.println("持续100天，每10秒刷一次礼物 " + total.toPlainString() + " " + total.doubleValue());
        Assert.assertTrue(total.compareTo(BigDecimal.ONE) == -1);

        // 持续1天，每秒都刷礼物
        total = IntStream.rangeClosed(1, 86400)
                .parallel()
                .boxed()
                .map(Integer::longValue)
                .map(gap -> getBidScore(price, gap))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        System.out.println("持续1天，每秒都刷 Total " + total.toPlainString() + String.format(" %.15f", total.doubleValue()));
        Assert.assertTrue(total.compareTo(BigDecimal.ONE) == -1);

    }

    @Test
    public void getBidScoreDecimal() {
        // 一天毫秒数
        BigDecimal day = new BigDecimal("86400000");
        BigDecimal duration = new BigDecimal("2");
        BigDecimal days = day.multiply(duration);
        System.out.println("持续天数 " + duration + " " + show(days));

        Long now = System.currentTimeMillis();
        Long startTime = now - 100;
        Long endGap = days.longValue();

        System.out.println(show(getBidScoreDecimal(now, startTime, endGap)));

        BigDecimal total = LongStream.rangeClosed(now, startTime + endGap)
                .parallel()
                .filter(time -> time % 200 == 0)
                .mapToObj(time -> getBidScoreDecimal(time, startTime, endGap))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        System.out.println("Total " + show(total));
    }

    @Test
    public void containsDuplicateDataTest() {
        List<Double> list = Arrays.asList(1D, 2D, 3D);
        Assert.assertFalse(containsDuplicateData(list));

        list = Arrays.asList(1.2D, 2.0D, 3.0D);
        Assert.assertFalse(containsDuplicateData(list));

        list = Arrays.asList(1.0D, 1.0D, 3.0D);
        Assert.assertTrue(containsDuplicateData(list));

        String now = System.currentTimeMillis() + "";
        System.out.println(now.length());
    }

    private BigDecimal getBidScore(Long price, Long timeGap) {
        return ReflectionTestUtils.invokeMethod(rtcRoomAuctionProvider, "getBidScore", price, timeGap);
    }

    private BigDecimal getBidScoreDecimal(Long now, Long startTime, Long endGap) {
        return ReflectionTestUtils.invokeMethod(rtcRoomAuctionProvider, "getBidScoreDecimal", now, startTime, endGap);
    }

    private boolean containsDuplicateData(Collection<Double> collection) {
        return ReflectionTestUtils.invokeMethod(rtcRoomAuctionProvider, "containsDuplicateData", collection);
    }

    private String show(BigDecimal decimal) {
        String str = decimal.toPlainString();
        int length = str.length();
        if (StringUtils.contains(str, ".")) {
            length = length - 2;
        }
        return String.format("%s length = %s", str, length);
    }
}